# Library of string manipulation functions
# -*- coding: UTF-8 -*- 

# All input and output arguments are character strings

# =============================== Internal functions ================================

def Ltrim(s=None):
    "returns s where leading spaces, tabs, new-line, etc. are removed"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    return ss.lstrip()

def Rtrim(s=None):
    "returns s where trailing spaces, tabs, new-line, etc. are removed"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    return ss.rstrip()

def Ftrim(s=None):
    "returns s where leading and trailing spaces, tabs, new-line, etc. are removed"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    return Ltrim(Rtrim(ss))

def IsNumeric(s=None):
    "returns '1' if s is a numeric expression else returns '0'"
    if s == None:
        return u'0'
    try:
        num = float(s)
    except:
        return u'0'
    else:
        return u'1'
# =============================== Internal functions ================================


def Capitalize(s=None):
    "returns s where the first character is in upper case and the others in lower case"
    if s == None:
        return u''
    if s == u'':
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    return ss[0].upper() + ss[1:].lower()

def UnFrench(s=None):
    "returns s where all accented characters have been developed in pure ASCII"
    if s == None:
        return u''
    if s == u'':
        return s
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    for f in frenchChar:
        pos = frenchChar.index(f)
        ss = ss.replace(f,unFrenchChar[pos])
    return ss

def French(s=None):
    "returns s where accented characters are restored from pure ASCII"
    if s == None:
        return u''
    if s == u'':
        return s
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    for f in unFrenchChar:
        pos = unFrenchChar.index(f)
        ss = ss.replace(f,frenchChar[pos])
    return ss

def ExtractLeft(s=None,n=None):
    "returns the first n characters of s"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    if n == None:
        return ss
    try:
        nn = int(float(n))
    except:
        return u'' 
    if nn >= len(ss):
        return ss
    return ss[:nn]

def ExtractRight(s=None,n=None):
    "returns the last n characters of s"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    if n == None:
        return ss
    try:
        nn = int(float(n))
    except:
        return u''
    if nn >= len(ss):
        return ss
    else:
        return ss[len(ss)-nn:]


def RemoveLeft(s=None,n=None):
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    if n == None:
        return ss
    try:
        nn = int(float(n))
    except:
        return u''
    nn = max(0,min(len(ss),nn)) # Clipping in [0,len(ss)]
    return ss[nn:]


def RemoveRight(s=None,n=None):
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    if n == None:
        return ss
    try:
        nn = int(float(n))
    except:
        return u''
    nn = max(0,min(len(ss),nn)) # Clipping in [0,len(ss)]
    return ss[:len(ss)-nn]


def InString(s=None,arg=None,pos=None):
    "returns the position of arg in string s, starting from position pos."
    "returns 0 if none"
    if s == None:
        return u'0'
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    if arg == None:
        return '0'
    iarg = arg
    if IsNumeric(arg) == u'1':
        iarg = unicode(arg)
    if pos == None:
        pos = u'1'
    try:
        ipos = int(pos)
    except:
        return u'0'
    if ipos < 1 or ipos > len(s):
        return u'0'
    else:
        res = ss[ipos-1:].find(iarg) + 1
        if res > 0:
            res += (ipos - 1)
        return unicode(res)

def NumToChar(i=None):
    "returns the character with code (i mod 256)"
    "returns Chr(0) if i < 0"
    if i == None:
        return chr(0)
    if not IsNumeric(i) == u'1':
        return chr(0)
    else:
        ii = int(i)
    if ii < 0:
        return chr(0)
    return chr(ii % 256)

def CharToNum(s=None):
    "returns the numeric code of first character of string s"
    "returns 0 if s is empty"
    if s == None:
        return u'0'
    elif s == u'':
        return u'0'
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    return unicode(ord(ss[0]))

def SQLunquote(s=None):
    "returns string s unquoted and with double quotes reduced"
    "s missing = empty string"
    if s == None:
        return u''
    if s == u'':
        return u'0'
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    ss = ss.replace(u"''",u"'")
    if ss[0] == u"'": ss = ss[1:]
    if ss[-1] == u"'": ss = ss[:-1]
    return ss

def SQLquote(s=None):
    "returns string s quoted in SQL style"
    "s missing = empty string"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    ss = ss.replace(u"'",u"''")
    ss = u"'"+ss+u"'"
    return ss

def SQLquote2(s=None):
    "same as SQLquote but without surrounding quotes"
    "s missing = empty string"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    ss = ss.replace(u"'",u"''")
    return ss


def KeyValue(s=None):
    if s == None:
        return u'',u''
    pos = s.find(u'=')
    if pos == -1:
        return s
    return Ftrim(s[:pos]),Ltrim(s[pos+1:])


def ExtractTaggedString(txt,head,tail):
    # returns the fragment of "txt" between "head" and first "tail" (excluded)
    # If "head" is empty, returns the fragment preceding "tail"
    # If "tail" is empty, returns the fragment following "head"
    # If both are empty, returns "txt"
    # If "head" and/or "tail" haven't been found, returns u''
    # If one of the arguments is not a unicode string, returns u''
   
    '''for I in range(0,150):
        f = I*200
        l = f + 200
        print txt[f:l]'''
        
    if IsNumeric(head):
        head = unicode(head)
    if IsNumeric(tail):
        tail = unicode(tail)

    hoffset = len(head)
    toffset = len(tail)

    try:
        pos1 = txt.find(head)
    except Exception, e:
        print u'In "ExtractTaggedString", one of the arguments is not a valid character string'
        return u''
    
    if pos1 == -1:
        extract = u''
    else:
        if toffset == 0:
            extract = txt[pos1+hoffset:]
        else:
            try:
                pos2 = txt.find(tail,pos1+hoffset)
            except  Exception, e:
                print u'In "ExtractTaggedString", one of the arguments is not a valid character string'
                return u''                
            if pos2 == -1:
                extract = u''
            else:
                extract = txt[pos1+hoffset:pos2]

    return extract


def NextSection(txt,pos1,search,head,tail,tags=1):
    # Extract from "txt", starting at position "pos1",
    # the fragment defined by tags "search"-"tail" (included)
    # Internal fragments "head"-"tail" are kepts but skipped.
    # "head" may not be empty.
    # "tags" tells whether enclosing tags are returned. 0 = No, 1 = yes.
    #
    # Examples:
    # nextSection(u'11<a x>22<a>33</a>44</a>55',0,u'<a x>',u'<a>',u'</a>')
    #     => ('<a x>22<a>33</a>44</a>',24)
    # nextSection(u'11<a>22<a>33</a>44</a>55',0,u'<a>',u'<a>',u'</a>')
    #     => ('<a>22<a>33</a>44</a>',22)

    def nextTag(pos):
        # Search "page" from "pos" for the next "head" or "tail" tag
        # case: 0 = no tag found
        #       1 = the closest tag is a "head"
        #       2 = the closest tag is a "tail"
        # t: position of the closest tag or -1

        t1 = txt.find(head,pos)
        t2 = txt.find(tail,pos)

        if t1 == -1:
            # No head 
            if t2 == -1:
                # No head, no tail
                case = 0
                t = -1
            else:
                # No head, a tail
                case = 2
                t = t2
        else:
            # A head tag has been found
            if t2 == -1:
                # A head, no tail
                case = 1
                t = t1
            else:
                # A head, a tail
                if t1 < t2:
                    # The closest tag is a head
                    case = 1
                    t = t1
                else:
                    # The closest tag is a tail
                    case = 2
                    t = t2
        return case,t

    def nextTail(pos):
        # Find the closest "tail" tag, while skipping in-between "head"-"tail" subsections
        stack = 0   # Embedding level
        while True:
            case,npos = nextTag(pos)
            if stack == 0:
                # No subsection in process
                if case == 2:
                    ##print '1. stack,case,pos',stack,case,npos
                    # Next tail found; exit
                    pos = npos
                    break
                elif case == 1:
                    ##print '2. stack,case,pos',stack,case,npos
                    # new "head" found; skip section; loop
                    stack += 1
                    pos = npos + hoffset
                    continue
                else:
                    ##print '3. stack,case,pos',stack,case,npos
                    # No tail found; exit
                    pos = -1
                    break
            else:
                # We are examinig a subsection
                if case == 2:
                    ##print '4. stack,case,pos',stack,case,npos
                    # Next tail found; internal subsection closed; loop
                    pos = npos + toffset
                    stack -= 1
                    continue
                elif case == 1:
                    ##print '5. stack,case,pos',stack,case,npos
                    # new "head" found; skip subsection; loop
                    stack += 1
                    pos = npos + hoffset
                    continue
                else:
                    ##print '6. stack,case,pos',stack,case,npos
                    # No "tail" found; exit
                    pos = -1
                    break
        return pos

    if IsNumeric(search):
        search = unicode(search)
    if IsNumeric(head):
        head = unicode(head)
    if IsNumeric(tail):
        tail = unicode(tail)

    if IsNumeric(pos1):
        pos1 = int(float(pos1))
    else:
        print u'In "nextSection", argument "pos1" is not numeric'
        return u'',-1
    if IsNumeric(tags):
        pos1 = int(float(pos1))
    else:
        print u'In "nextSection", argument "tags" is not numeric'
        return u'',-1
        
    if head == u'':
        return u'',-1
    
    soffset = len(search)
    hoffset = len(head)
    toffset = len(tail)

    if pos1 < 0:
        pos1 = 0
    if pos1 >= len(txt):
        return u'',-1
        
    try:
        pos1 = txt.find(search,pos1)
    except Exception, e:
        print u'In "nextSection", one of the arguments is not a valid character string'
        return u'',-1
    if pos1 == -1:
        # "search" tag not found
        section = u''
        pos = -1
    else:
        # "search" tag found
        pos2 = nextTail(pos1+soffset)
        if pos2 > -1:
            # Closing "tail" tag found
            if tags == 1:
                # Add enclosing tags
                section = txt[pos1:pos2+toffset]
            else:
                # Drop enclosing tags
                section = txt[pos1+soffset:pos2]
            pos = pos2 + toffset
        else:
            # "head" found but no "tail": invalid!
            section = u''
            pos = -1

    return section,pos


# ================ PrettyPrint function ==================== #

# Simple "pretty printer" for "select" SQL queries.
# At least as good as those found on the web!
# May not produce satisfying results for some complex queries
# Returns the input string in case of problem
# Input string must be in unicode

def _ExtractConstants(query):
    # Extract string constants from "query" and create constant dictionary
    global constantDic  # dictionary of string constants
    
    quote = u'\''
    lenQ = len(query)   # length of input query
    reducedQ = []       # character list of the reduced query
    level = 0           # level of current char. 0 = outside a constant; 1 = within a constant
    iChr = 0            # index of current character "C"
    intQ = 0            # integer part of the key of the next string constant
    constant = []       # current string constant
    
    while True:
        if iChr == lenQ:
            break

        C = query[iChr]
        if C <> quote:
            # 'C' is a pure character.
            # Add it to the query or the constant according to the level
            if level == 0:
                reducedQ.append(C)
            else:
                constant.append(C)
            iChr += 1
        else:
            # 'C' is a quote.
            # 3 cases: start of constant, first of double quote, end of constant
            if level == 0:
                # Start of a constant
                constant.append(C)
                iChr += 1
                level = 1
            else:
                # Within a constant, 1 or 2 consecutive quotes?
                constant.append(C)
                iChr += 1
                if query[iChr] == quote:
                    # The next one is also a quote
                    constant.append(query[iChr])
                    iChr += 1
                else:
                    # This quote is alone: end of constant
                    quotedId = u'#'+unicode(intQ)+u'#'
                    constantDic[quotedId] =u''.join(constant)
                    level = 0
                    constant = []
                    reducedQ.append(quotedId)
                    intQ += 1

    return  u''.join(reducedQ)


def _TokenizeQuery(reducedQuery):
    # Tokenize query "reducedQuery" into list "tokens"
    # Splitting character = single space

    prepQuery = reducedQuery
    
    # Add spaces around symbols to prepare "split"
    for sym in [u',',u'(',u')',u'>=','<=',u'=','<>','>','<']:
        prepQuery = prepQuery.replace(sym,u' '+sym+u' ')

    # Reduce long space strings to single space (quick & dirty!)
    prepQuery = prepQuery.replace(u'    ',u' ').replace(u'    ',u' ')\
                .replace(u'   ',u' ')\
                .replace(u'  ',u' ').replace(u'  ',u' ')

    # Oups! Restore combined operators
    prepQuery = prepQuery\
                .replace(u'> =',u'>=')\
                .replace(u'< =',u'<=')\
                .replace(u'= >',u'>=')\
                .replace(u'= <',u'<=')\
                .replace(u'< >',u'<>')\
                .replace(u'> <',u'><')

    # Decompose the query into a list of tokens
    tokens = prepQuery.split(u' ')
    return tokens


def _InsertIndentTags(tokens):
    # Create a query in which indent tags have been inserted.
    # The SQL words depending on a "select" word are assigned the same tagID
    # Each "select" word receives a new tag, that is assigned to all its 
    # dependent words: "from", "where", "group", "order", "having", "union", etc.
    # "select" tag = '\@04@\', the other tags = '@@\\\@04@\'
    # Also performs some nice space reduction and alignment
    global indentDic
    
    level = 1                       # the current level: '(' = +1; ')' = -1
    levels = [1]                    # stack of current tag ID of each level: 
    tagID = 1                       # current tag ID. Assigned monotonically; '(' = +1
    indentDic = {u'\\@01@\\':None}  # dictionary of tag ID. Will map to real indent size
    lenTokens = len(tokens)
    for iT in range(lenTokens):
        token = tokens[iT]
        # print '-- token:',token,' - level:',level,' - lastID:',lastID,' - levels:',levels,' - indent:',indent
        levStr = '%02d' % levels[level-1]
        if token == u'(':
            level += 1
            tagID += 1
            levels.append(tagID)
            indentDic[u'\\@'+('%02d' % tagID)+u'@\\'] = None
        elif token == u')':
            del levels[level-1]
            level -= 1
        elif token.lower() == u'select':
            tokens[iT] = (u'\\@'+levStr+u'@\\')+token+u' '
        elif token.lower() in [u'from',u'where',u'and',u'or',u'group',u'order',u'having',
                               u'union',u'except',u'inter']:
            tokens[iT] = (u'@@\\\\\\@'+levStr+u'@\\')+token+u' '

    # Insert spaces around key words
    tokens2 = [u' '+el+u' ' if (el.lower() in [u'as','in',u'not',u'like',u'between',u'by',
                                               u'distinct',u'asc',u'desc',u'on',u'join',u'left'])
               else el
               for el in tokens]

    # Build query from tokens
    Q5 = u' '.join(tokens2)

    # Insert spaces around operators
    taggedQuery = Q5
    for sym in [u'>=','<=',u'=','<>','>','<']:
        taggedQuery = taggedQuery.replace(sym,u' '+sym+u' ')

    # (once more) restore combined operators
    taggedQuery = taggedQuery.replace(u' >  = ',u'>=').replace(u' <  = ',u'<=').replace(u' <  > ',u'<>')

    # Reduce space strings
    taggedQuery = taggedQuery.replace(u'   ',u' ').replace(u'  ',u' ').replace(u'  ',u' ')
    taggedQuery = taggedQuery.replace(u') )',u'))').replace(u'( (',u'((').rstrip()

    # Align 'select', 'from', 'where', 'and', 'or' key words
    taggedQuery = taggedQuery\
                  .replace(u'@\\from ',u'@\\from   ')\
                  .replace(u'@\\FROM ',u'@\\FROM   ')\
                  .replace(u'@\\where ',u'@\\where  ')\
                  .replace(u'@\\WHERE ',u'@\\WHERE  ')\
                  .replace(u'@\\and ',u'@\\and    ')\
                  .replace(u'@\\AND ',u'@\\AND    ')\
                  .replace(u'@\\or ',u'@\\or     ')\
                  .replace(u'@\\OR ',u'@\\OR     ')

    # Remove space after '(', before ')' and before ','
    taggedQuery = taggedQuery.replace(u'( ',u'(').replace(u' )',u')').replace(u' ,',u',')

    # Remove space between a function name and  '('
    taggedQuery = taggedQuery\
                  .replace(u'sum (',u'sum(')\
                  .replace(u'avg (',u'avg(')\
                  .replace(u'count (',u'count(')\
                  .replace(u'min (',u'min(')\
                  .replace(u'max (',u'max(')\
                  .replace(u'group_concat (',u'group_concat(')

    return taggedQuery

def _InjectConstant(query):
    global constantDic

    injectedQ = query
    for const in constantDic.keys():
        injectedQ = injectedQ.replace(const,constantDic[const])
    return injectedQ

def _CreateMultiLine(taggedQuery):
    # Convert query "" into a list of lines
    # Break the query on main key words (except 'select', which are protected)
    
    multiLineQuery = taggedQuery.split(u'@@\\\\')
    
    return multiLineQuery

def _CompleteIndentDictionary(multiLineQuery):
    # Assign a number of spaces to each tagID of "indentDic" dictionary
    global indentDic

    for line in multiLineQuery:
        ##print '\n   -- line:',line
        pos = 0
        inst = 0        # number of tags found in this line
        initIndent = 0  # indent of the current line
        while True:
            ##print '   -- inst:',inst
            pos = line.find(u'@\\',pos+1)
            ##print '   -- pos:',pos
            if pos == -1:
                ##print '   -- no more tag:'
                break
            ##print '   -- tag found at:',pos
            num = line[pos-2:pos]
            levKey = u'\\@'+num+u'@\\'
            ##print '   -- num:',num,'- levKey:',levKey
            if line[pos+2:].lower().startswith(u'select'):
                # "select" tag found
                ##print '   -- select tag found at:',pos
                if indentDic[levKey] is None:
                    indentDic[levKey] = pos-4 - inst*6 + initIndent
                    ##print '   -- indentDic[levKey]:',indentDic[levKey]
            else:
                # non-"select" tag found.
                # If first one in the line, record its offset
                if initIndent == 0:
                    initIndent = indentDic[levKey]
                
            inst += 1

    return

def _PrettyQuery(multiLineQuery,linebreak):
    # Assembles the lines of "multiLineQuery" into the final query
    import sysLib
    
    indentedQ = []      # list of lines of the "pretty" query
    for line in multiLineQuery:
        fline = line
        for key in indentDic.keys():
            indentKey = indentDic[key]
            if indentKey is None:
                # Skip unmapped tagID
                continue

            pos = fline.find(key)
            if fline[pos-1] == u'(' and fline[pos+6:].lower().startswith(u'select'):
                # No insertion if "select" after a parenthesis
                fline = fline.replace(key,u'')
            else:
                # Others: insert the space string
                fline = fline.replace(key,indentKey*u' ')

        nspos = sysLib.skipSpaces(fline,0)
        if fline[nspos:nspos+6].lower() in (u'union ',u'inter ',u'except'):
            # Special cases: lines with set operator + 'select' are split into two
            spos = fline.find(u' ',nspos)
            indentedQ.append(u'  '+fline[:spos])
            indentedQ.append(fline[spos+1:])
        else:
            indentedQ.append(fline)

    return linebreak.join(indentedQ)


def PrettyPrint(rawQuery,linebreak = u'eol'):
    global constantDic  # dictionary of string constants

    if not isinstance(rawQuery,unicode):
        return rawQuery
    
    if rawQuery.strip() == u'':
        return rawQuery

    if not rawQuery.lstrip().lower().startswith(u'select'):
        # Unrecognized syntax
        return u'[Unrecognized syntax in "SQLprettyPrint"] '+rawQuery

    # Remove source linebreaks and tabs
    rawQuery = rawQuery.replace(u'\n',u' ').replace(u'\t',u' ').strip()
    
    # Normalize linebreak. 'eol' --> '\n'. Otherwise, can be anything else.
    if linebreak == u'eol':
        linebreak = u'\n'
    
    rawQuery += u' '    # sentinel. Will disappear

    try:
        reducedQuery   = _ExtractConstants(rawQuery)
        tokens         = _TokenizeQuery(reducedQuery)
        taggedQuery    = _InsertIndentTags(tokens)
        injectedQuery  = _InjectConstant(taggedQuery)
        multiLineQuery = _CreateMultiLine(injectedQuery)
        _CompleteIndentDictionary(multiLineQuery)
        prettyQuery    = _PrettyQuery(multiLineQuery,linebreak)
    except:
        # Unrecognized syntax
        return u'[Unrecognized syntax in "SQLprettyPrint"] '+rawQuery

    return prettyQuery


# ================ Cryptographic functions ==================== #

# Hashing
# -------

def hash(S,form):
    # Returns the hash value of string S as a string of
    #   - 64 hexa characters    if "form" = 1
    #   - 44 base64 characters  if  "form" = 2
    # S can be numeric. Hashes '' if S is empty
    # Uses the sha256 algorithm

    from SQLfastUDFlib import SQLisNumeric
    import hashlib
    import base64

    # Check "form"
    if form not in ('1','2'):
        form = 1
    else:
        form = int(form)

    # If needed, convert "S" into char string
    try:
        if SQLisNumeric(S):
            S = unicode(S)
    except Exception,e:
        print u'Error in LStr:hash.S.',e
        return ''

    # S must be a byte string
    if type(S) == unicode:
        S = S.encode('latin-1')

    res = hashlib.sha256(S)
    if form == 1:
        res = res.hexdigest()
    else:
        try:
            res = base64.b64encode(res.digest(),'_-')
        except Exception,e:
            print u'Error in LStr:hash.res.',e
            return ''

    return res

def miningHash(S,prefix,hashNbr,form):
    # PoW mining version of hash.
    # Returns "res", the first hash value (sha256) of string "S" the binary value
    #  of which starts with the '0'-bitstring specified by "prefix"
    # ('00000' = starts with at least 5 zero bits).
    # At most "hashNbr" are computed. 
    # Also returns "nonce", the nonce that produces this hash
    #              "N", the number of hashes that have been calculated.
    # 
    # "res" is a string of
    #   - 64 hexa characters    if "form" = 1
    #   - 44 base64 characters  if  "form" = 2

    from SQLfastUDFlib import SQLisNumeric
    import hashlib
    import base64

    # Check "S"
    if S is None or S == '':
        return '','',0

    try:
        if SQLisNumeric(S):
            S = unicode(S)
    except Exception,e:
        print u'Error in LStr:miningHash.S.',e
        return '','',0

    # S must be a byte string
    if type(S) == unicode:
        S = S.encode('latin-1')

    # Check "prefix"
    if prefix is None or prefix == '':
        return '','',0

    try:
        if SQLisNumeric(prefix):
            prefix = unicode(prefix)
    except Exception,e:
        print u'Error in LStr:miningHash.prefix.',e
        return '','',0

    if len(prefix) == 0:
        print u'Error in LStr:miningHash.prefix.','Cannot be empty.'
        return '','',0
    if len(prefix) > 64:
        print u'Warning in LStr:miningHash.prefix.','Prefix too long; shortened to 64 char.'
        prefix = prefix[:64]

    # Check "hashNbr"
    if hashNbr is None or hashNbr == '':
        return '','',0

    if not SQLisNumeric(hashNbr):
        print u'Error in LStr:miningHash.hashNbr.','"hashNbr" must be a numeric value.'
        return '','',0
    else:
        hashNbr = int(float(hashNbr))

    if hashNbr < 1:
        print u'Error in LStr:miningHash.hashNbr.','"hashNbr" must be a positive number.'
        return '','',0

    if hashNbr > 100*1000*1000:
        print u'Warning in LStr:miningHash.hashNbr.','"hashNbr" too large. Reduced to 100,000,000.'
        hashNbr = 100*1000*1000

    # Check "form"
    if form not in ('1','2'):
        form = 1
    else:
        form = int(form)

    # Search for a legal hash
    lenPrefix = len(prefix)
    nonce = ''
    for N in range(hashNbr):
        extS = S.encode('latin-1') + str(N)
        #print '   extS: ',extS
        Bhash = hashlib.sha256(extS)
        #print '   Bhash:',Bhash.digest()
        Chash = ''.join('{0:08b}'.format(ord(x), 'b') for x in Bhash.digest()[:lenPrefix//8+1])[:lenPrefix]
        #print '   Chash:',Chash
        if Chash == prefix:
            nonce = str(N)
            #print '   nonce:',nonce
            break

    if nonce == '':
        # No legal hash has been found. Augment the number of trials
        return '','',N
    
    if form == 1:
        res = Bhash.hexdigest()
        #print '      - res.hexa():',res
    else:
        res = base64.b64encode(Bhash.digest(),'_-')
        #print '      - res.b64():',res

    return res,nonce,N


# Symmetric encryption
# --------------------

def generateSYMkey(n):
    # Generate a random key of "n" bytes converted in base64
    from SQLfastUDFlib import SQLrandomblob

    try:
        vn = float(n)
        vn = SQLint(vn)
    except Exception,e:
        vn = 1

    symKey = SQLrandomblob(n,2)

    return symKey


def encryptSYM(message,symKey):
    # Encrypts byte string "message" with key "symkey" and converts it in base64.
    from SQLfastUDFlib import SQLencrypt
    crypto = SQLencrypt(message,symKey)
    return crypto


def decryptSYM(crypto,symKey):
    # decrypts base64 character string "crypto" with key "symkey" and returns it as a byte string
    from SQLfastUDFlib import SQLdecrypt
    message = SQLdecrypt(crypto,symKey)
    return message


# RAS encryption
# --------------

def generateRSAkeys(n):
    # Generate secret and public keys, converted in base64
    # The public key is "n" bit long (default = 512)
    import rsa
    from SQLfastUDFlib import SQLint
    #print 'generateRSAkeys:n:',n
    try:
        vn = float(n)
        vn = SQLint(vn)
    except Exception,e:
        print 'LStr:generateRSAkeys.vn error:',e
        vn = 512

    # Generate secret and public keys (= usually (public,private))
    (secret,public) = rsa.newkeys(vn,accurate=False)

    # Serialize these keys
    secret_serial = secret.save_pkcs1(format='PEM')
    public_serial = public.save_pkcs1(format='PEM')

    # Condense keys
    secret_serial = secret_serial.replace(eol,'').replace(headerSecret,'').replace(footerSecret,'')
    public_serial = public_serial.replace(eol,'').replace(headerPublic,'').replace(footerPublic,'')

    return secret_serial, public_serial


def encryptRSA(mess,key):
    # encrypt message "mess" with secret key "key"
    # and convert it in base64
    import rsa
    from SQLfastUDFlib import SQLb64encode
   
    #print '\nencryptRSA:mess =',mess
    
    # Useless: # Convert '@n' into eol
    ##print '\nencryptRSA:key 1 =',key
    # Useless: key = key.replace('@n',eol)
    ##print '\nencryptRSA:key 2 =',key

    # Restore full PEM format
    #print '\nencryptRSA:key1 =',key
    key = headerSecret + eol + key + eol + footerSecret
    #print '\nencryptRSA:key2 =',key
    
    # Convert the PEM format into a secret key object
    try:
        secretKey = rsa.PublicKey.load_pkcs1(key,format='PEM')
    except Exception,e:
        print '\nencryptRAS:load_pkcs1 error:',e
    #print '\nencryptRSA:secretKey =',secretKey
    
    # Encrypt the message with the secret key
    try:
        crypto = rsa.encrypt(mess.encode('utf8'),secretKey)
    except Exception,e:
        print '\nencryptRAS:encrypt error:',e
    #print '\nencryptRSA:crypto =',crypto.encode('hex')
    
    # Convert the encrypted message in base64
    crypto = SQLb64encode(crypto,'url')
    #print '\nencryptRSA:crypto =',crypto
    
    return crypto

def decryptRSA(crypto,key):
    # decrypt encrypted message "crypto" with public key "key"
    import rsa
    from SQLfastUDFlib import SQLb64decode
    
    # Convert '@n' into eol
    # Useless: key = key.replace('@n',eol)
    
    # Restore full PEM format
    key = headerPublic + eol + key + eol + footerPublic

    # Convert the PEM format into a public key object
    publicKey = rsa.PrivateKey.load_pkcs1(key,format='PEM')

    # Restore the message in byte string from its base64 format
    crypto = SQLb64decode(crypto).encode('latin-1')
    
    # Decrypt the message with the public key
    try:
        mess = rsa.decrypt(crypto,publicKey)
    except Exception,e:
        print '\nLStr:decryptRSA error',e

    return mess


# General constants
    
eol = '\n'

# Cryptographic constants
    
headerSecret = '-----BEGIN RSA PUBLIC KEY-----'
footerSecret = '-----END RSA PUBLIC KEY-----'
headerPublic = '-----BEGIN RSA PRIVATE KEY-----'
footerPublic = '-----END RSA PRIVATE KEY-----'


# Initialize character maps
# -------------------------

frenchChar = [u'ä',u'á',u'à',u'â',\
              u'ç',\
              u'ë',u'é',u'è',u'ê',\
              u'ï',u'í',u'ì',u'î',\
              u'ö',u'ó',u'ò',u'ô',\
              u'ü',u'ú',u'ù',u'û',\
              u'Ä',u'Á',u'À',u'Â',\
              u'Ç',\
              u'Ë',u'É',u'È',u'Ê',\
              u'Ï',u'Í',u'Ì',u'Î',\
              u'Ö',u'Ó',u'Ò',u'Ô',\
              u'Ü',u'Ú',u'Ù',u'Û']

unFrenchChar = [u'a```',u'a``',u'a`',u'a^',\
                u'c`',\
                u'e```',u'e``',u'e`',u'e^',\
                u'i```',u'i``',u'i`',u'i^',\
                u'o```',u'o``',u'o`',u'o^',\
                u'u```',u'u``',u'u`',u'u^',\
                u'A```',u'A``',u'A`',u'A^',\
                u'C`',\
                u'E```',u'E``',u'E`',u'E^',\
                u'I```',u'I``',u'I`',u'I^',\
                u'O```',u'O``',u'O`',u'O^',\
                u'U```',u'U``',u'U`',u'U^']

# Globals for "PrettyPrint"
constantDic = {}    # dictionary of string constants
indentDic   = {}    # dictionary of indent tags
