
### -*- coding: UTF-8 -*-

# Temporal operators
# Last updated 2019-04-19

import sqlite3
from sysLib import *

debug = False


def getIntervalNames(cur,table):
    # Returns the names of the last two columns of table "table"
    # They are supposed to denote "start" and "end" columns.
    startName,endName = u'',u''
    try:
        cur.execute(u"select * from "+table+u" where 1 = 2")
        rowDesc = cur.description
        startName = rowDesc[-2][0]
        endName   = rowDesc[-1][0]
    except Exception, e:
        print '"LTemp:normalize.getIntervalNames", error in getting "start" and "end" names of "'+table+'":',e
        
    if debug: print '\n"LTemp:normalize": "startName" and "endName" columns in '+table+' =',startName,endName    
    return startName,endName


def queryDecomposition(Q):
    # Decompose query "Q" (no table alias):
    # Model: select <columns> from <source> where <condition> group by <groupBy> into <target>;

    if Q.strip() == u'':
        return '','','','',''

    # From left: skip 'select'
    if head(Q) == u'select':
        Q1 = tail(Q)
    else:
        return '','','','',''

    # Q1 = <columns> from <source> where <condition> group by <groupBy> into <target>;
    #print 'Q1:',Q1
    
    # From right: extract and skip 'into <target>'
    if reverseHead(reverseTail(Q1)).strip() == 'into':
        target = reverseHead(Q1)
        pos = Q1.rfind(u' into ')
        Q1 = Q1[:pos].strip()
    else:
        target = u''

    # Q1 = <columns> from <source> where <condition> group by <groupBy>;
    # target = <target>
    #print 'Q1:',Q1

    # From left: skip 'from' and extract 'columns'
    pos = Q1.find(u' from ')
    if pos == -1:
        return '','','','',''

    columns = Q1[:pos].strip()
    Q2 = Q1[pos+6:]

    # Q2 = <source> where <condition> group by <groupBy>;
    # target  = <target>
    # columns = <columns>
    #print 'Q2:',Q2

    # extract 'source'
    source = head(Q2)
    if source == u'':
        return '','','','',''
    Q3 = tail(Q2)

    # Q3 = where <condition>;
    # Q3 = where <condition> group by <groupBy>;
    # Q3 = group by <groupBy>;
    # target  = <target>
    # columns = <columns>
    # source  = <source>
    #print 'Q3:',Q3

    # Extract 'condition', and 'grouBy', if any
    if Q3 == u'':
        condition = u''
        groupBy = u''
    else:
        if head(Q3) == u'where':
            Q4 = tail(Q3).rstrip()
            # Q4 = <condition> group by <groupBy>;
            # Q4 = <condition>;
            #print 'Q4:',Q4
            pos = Q4.rfind(u' group ')
            if pos > -1:
                # Q4 = <condition> group by <groupBy>;
                #print 'Q4:',Q4
                condition = Q4[:pos].rstrip()
                Q5 = Q4[pos+6:]
                # Q5 = by <groupBy>
                #print 'Q5:',Q5
                if head(Q5) == u'by':
                    groupBy = tail(Q5)
                else:
                    groupBy = u''
            else:
                # Q4 = <condition>;
                #print 'Q4:',Q4
                condition = Q4
                groupBy = u''
        elif head(Q3) == u'group':
            condition = u''
            Q4 = tail(Q3).rstrip()
            # Q4 = by <groupBy>
            #print 'Q4:',Q4
            if head(Q4) == u'by':
                groupBy = tail(Q4)
            else:
                return '','','','',''                
        else:
            return '','','','',''

    return source,condition,columns,groupBy,target


def queryDecompositionE(Q):
    # Decompose query "Q" (includes table alias):
    # Model: select <columns> from <source> <alias> where <condition> group by <groupBy> into <target>;

    if Q.strip() == u'':
        return '','','','','',''

    # From left: skip 'select'
    if head(Q) == u'select':
        Q1 = tail(Q)
    else:
        return '','','','','',''

    # Q1 = <columns> from <source> where <condition> group by <groupBy> into <target>;
    #print 'Q1:',Q1
    
    # From right: extract and skip 'into <target>'
    if reverseHead(reverseTail(Q1)).strip() == 'into':
        target = reverseHead(Q1)
        pos = Q1.rfind(u' into ')
        Q1 = Q1[:pos].strip()
    else:
        target = u''

    # Q1 = <columns> from <source> where <condition> group by <groupBy>;
    # target = <target>
    #print 'Q1:',Q1

    # From left: skip 'from' and extract 'columns'
    pos = Q1.find(u' from ')
    if pos == -1:
        return '','','','','',''

    columns = Q1[:pos].strip()
    Q2 = Q1[pos+6:]

    # Q2 = <source> where <condition> group by <groupBy>;
    # target  = <target>
    # columns = <columns>
    #print 'Q2:',Q2

    # extract 'source'
    source = head(Q2)
    if source == u'':
        return '','','','','',''
    Q3 = tail(Q2)
    alias = head(Q3)
    Q3 = tail(Q3)

    # Q3 = where <condition>;
    # Q3 = where <condition> group by <groupBy>;
    # Q3 = group by <groupBy>;
    # target  = <target>
    # columns = <columns>
    # source  = <source>
    # alias  = <alias>
    #print 'Q3:',Q3

    # Extract 'condition', and 'grouBy', if any
    if Q3 == u'':
        condition = u''
        groupBy = u''
    else:
        if head(Q3) == u'where':
            Q4 = tail(Q3).rstrip()
            # Q4 = <condition> group by <groupBy>;
            # Q4 = <condition>;
            #print 'Q4:',Q4
            pos = Q4.rfind(u' group ')
            if pos > -1:
                # Q4 = <condition> group by <groupBy>;
                #print 'Q4:',Q4
                condition = Q4[:pos].rstrip()
                Q5 = Q4[pos+6:]
                # Q5 = by <groupBy>
                #print 'Q5:',Q5
                if head(Q5) == u'by':
                    groupBy = tail(Q5)
                else:
                    groupBy = u''
            else:
                # Q4 = <condition>;
                #print 'Q4:',Q4
                condition = Q4
                groupBy = u''
        elif head(Q3) == u'group':
            condition = u''
            Q4 = tail(Q3).rstrip()
            # Q4 = by <groupBy>
            #print 'Q4:',Q4
            if head(Q4) == u'by':
                groupBy = tail(Q4)
            else:
                return '','','','','',''                
        else:
            return '','','','','',''

    return source,alias,condition,columns,groupBy,target


def project(db,tQuery):
    # General PROJECTION procedure
    # Project table "source" on columns "columns" in database through connection "db"
    # and store the result in table "target" (need not preexist)
    # Columns "start" and "end" must be the last ones in "source" and "target"
    # If "target" does nor exist, it is created
    # If the schema of "target" is invalid, a new "target" table is created
    #
    # Examples:
    # - function status = LTemp:project {select CITY,PROJECT
    #                                    from   H_EMPLOYEE
    #                                    where  SALARY > 6000
    #                                    into   H_CITY_PROJECT};
    #   The "where" clause is optional
    # status:
    #  0: OK
    #  1: database not opened
    #  2: query syntax error
    #  3: unable to extract source data
    #  4: unable to insert result in target table

    """
    # PRINCIPLE:
    cur1.execute("drop table if exists H_CITY_PROJECT")
    cur1.execute("create temp table H_CITY_PROJECT as select CITY,PROJECT,start,end from H_EMPLOYEE where 1 = 2")

    cur1.execute("select CITY,PROJECT,start,end from H_EMPLOYEE order by CITY,PROJECT,start")
    first = True
    for row in cur1:
        valRow = row[:-2]
        (start,end) = row[-2:]
        if first:
            curValRow,(curStart,curEnd) = valRow,(start,end)
            first = False
        else:
            if valRow == curValRow and start <= curEnd:
                if end > curEnd:
                    curEnd = end
            else:
                cur2.execute("insert into H_CITY_PROJECT values (?,?,?,?,)",
                             (curValRow + (curStart,curEnd)))
                curValRow,(curStart,curEnd) = valRow,(start,end)
    if not first:
        cur2.execute("insert into H_CITY_PROJECT values (?,?,?,?,)",
                     (curValRow + (curStart,curEnd)))
    """

    if debug:
        print '\n\n-----------------------------------------'
        print     '--------- Enter LTemp:project   ---------'
        print     '-----------------------------------------'
    
    status = 0

    try:
        cur1 = db.cursor()
        cur2 = db.cursor()
    except:
        print 'Error in cursor extraction',e
        status = 1
        return status

    # ============================================================================
    # 1. GET BASE PARAMETERS
    # ============================================================================

    # 1.1 Extract query components
    # ----------------------------

    source,condition,columns,groupBy,target = queryDecomposition(tQuery)
    
    if debug: print '\nIn "project":'
    if debug: print '   source   :',source
    if debug: print '   condition:',condition
    if debug: print '   columns  :',columns
    if debug: print '   groupBy  :',groupBy
    if debug: print '   target   :',target

    if source == u'' or columns == u'' or target == u'':
        status = 2
        return status


    # 1.2 Get the names of "start" and "end" columns
    # ----------------------------------------------

    startName,endName = getIntervalNames(cur1,source)

    if startName == u'':
        print '"LTemp:project", error in getting "start" and "end" names of '+source
        status = 3
        return status
    
    allColumns = columns+u","+startName+","+endName

    # ============================================================================
    # 2. CREATE TABLE  "target"
    # ============================================================================

    try:
        cur1.execute(u"select 1 from " + target + u" limit 1")
    except Exception,e:
        if debug: print u"Table "+target+" missing. Will be created" 
        try:
            cur1.execute(u"create temp table " + target + u" as select " + allColumns + u" from " + source + u" where 1 = 2")
        except Exception,e:
            print 'Error in "target" table creation',e
            status = 4
            return status


    # ============================================================================
    # 3. COMPUTE PROJECTION
    # ============================================================================

    # 3.1 Build main queries "Qsource" and "Qinsert"
    # ----------------------------------------------

    # Scan source table
    if condition == u'':
        Qsource = u"select "+ allColumns +u" from "+source\
                 +u" order by "+columns+u","+startName
    else:
        Qsource = u"select "+columns+u",start,end from "+source\
                 +u" where "+condition\
                 +u" order by "+columns+u","+startName

    # insert into "target"
    formCols = ["?" for c in allColumns.split(u',')]
    Qinsert = u"insert into "+target+u" values ("+','.join(formCols)+u")"

    if debug: print '-- Qsource:',Qsource
    if debug: print '-- Qinsert:',Qinsert

    # 3.2 Prepare the parsing of "source" data
    # ---------------------------------------

    if debug: print '\nparsing "source" table '+source
    try:
        cur1.execute(Qsource)
    except Exception, e:
        print '"normalize": error in "Qsource"',e
        print Qsource
        status = 3
        return status

    # 3.3 Parse "source" data
    # -----------------------
    first = True    # First or next row extracted?
    for row in cur1:
        valRow = row[:-2]
        (start,end) = row[-2:]
        if first:
            curValRow,(curStart,curEnd) = valRow,(start,end)        # curRow <- row
            first = False
        else:
            # if curRow and new row have same values and new interval overlaps or touches current interval
            if valRow == curValRow and start <= curEnd:
                if end > curEnd:            # if new end exceeds the current one.
                    curEnd = end                # extends current interval
            else:
                try:                    # different new values; record curRow ans start a new curRow
                    cur2.execute(Qinsert,(curValRow + (curStart,curEnd)))   # insert curRow
                except Exception, e:
                    status = 4
                    return status
                curValRow,(curStart,curEnd) = valRow,(start,end)    # curRow <- row
    if not first:
        cur2.execute(Qinsert,(curValRow + (curStart,curEnd)))       # insert curRow


    # ============================================================================
    # 4. CLOSE PROCESS
    # ============================================================================

    cur2.close()
    cur1.close()
    
    db.commit()

    
    if debug:
        print '\n-----------------------------------------'
        print   '---------- EXIT LTemp:project   ---------'
        print   '-----------------------------------------'

    return status


def reduce(db,tQuery):
    # General COALESCE procedure.
    # Same as "project" but:
    # - no condition
    # - no target clause
    # - modify the source table itself
    # - "source" must be a real table (not a view)
    # Examples:
    # - function status = LTemp:reduce {select CITY,PROJECT
    #                                   from   H_EMP};
    # status:
    #  0: OK
    #  1: database not opened
    #  2: query syntax error
    #  3: unable to extract source data
    #  4: unable to insert result into source table (various issues)
   
    if debug:
        print '\n\n-----------------------------------------'
        print     '--------- Enter LTemp:reduce    ---------'
        print     '-----------------------------------------'

    status = 0
    
    try:
        cur1 = db.cursor()
        cur2 = db.cursor()
    except:
        print '"reduce": Error in cursor extraction',e
        status = 1
        return status


    # ============================================================================
    # 1. GET BASE PARAMETERS
    # ============================================================================

    # 1.1 Extract query components
    # ----------------------------

    source,condition,columns,groupBy,target = queryDecomposition(tQuery)

    if debug: print '\nIn "reduce":'
    if debug: print '   source   :',source
    if debug: print '   condition:',condition
    if debug: print '   columns  :',columns
    if debug: print '   groupBy  :',groupBy
    if debug: print '   target   :',target
    
    if source == u'' or condition != u'' or target != u'':
        status = 2
        return status

    # Does "source" table exist?
    try:
        cur1.execute(u"select "+columns+",start,end from " + source + u" limit 1")
        # "source" table exists: OK
    except Exception,e:
        # Either "source" table is missing or it has an invalid structure: abort
        print '"reduce": Source table missing or invalid.',e
        status = 3
        return status


    # 1.2 Get the names of "start" and "end" columns
    # ----------------------------------------------

    startName,endName = getIntervalNames(cur1,source)

    if startName == u'':
        print '"LTemp:reduce", error in getting "start" and "end" names of '+source
        status = 3
        return status
    
    allColumns = columns+u","+startName+","+endName



    # ============================================================================
    # 2. CREATE TABLE WORK
    # ============================================================================

    Qcreate  = u"create temp table WORK as select * from "+source+u" where 1 = 2"
    
    if debug: print '-- Qcreate:',Qcreate

    try:
        cur1.execute(Qcreate)
    except Exception, e:
        print '"reduce": error in "Qcreate"',e
        print Qcreate
        status = 4
        return status


    # ============================================================================
    # 3. COMPUTE REDUCTION
    # ============================================================================

    # 3.1 Build main queries
    # ----------------------

    # Scan source table
    Qsource  = u"select "+allColumns+u" from "+source\
              +u" order by "+columns+u","+startName
    
    # insert into WORK
    formCols = ["?" for c in allColumns.split(u',')]
    Qinsert  = u"insert into WORK values ("+','.join(formCols)+u")"

    # empty table "source"
    Qdelete  = u"delete from "+source

    # copy WORK to "source"
    Qmove    = u"insert into "+source+u" select * from WORK"

    if debug: print '-- Qsource:',Qsource
    if debug: print '-- Qinsert:',Qinsert
    if debug: print '-- Qdelete:',Qdelete
    if debug: print '-- Qmove:',Qmove


    # 3.2 Prepare the parsing of "source" data
    # ---------------------------------------
    
    try:
        cur1.execute(Qsource)
    except Exception, e:
        print '"reduce": error in "Qsource"',e
        print Qsource
        status = 3
        return status


    # 3.3 Parse "source" data
    # -----------------------
    first = True
    for row in cur1:
        valRow = row[:-2]
        (start,end) = row[-2:]
        if first:
            curValRow,(curStart,curEnd) = valRow,(start,end)        # curRow <- row
            first = False
        else:
            # if curRow and new row have same values and new interval overlaps or touches current interval
            if valRow == curValRow and start <= curEnd:
                if end > curEnd:            # if new end exceeds the current one.
                    curEnd = end                # extends current interval
            else:
                try:                    # different new values; record curRow ans start a new curRow
                    cur2.execute(Qinsert,(curValRow + (curStart,curEnd)))   # insert curRow
                    if debug: print 'Inserting:',(curValRow + (curStart,curEnd))
                except Exception, e:
                    print '"reduce": error in "Qinsert"',e
                    status = 4
                    return status
                curValRow,(curStart,curEnd) = valRow,(start,end)    # curRow <- row
    if not first:
        cur2.execute(Qinsert,(curValRow + (curStart,curEnd)))       # insert curRow
        if debug: print 'Inserting:',(curValRow + (curStart,curEnd))

    # Empty source table
    try:
        cur1.execute(Qdelete)
    except Exception, e:
        print '"reduce": error in "Qdelete"',e
        status = 4
        return status

    # Copy WORK rows to source table
    try:
        cur1.execute(Qmove)
    except Exception, e:
        print '"reduce": error in "Qmove"',e
        status = 4
        return status


    # ============================================================================
    # 4. CLOSE PROCESS
    # ============================================================================

    # Drop WORK table
    try:
        cur1.execute("drop table WORK")
    except Exception, e:
        print '"reduce": error in "drop"',e
        status = 4
        return status

    cur2.close()
    cur1.close()
    
    db.commit()
    
    if debug:
        print '\n-----------------------------------------'
        print   '---------- EXIT LTemp:reduce    ---------'
        print   '-----------------------------------------'
    
    return status


#def normalize(db,tQuery):
def normalize(db,source,compare,Scolumns,Ccolumns,target=u''):
    # Normalization procedure
    # Insert missing intervals in a temporal table <source>
    # Argument: "update <source> [wrt <compare>] on <Scolumns> [= <Ccolumns>] [into <target>]"
    
    # PRINCIPLES
    # ----------
    #
    # 1. GET BASE PARAMETERS
    # ----------------------
    # source,compare,Scolumns,Ccolumns,target = queryDecomposition(tQuery)
    # SstartName,SendName = getIntervalNames of "source"
    # CstartName,CendName = getIntervalNames of "compare"
    # Scount, Ccount = size of "source" and "compare" tables
    # 
    # 2. CREATE TABLES
    # ----------------
    # if "target" specified and missing:
    #     Create "target" table (= empty "source")
    # Create table MISSING      (= empty "source")
    # 
    # 3. "NO CHANGE" CASES
    # --------------------
    # Return if (Ccount == 0 and Scount == 0)
    # 
    # 4. EXTRACT INTERNAL MISSING INTERVALS
    # -------------------------------------
    # If "source" not empty:
    #     extract its internal missing intervals
    # 
    # 5. EXTRACT COMPARE-BASED MISSING INTERVALS
    # ------------------------------------------
    # If "compare" exists, is not empty and "source" is not empty:
    #     extract leading missing intervals
    #     extract trailing missing intervals
    # If "compare" exists, is not empty and "source" is empty:
    #     extract life intervals for all values of "Ccolumns" in "compare"
    # If "compare" exists, is not empty and "source" is not empty:
    #     extract life intervals for all values of "Ccolumns" in "compare"
    #     without match in "source"
    # 
    # 6. COPY MISSING INTERVALS INTO "target"
    # ---------------------------------------
    # if "target" exists:
    #    store "source" into "target"
    #    store MISSING into "target"
    # else:
    #    store MISSING into "source"
    # endif
    # 
    # 7. CLOSE PROCESS
    # ----------------
    # drop table MISSING
    # commit


    # CASES:
    # no "compare"       and "source" empty     : "source" unchanged
    # no "compare"       and "source" not empty : internal intervals inserted into "source"
    # "compare" empty    and "source" empty     : "source" unchanged
    # "compare" empty    and "source" not empty : internal intervals inserted into "source"
    # "compare not empty and "source" empty     : life intervals of "compare" inserted into "source"
    # "compare not empty and "source" not empty : internal & external intervals inserted into "source"

    #debug = True

    if debug:
        print '\n\n-----------------------------------------'
        print     '--------- Enter LTemp:normalize ---------'
        print     '-----------------------------------------'
    
    status = 0


    if compare != u'' and Ccolumns == u'':
        Ccolumns = Scolumns
        
    if source == u'' or Scolumns == u'':
        print '"normalize": syntax error'
        status = 2
        return status

    try:
        cur1 = db.cursor()
        cur2 = db.cursor()
    except:
        print '"normalize": Error in cursor extraction',e
        status = 1
        return status


    # ============================================================================
    # 1. GET BASE PARAMETERS
    # ============================================================================

    # 1.1 Extract query components
    # ----------------------------

    # source,compare,Scolumns,Ccolumns,target = queryDecomposition(tQuery)

    if debug:
        if compare != u'':
            print '\nfunction Ltemp:normalize {update '+source+' wrt '+compare+' on ('+Scolumns+') = ('+Ccolumns+')}'
        else:
            print '\nfunction Ltemp:normalize {update '+source+' on '+Scolumns+'}'
    if debug: print '\n   source  :',source
    if debug: print   '   compare :',compare
    if debug: print   '  Scolumns :',Scolumns
    if debug: print   '  Ccolumns :',Ccolumns
    if debug: print   '    target :',target

    listScolumns = Scolumns.split(u',')
    lenSColumns = len(listScolumns)

    # 1.2 Get the names of "start" and "end" columns
    # ----------------------------------------------

    SstartName,SendName = getIntervalNames(cur1,source)
    CstartName,CendName = getIntervalNames(cur1,compare)

    if SstartName == u'' or CstartName == u'':
        print '"LTemp:normalize", error in getting "start" and "end" names of tables'
        status = 3
        return status
    
    allColumns = Scolumns+u","+SstartName+","+SendName

    # 1.3 Extract size of "source" and "compare" tables
    # -------------------------------------------------

    Scount,Ccount = 0,0
       
    try:
        cur1.execute(u"select count(*) from "+source)
        Scount = cur1.fetchone()[0]
    except Exception, e:
        print '"normalize": error in getting statistics of "source":',e
        status = 3
        return status
    if debug: print 'Statistics in "source" '+source+':',Scount
    
    if compare != u'':
        try:
            cur1.execute(u"select count(*) from "+compare)
            Ccount = cur1.fetchone()[0]
        except Exception, e:
            print '"normalize": error in getting statistics of "compare":',e
            status = 3
            return status
        if debug: print 'Statistics in "compare" '+compare+':',Ccount
    

    # ============================================================================
    # 2. CREATE TABLES
    # ============================================================================

    # 2.1 Create table "target" if required
    # -------------------------------------

    if target != u'':
        try:
            cur1.execute(u"select 1 from " + target + u" limit 1")
        except Exception,e:
            if debug: print u"Table "+target+" missing. Will be created" 
            try:
                cur1.execute(u"create temp table " + target + u" as select * from " + source + u" where 1 = 2")
            except Exception,e:
                print 'Error in "target" table creation',e
                status = 4
                return status


    # 2.2 Create table MISSING
    # ------------------------
    
    Qcreate  = u"create temp table MISSING as select * from "+source+u" where 1 = 2"
    if debug: print '\nTable MISSING for '+source+' created.'
    try:
        cur1.execute(Qcreate)
    except Exception,e:
        print '"normalize": error in creating MISSING table:',e
        status = 2
        return status


    # ============================================================================
    # 3. "NO CHANGE" CASES
    # ============================================================================

    # Cases: (no "compare" and "source" empty) and ("compare" empty and "source" empty)

    if Ccount == 0 and Scount == 0:
        if debug: print '\nnormalize: "source" and "target" tables '+source+' and '+compare+' missing or empty. "source" unchanged.'
        status = 0
        return status


    # ============================================================================
    # 4. EXTRACT INTERNAL MISSING INTERVALS
    # ============================================================================
   
    # If table "source" is not empty, extract its internal missing intervals

    if Scount > 0:

        # Build main queries "Qsource" and "Qinsert"
        # ------------------------------------------
    
        # Scan source table
        Qsource  = u"select " + allColumns + u" from " + source + u" order by " + Scolumns + u"," + SstartName
        if debug: print '\n-- Qsource:',Qsource

        # Store missing row into MISSING
        Qinsert  = u"insert into MISSING("+allColumns+u") values("+lenSColumns*u'?,'+"?,?)"
        if debug: print '-- Qinsert:',Qinsert

        # Prepare the parsing of "source" data
        # ------------------------------------
        
        if debug: print '\nparsing "source" table '+source
        try:
            cur1.execute(Qsource)
        except Exception, e:
            print '"normalize": error in "Qsource"',e
            print Qsource
            status = 3
            return status
        
        first = True    # First or next row extracted?
        nMissing = 0    # Number of missing intervals detected

        # Parse "source" data
        # -------------------
        
        for row in cur1:
            if debug: print '  - examining row:',row
            valCol = row[:-2]
            (start,end) = row[-2:]
            if first:
                curValCol,(curStart,curEnd) = valCol,(start,end)        # curRow <- row
                first = False
                if debug: print '    --> first current row:',(curValCol + (curStart,curEnd))
            else:
                if valCol == curValCol:
                    # Same column values
                    if curEnd < start:
                        # Missing interval detected
                        if debug: print '  ! Missing interval detected.',(curValCol + (curEnd,start))
                        valueList = [unicode(e) for e in list(valCol + (curEnd,start))]
                        valueListStr = ','.join(valueList)
                        #print '    missing:',(curValCol + (curEnd,start))
                        if debug: print '  execute: "insert into MISSING('+Scolumns+','+SstartName+','+SendName+')'\
                                +u' values('+valueListStr+')"'
                    
                        try:
                            cur2.execute(Qinsert,(curValCol + (curEnd,start)))
                        except Exception,e:
                            print '"normalize": error in "Qinsert"',e
                            print Qinsert
                            status = 4
                            return status
                        
                        curStart,curEnd = start,end
                        if debug: print '    --> new current row:',(curValCol + (curStart,curEnd))
                        nMissing += 1
                    else:
                        # No missing interval. Get next.
                        if debug: print '  No missing interval.'
                        curEnd = end
                        if debug: print '    --> new current row:',(curValCol + (curStart,curEnd))
                else:
                    # New column values
                    curValCol,(curStart,curEnd) = valCol,(start,end)
                    if debug: print '    --> new current row:',(curValCol + (curStart,curEnd))

        if debug: print '\nNumber of internal missing intervals in ',source,':',nMissing


    # ============================================================================
    # 5. EXTRACT COMPARE-BASED MISSING INTERVALS
    # ============================================================================

    # 5.1 Extract leading and trailing missing intervals if "compare" exists, is not empty and "source" not empty
    # -----------------------------------------------------------------------------------------------------------
    
    if compare != u'' and Ccount > 0 and Scount > 0:
                    
        prefCol1 = u'_COM.'+Ccolumns.replace(u',',u',_COM.')
        prefCol2 = u'_SOU.'+Scolumns.replace(u',',u',_SOU.')
    
        # Leading missing intervals
        # -------------------------
        if debug: print '\nadding to MISSING leading missing intervals of "source" '+source+' wrt "compare" '+compare+"."

        Qleading =   "select "+prefCol2+",min(_COM."+CstartName+") as _start, min(_SOU."+SstartName+") as _end "\
                   + "from   "+source+" _SOU, "+compare+" _COM "\
                   + "where  ("+prefCol1+") = ("+prefCol2+") "\
                   + "group by "+prefCol2+" "\
                   + "having _start < _end"
        if debug: print '-- Qleading:',Qleading

        try:
            cur1.execute(u"insert into MISSING("+Scolumns+","+SstartName+","+SendName+") " + Qleading)
        except Exception,e:
            print '"normalize": error in "Qleading":',e
            print Qinsert
            status = 4
            return status


        # Trailing missing intervals
        # --------------------------
        if debug: print '\nadding to MISSING trailing missing intervals of "source" '+source+' wrt "compare" '+compare+"."
   
        Qtrailing =   "select "+prefCol2+",max(_SOU."+SendName+") as _start, max(_COM."+CendName+") as _end "\
                    + "from   "+source+" _SOU, "+compare+" _COM "\
                    + "where  ("+prefCol1+") = ("+prefCol2+") "\
                    + "group by "+prefCol2+" "\
                    + "having _start < _end"
        if debug: print '-- Qtrailing:',Qtrailing

        try:
            cur1.execute(u"insert into MISSING("+Scolumns+","+SstartName+","+SendName+") " + Qtrailing)
        except Exception,e:
            print '"normalize": error in "Qtrailing":',e
            print Qinsert
            status = 4
            return status


    # 5.2 Extract intervals of "compare" that have no match in "source" 
    # -----------------------------------------------------------------
    # If a value of "Ccolumns" in "compare" does not exist in "source",
    # a null missing interval with this value must be created 


    # 5.2.1 "compare" exists, is not empty and "source" IS EMPTY (trivial case)
    # -------------------------------------------------------------------------
    
    if compare != u'' and Ccount > 0 and Scount == 0:
        Qempty1 = u"insert into MISSING("+allColumns+u") "\
                + "select " + Ccolumns + ",min("+CstartName+u"), max("+CendName+u") "\
                + "from "+compare+" group by "+Ccolumns
        if debug: print '-- Qempty1:',Qempty1

        try:
            cur1.execute(Qempty1)
        except Exception,e:
            print '"normalize": error in "Qempty1"',e
            print Qempty1
            status = 3
            return status

    # 5.2.2 "compare" exists, is not empty and "source" IS NOT EMPTY (non trivial case)
    # ---------------------------------------------------------------------------------

    if compare != u'' and Ccount > 0 and Scount > 0:
        Qempty2 = u"insert into MISSING("+allColumns+u") "\
                + "select " + Ccolumns + ",min(" + CstartName + u"), max(" + CendName + u") "\
                + "from " + compare\
                + " where  (" + Ccolumns + ") not in (select " + Scolumns + " from " + source + ")"\
                + " group by " + Ccolumns
        if debug: print '-- Qempty2:',Qempty2

        try:
            cur1.execute(Qempty2)
        except Exception,e:
            print '"normalize": error in "Qempty2"',e
            print Qempty2
            status = 3
            return status


    # ============================================================================
    # 6. COPY MISSING INTERVALS INTO "target"
    # ============================================================================

    if debug:
        if target == u'':
            print '\nComplete "source" '+source+' with missing intervals.'
        else:
            print '\nCreate "target" '+target+' with ' + source + ' + missing intervals.'

    if target != u'':
        try:
            cur1.execute(u"insert into " + target + u" select * from " + source)
            cur1.execute(u"insert into " + target + u" select * from MISSING")
        except Exception,e:
            print '"normalize": error ',e
            print u'in "insert into '+source+u' select * from MISSING"'
            status = 4
            return status
    else:
        try:
            cur1.execute(u"insert into " + source + u" select * from MISSING")
        except Exception, e:
            print '"normalize": error ',e
            print u'in "insert into '+source+u' select * from MISSING"'
            status = 4
            return status


    # ============================================================================
    # 7. CLOSE PROCESS
    # ============================================================================

    # cur1.execute(u"drop table MISSING")

    cur2.close()
    cur1.close()

    db.commit()

    if debug:
        print '\n-----------------------------------------'
        print   '---------- EXIT LTemp:normalize ---------'
        print   '-----------------------------------------'
    
    return status


def valuedInterval(db,tQuery):
    # General AGGREGATION procedure
    # Argument: "select <columns> from <source> where <condition> group by <groupBy> into <target>"
    # Compute the stable intervals of the rows of table <source>
    #         [that satisfy <condition>]
    #         valued with colums <columns>
    #         [grouped by <groupBy>]
    # and return the result in temp table <target> (preexisting target table is deleted).
    # If <groupBy> is missing, the stable intervals are not partitionned but common to all the rows.
    # <columns> is a list of unprefixed column names.
    # Columns "start" and "end" of <source> must be the last ones
    # Examples:
    # - function status = LTemp:valuedInterval {select CITY,PROJECT,SALARY
    #                                           from   H_EMPLOYEE
    #                                           where  SALARY > 6000
    #                                           group by CITY,PROJECT
    #                                           into VALUED_STABLE_INTERVAL};
    # - function status = LTemp:valuedInterval {select CITY,PROJECT,SALARY
    #                                           from   H_EMPLOYEE
    #                                           where  SALARY > 6000
    #                                           into COMMON_VALUED};
    # status:
    #  0: OK
    #  1: database not opened
    #  2: query syntax error
    #  3: unable to extract source data
    #  4: unable to insert result in target table
    

    if debug:
        print '\n\n-----------------------------------------------'
        print     '--------- Enter LTemp:valuedInterval  ---------'
        print     '-----------------------------------------------'

    status = 0

    try:
        cur1 = db.cursor()
        cur2 = db.cursor()
    except Exception,e:
        print 'Error in cursor extraction',e
        status = 1
        return status


    # ============================================================================
    # 1. GET BASE PARAMETERS
    # ============================================================================

    # 1.1 Extract query components
    # ----------------------------

    source,condition,columns,groupBy,target = queryDecomposition(tQuery)

    if debug: print '\nIn "valuedInterval":'
    if debug: print '   source   :',source
    if debug: print '   condition:',condition
    if debug: print '   columns  :',columns
    if debug: print '   groupBy  :',groupBy
    if debug: print '   target   :',target
    
    if source == u'' or target == u'':
        status = 2
        return


    # 1.2 Get the names of "start" and "end" columns
    # ----------------------------------------------

    startName,endName = getIntervalNames(cur1,source)

    if startName == u'':
        print '"LTemp:valuedInterval", error in getting "start" and "end" names of '+source
        status = 3
        return status
    
    allColumns = columns+u","+startName+","+endName




    # Clean the table
    # ---------------
    try:
        cur1.execute("drop table if exists _EVENT")
        cur1.execute("drop table if exists _STABLE_INTERVAL")
        cur1.execute("drop table if exists " + target)
    except Exception, e:
        print 'Error in table drop',e
        status = 3
        return status

    # Build the extraction queries
    # ----------------------------

    # Prepare prefixed column lists
    EcolumnsL = [u'E.'+e.strip() for e in columns.split(u',')]
    EgroupByL = [u'E.'+e.strip() for e in groupBy.split(u',')]
    SgroupByL = [u'S.'+e.strip() for e in groupBy.split(u',')]
    
    Eprefcolumns = u','.join(EcolumnsL)
    EprefgroupBy = u','.join(EgroupByL)
    SprefgroupBy = u','.join(SgroupByL)
    
    if debug: print '   Eprefcolumns:',Eprefcolumns
    if debug: print '   EprefgroupBy:',EprefgroupBy
    if debug: print '   SprefgroupBy:',SprefgroupBy

    # Process the "where" clause
    where1 = u'' if condition == u'' else u' where ' + condition
    where2 = u'' if condition == u'' else u' and ' + condition

    if groupBy != u'':
        # Queries for partitioned stable intervals
        # ----------------------------------------
        Qevent =  u"create temp table _EVENT as "\
                + u"  select " + groupBy + u",Instant "\
                + u"  from (select " + groupBy + u"," + startName + u" as Instant "\
                + u"        from " + source + where1\
                + u"          union "\
                + u"        select " + groupBy + u",end as Instant "\
                + u"        from " + source + where1 + u")"
        
        Qinterval1 = u"create temp table _STABLE_INTERVAL as "\
                   + u"  select " + groupBy + u",Instant as Start,Instant as End "\
                   + u"  from   _EVENT where  1 = 2 "
    
        Qindex = u"create index XSE on _STABLE_INTERVAL(" + groupBy + u",start);"
    
        Qinterval2 = u"select " + groupBy + u",Instant "\
                   + u"from   _EVENT order by "+ groupBy + u",Instant"

        formCols = ["?" for c in groupBy.split(u',')] + [u'?',u'?']
        Qinsert  = u"insert into _STABLE_INTERVAL values ("+','.join(formCols)+u')'

    
        Qvalued = u"create temp table " + target + " as "\
                + u"  select " + Eprefcolumns + u",S.Start,S.End "\
                + u"  from   _STABLE_INTERVAL S, " + source + " E "\
                + u"  where  (" + EprefgroupBy + u") = (" + SprefgroupBy + u")" + where2\
                + u"  and    E." + startName + u" <= S.Start and S.Start < E." + endName

    else:
        # Queries for common stable intervals
        # -----------------------------------
        Qevent = u"create temp table _EVENT as"\
               + u"  select Instant "\
               + u"  from (select start as Instant"\
               + u"        from " + source + where1\
               + u"           union"\
               + u"        select end as Instant"\
               + u"        from " + source + where1 + u") "
        
        Qinterval1 = u"create temp table _STABLE_INTERVAL as "\
                   + u"  select Instant as Start,Instant as End"\
                   + "   from   _EVENT where  1 = 2 "
    
        Qindex = u"create index XSE on _STABLE_INTERVAL(start);"
    
        Qinterval2 = u"select Instant from _EVENT order by Instant"

        Qinsert  = u"insert into _STABLE_INTERVAL values (?,?)"

    
        Qvalued = u"create temp table " + target + " as "\
                + u"  select " + Eprefcolumns + u",S.Start,S.End "\
                + u"  from   _STABLE_INTERVAL S, " + source + " E "\
                + u"  where  E." + startName + u" <= S.Start and S.Start < E." + endName + where2

    if debug: print '\n-- Qevent:',Qevent
    if debug: '-- Qinterval1:',Qinterval1
    if debug: '-- Qinterval2:',Qinterval2
    if debug: '-- Qinsert:',Qinsert
    if debug: '-- Qvalued:',Qvalued

    # Extract events into _EVENT
    # --------------------------
    try:
        cur1.execute(Qevent)
    except Exception, e:
        print 'Error in _EVENT extraction:',e
        try:
            cur1.execute("drop table _EVENT")
        except:
            pass
        status = 3
        return status

    # Build stables intervals
    # -----------------------

    # Create table _STABLE_INTERVAL
    try:
        cur1.execute(Qinterval1)
        cur1.execute(Qindex)
    except Exception, e:
        print 'Error in _STABLE_INTERVAL creation:',e
        try:
            cur1.execute("drop table _EVENT")
            cur1.execute("drop table _STABLE_INTERVAL")
        except:
            pass
        status = 3
        return status

    # Build stable intervals into _STABLE_INTERVAL
    try:
        cur1.execute(Qinterval2)
    except Exception, e:
        print 'Error in _STABLE_INTERVAL extraction:',e
        try:
            cur1.execute("drop table _EVENT")
            cur1.execute("drop table _STABLE_INTERVAL")
        except:
            pass
        status = 3
        return status

    # Get the first event
    event = cur1.fetchone()
    if event is None:
        Terminer
    values1  = event[:-1]
    instant1 = event[-1:][0]
    if debug: print '  values1: ',values1
    if debug: print '  instant1:',instant1

    # Now, get and process next events
    for event in cur1:
        if debug: print 'Next event:',event
        values2  = event[:-1]
        instant2 = event[-1:][0]
        if debug: print '  Inserting:',(values1 + (instant1,instant2))
        cur2.execute(Qinsert,(values1 + (instant1,instant2)))   # insert current interval
        (values1,instant1) = (values2,instant2)

    if debug: print 'Insertion closed\n'

    # Build valued stable intervals
    # -----------------------------
    try:
        cur1.execute(Qvalued)
    except Exception, e:
        print 'Error in ' + target + ' loading:',e
        try:
            cur1.execute("drop table _EVENT")
            cur1.execute("drop table _STABLE_INTERVAL")
            cur1.execute("drop table " + target)
        except:
            pass
        status = 4
        return status

    # ============================================================================
    # 4. CLOSE PROCESS
    # ============================================================================


    # Drop internal tables
    # --------------------
    try:
        cur1.execute("drop table _EVENT")
        cur1.execute("drop table _STABLE_INTERVAL")
        pass
    except Exception, e:
        print 'Error in table drop:',e
        status = 4
        return status

    cur2.close()
    cur1.close()
    
    db.commit()
    
    if debug:
        print '\n-----------------------------------------------'
        print   '---------- EXIT LTemp:valuedInterval  ---------'
        print   '-----------------------------------------------'

    return status

"""
openDB TDB.db;
function status = LTemp:valuedIntervalE {select X.PROJECT,X.CITY,X.CODE,X.SALARY,X.SALARY/12 as MONTHLY
                                         from H_EMPLOYEE X into VALUED_INTERVAL};
write $status$;
select thousandSep(count(*),',') as "VALUED_NTERVAL" from VALUED_INTERVAL;
select * from VALUED_INTERVAL order by PROJECT,CITY,start;
   write-b @sAnother view of VALUED_INTERVAL.;
   select PROJECT,CITY,group_concat(SALARY) as Salaries,Start,End
   from   VALUED_INTERVAL group by PROJECT,CITY,Start order by PROJECT,CITY,Start;

closeDB;
"""
def valuedIntervalE(db,tQuery):
    #def valuedStableE(db,tQuery):
    # General AGGREGATION procedure with alias
    # Argument: "select <columns> from <source> <alias> where <condition> group by <groupBy> into <target>"
    # Compute the stable intervals of the rows of table <source> <alias>
    #         [that satisfy <condition>]
    #         valued with colums <columns>
    #         [grouped by <groupBy>]
    # and return the result in table <target> (preexisting target table is deleted)
    # <columns> is a list of SQL scalar expressions where column names are prefixed
    #      with <alias> for <source> columns
    #      and 'S' for Start and End from "_STABLE_INTERVAL".
    # If <groupBy> is missing, the stable intervals are not partitionned but common to all the rows.
    # Columns "start" and "end" of <source> must be the last ones
    # Examples:
    # - function status = LTemp:valuedIntervalE {select E.CITY,E.PROJECT,E.SALARY,durationDays(S.Start,S.End) as C
    #                                            from   H_EMPLOYEE E
    #                                            where SALARY > 6000
    #                                            group by E.CITY,E.PROJECT
    #                                            into H_CITY_PROJECT};
    # - function status = LTemp:valuedIntervalE {select E.CITY,E.PROJECT,E.SALARY/12 as MONTHLY
    #                                            from   H_EMPLOYEE E
    #                                            where  E.SALARY > 6000
    #                                            into COMMON_VALUED};
    # status:
    #  0: OK
    #  1: database not opened
    #  2: query syntax error
    #  3: unable to extract source data
    #  4: unable to insert result in target table

    status = 0

    try:
        cur1 = db.cursor()
        cur2 = db.cursor()
    except Exception,e:
        print 'Error in cursor extraction:',e
        status = 1
        return status

    # Extract query components
    # ------------------------
    source,alias,condition,columns,groupBy,target = queryDecompositionE(tQuery)
    if debug: print '\nIn "valuedIntervalE":'
    if debug: print '   source   :',source
    if debug: print '   alias    :',alias
    if debug: print '   condition:',condition
    if debug: print '   columns  :',columns
    if debug: print '   groupBy  :',groupBy
    if debug: print '   target   :',target
    
    if source == u'' or target == u'':
        status = 2
        return

    # Clean the table
    # ---------------
    try:
        cur1.execute("drop table if exists _EVENT")
        cur1.execute("drop table if exists _STABLE_INTERVAL")
        cur1.execute("drop table if exists " + target)
    except Exception, e:
        print 'Error in table drop:',e
        status = 3
        return status

    # Build the extraction queries
    # ----------------------------

    # Prepare prefixed column lists
    SgroupBy = groupBy.replace(alias+u'.',u'S.')
    NgroupBy = groupBy.replace(alias+u'.',u'')
    
    if debug: print '\n   SgroupBy:',SgroupBy
    if debug: print '   NgroupBy:',NgroupBy

    where1 = u'' if condition == u'' else u' where ' + condition
    where2 = u'' if condition == u'' else u' and ' + condition

    if groupBy != u'':
        # Queries for partitionned stable intervals
        # -----------------------------------------
        Qevent = u"""create temp table _EVENT as
                     select """ + NgroupBy + u""",Instant
                     from (select """ + NgroupBy + u""",start as Instant from """ + source + where1 + u"""
                           union
                           select """ + NgroupBy + u""",end as Instant from """ + source +where1 + u""") """

        Qinterval1 = u"""create temp table _STABLE_INTERVAL as    
                         select """ + NgroupBy + u""",Instant as Start,Instant as End
                         from   _EVENT where  1 = 2 """
    
        Qindex = u"""create index XSE on _STABLE_INTERVAL(""" + NgroupBy + u""",Start);"""
    
        Qinterval2 = u"""select """ + NgroupBy + u""",Instant
                         from   _EVENT order by """ + NgroupBy + u""",Instant"""

        formCols = ["?" for c in groupBy.split(u',')] + [u'?',u'?']
        Qinsert  = u"insert into _STABLE_INTERVAL values ("+','.join(formCols)+u')'

        sourceAlias = source + u' ' + alias
        Qvalued = u"""create temp table """ + target + """ as
                     select """ + columns + u""",S.Start,S.End
                     from   _STABLE_INTERVAL S, """ + sourceAlias + """
                     where  (""" + groupBy + u""") = (""" + SgroupBy + u""")""" + where2 + """
                     and    """ + alias + """.Start <= S.Start and S.Start < """ + alias + """.end """

    else:
        # Queries for common stable intervals
        # -----------------------------------
        Qevent = u"""create temp table _EVENT as
                     select Instant
                     from (select start as Instant from """ + source + where1 + u"""
                           union
                           select end as Instant from """ + source +where1 + u""") """

        Qinterval1 = u"""create temp table _STABLE_INTERVAL as    
                         select Instant as Start,Instant as End
                         from   _EVENT where  1 = 2 """
    
        Qindex = u"""create index XSE on _STABLE_INTERVAL(start);"""
    
        Qinterval2 = u"""select Instant from _EVENT order by Instant"""

        Qinsert  = u"insert into _STABLE_INTERVAL values (?,?)"

        sourceAlias = source + u' ' + alias
        Qvalued = u"""create temp table """ + target + """ as
                     select """ + columns + u""",S.Start,S.end
                     from   _STABLE_INTERVAL S, """ + sourceAlias + """
                     where """ + alias + """.Start <= S.Start and S.Start < """ + alias + """.end """ + where2
    

    if debug: print '\n-- Qevent:',Qevent
    if debug: print '-- Qinterval1:',Qinterval1
    if debug: print '-- Qinterval2:',Qinterval2
    if debug: print '-- Qinsert:',Qinsert
    if debug: print '-- Qvalued:',Qvalued

    # Extract events into _EVENT
    # --------------------------
    try:
        cur1.execute(Qevent)
    except Exception, e:
        print 'Error in EVENT extraction:',e
        try:
            cur1.execute("drop table _EVENT")
        except:
            pass
        status = 3
        return status


    # Build stables intervals
    # -----------------------
    
    # Create table _STABLE_INTERVAL
    try:
        cur1.execute(Qinterval1)
        cur1.execute(Qindex)
    except Exception, e:
        print 'Error in _STABLE_INTERVAL creation:',e
        try:
            cur1.execute("drop table _EVENT")
            cur1.execute("drop table _STABLE_INTERVAL")
        except:
            pass
        status = 3
        return status

    # Build stable intervals into _STABLE_INTERVAL
    try:
        cur1.execute(Qinterval2)
    except Exception, e:
        print 'Error in _STABLE_INTERVAL extraction:',e
        try:
            cur1.execute("drop table _EVENT")
            cur1.execute("drop table _STABLE_INTERVAL")
        except:
            pass
        status = 3
        return status

    # Get the first event
    event = cur1.fetchone()
    if debug: print '\nFirst event:',event    
    if event is None:
        Terminer
    values1  = event[:-1]
    instant1 = event[-1:][0]
    if debug: print '  values1: ',values1
    if debug: print '  instant1:',instant1

    # Now, get and process next events
    for event in cur1:
        if debug: print 'Next event:',event
        values2  = event[:-1]
        instant2 = event[-1:][0]
        if debug: print '  Inserting:',(values1 + (instant1,instant2))
        cur2.execute(Qinsert,(values1 + (instant1,instant2)))   # insert current interval
        (values1,instant1) = (values2,instant2)

    #print 'Insertion closed\n'


    # Build valued stable intervals
    # -----------------------------
    try:
        cur1.execute(Qvalued)
    except Exception, e:
        print 'Error in ' + target +' extraction:',e
        try:
            cur1.execute("drop table _EVENT")
            cur1.execute("drop table _STABLE_INTERVAL")
            cur1.execute("drop table " + target)
        except:
            pass
        status = 4
        return status

    # Drop internal tables
    # --------------------
    try:
        cur1.execute("drop table _EVENT")
        cur1.execute("drop table _STABLE_INTERVAL")
    except Exception, e:
        print 'Error in table drop:',e
        status = 4
        return status

    return status

def counting(db,tQuery):
    #
    # Procedural simulation of:
    #  select PROJECT,count(*) as Number,start,end
    #  from   VALUED_STABLE_EMPLOYEE group by PROJECT,start,end;

    cur1 = db.cursor()
    cur2 = db.cursor()
    cur1.execute('select PROJECT,start,end from STABLE_EMPLOYEE order by PROJECT,start')
    # Fails if no "order by"
    first = True
    for (project,start,end) in cur1:
        #print '   Examining',project,start,end
        if first:
            curProject,curStart,curEnd = project,start,end
            first = False
            count = 1
        else:
            if (project,start) == (curProject,curStart):
                curEnd = end
                count += 1
            else:
                #print curProject,curStart,curEnd,count
                cur2.execute("insert into E_COUNT values (?,?,?,?)",(curProject,count,curStart,curEnd))
                curProject,curStart,curEnd = project,start,end
                count = 1
    if count > 0:
        #print curProject,count,curStart,curEnd
        cur2.execute("insert into E_COUNT values (?,?,?,?)",(curProject,count,curStart,curEnd))
    return


