-- Chapitre 22, 4e dition [2020-05-27]
-- Un tudiant signale une erreur : chapitre 22, page 680, dans le tableau, la raction  l'vnement
-- "update CLIENT set Total" intitule "trigger annulant l'opration" est incorrecte.
-- En effet, ce trigger empchera galement les updates induits par les autres
-- vnements.  Ci-dessous un petit script SQLfast qui illustre, sur un exemple
-- plus simple, le problme identifi et quatre solutions. Attention, la solution n 1
-- (colonne calcule) ncessite la version 4 de SQLfast.

ask scenario = Scenario:[(PROBLEM,SOLUTION-1,SOLUTION-2,SOLUTION-3,SOLUTION-4)];
goto $scenario$;

label PROBLEM;
-- ----------
createDB InMemory;
create table T(A integer not null primary key,B integer);

create trigger TRG_T_INSERT after insert on T
begin update T set B = new.A * new.A where A = new.A;end;

create trigger TRG_T_UPDATE_B after update of B on T
begin select raise(ABORT,'Invalid operation');end;

onError continue;
insert into T(A) values (1),(2),(3);
write $EXTENDEDdiag$;
onError stop;
goto FIN;

label SOLUTION-1;
-- -------------
-- Using generated column --
createDB InMemory;
create table T(A integer not null primary key,
               B integer generated always as (A*A));

onError continue;
insert into T(A) values (1),(2),(3);
write $EXTENDEDdiag$;
select * from T;
update T set B = 5 where A = 2;
write $EXTENDEDdiag$;
goto FIN;

label SOLUTION-2;
-- -------------
-- Using views and "instead of" triggers (ABORT) --
createDB InMemory;
create table _T(A integer not null primary key,B integer);
create view T as select * from _T;

create trigger TRG_T_INSERT instead of insert on T
begin insert into _T values (new.A, new.A * new.A);end;

create trigger TRG_T_UPDATE_B instead of update of B on T
begin select raise(ABORT,'Invalid operation');end;

onError continue;
insert into T(A) values (1),(2),(3);
select * from T;
update T set B = 5 where A = 2;
write $EXTENDEDdiag$;
goto FIN;

label SOLUTION-3;
-- -------------
-- <idem> + automatic recomputation --
createDB InMemory;
create table _T(A integer not null primary key,B integer);
create view T as select * from _T;

create trigger TRG_T_INSERT instead of insert on T
begin insert into _T values (new.A, new.A * new.A);end;

create trigger TRG_T_UPDATE_B instead of update of B on T
begin update _T set B = new.A*new.A where A = new.A;end;

onError continue;
insert into T(A) values (1),(2),(3);
select * from T;
update T set B = 5 where A = 2;
write $EXTENDEDdiag$;
goto FIN;

label SOLUTION-4;
-- -------------
createDB InMemory;
create table T(A integer not null primary key,B integer);

create trigger TRG_T_INSERT after insert on T
begin update T set B = new.A*new.A where A = new.A;end;

create trigger TRG_T_UPDATE_B after update of B on T
when not (new.B = new.A*new.A)
begin update T set B = new.A*new.A where A = new.A;end;

onError continue;
insert into T(A) values (1),(2),(3);
write $EXTENDEDdiag$;
select * from T;
update T set B = 5 where A = 2;
write $EXTENDEDdiag$;
goto FIN;

label FIN;
onError stop;
