-- --------------------------------------------------- --
-- RETAIL schema of the standard 3-tier implementation --
-- --------------------------------------------------- --

createOrReplaceDB RETAIL.db;

create table CUSTOMER (
        CustID  char(10) not null,
        Name    char(32) not null,
        Address char(60) not null,
        City    char(30) not null,
        Account decimal(9,2) not null,
        constraint PKCUST primary key (CustID));

create table ITEM (
        ItemID      char(15) not null,
        Description char(60) not null,
        Price       decimal(4,2) not null,
        SuppID      char(10),
        SuppPrice   decimal(4,2),
        QonHand     integer not null,
        Qord        integer default 0,
        Qavail      integer not null,
        Qeco        integer not null,
        constraint PKITEM primary key (ItemID));

create table CUSTORDER (
        OrdID   integer  not null,
        DateOrd date     not null,
        CustID  char(10) not null,
        ItemID  char(15) not null,
        Price   decimal(4,2),
        Qty     integer  not null,
        State   char(20) not null,
        constraint PKCORD primary key (OrdID),
        constraint FKORDCUST foreign key (CustID)
           references CUSTOMER
           on delete no action on update cascade,
        constraint FKORDITEM foreign key (ItemID)
           references ITEM
           on delete no action on update cascade);

create table CUSTINVOICE (
        InvID   integer  not null,
        DateInv date not null,
        OrdID   integer  not null,
        CustID  char(10) not null,
        ItemID  char(15) not null,
        Price   decimal(4,2) not null,
        Qty     integer  not null,
        Amount  integer  not null,
        State   char(20) not null,
        constraint PKCINV primary key (InvID),
        constraint FKINVORD foreign key (OrdID)
           references CUSTORDER
           on delete no action on update cascade);

create table SUPPLIER (
        SuppID  char(10) not null,
        Name    char(32) not null,
        City    char(30) not null,
        constraint PKSUPP primary key (SuppID));

create table OFFER (
        SuppID  char(10) not null,
        ItemID  char(15) not null,
        Price   decimal(4,2) not null,
        Delay   integer not null,
        constraint PKOFF primary key (SuppID,ItemID),
        constraint FKOFFSUPP foreign key (SuppID)
           references SUPPLIER
           on delete no action on update cascade,
        constraint FKOFFITEM foreign key (ItemID)
           references ITEM
           on delete no action on update cascade);

create table SUPPORDER (
        OrdID   integer not null,
        DateOrd date not null,
        ItemID  char(15) not null,
        SuppID  char(10) not null,
        Price   decimal(4,2) not null,
        Qty     integer  not null,
        State   varchar(20) not null,
        constraint PKSORD primary key (OrdID)
        constraint FKORDSUPP foreign key (SuppID,ItemID)
           references OFFER
           on delete no action on update cascade);

closeDB;

return;











