write-b -- ----------------------------- --;
write   -- Active databases              --;
write   -- Data modification logging     --;
write-a -- ----------------------------- --;

set ; = ;;

createDB InMemory;

create table CUSTORDER(
       OrdID   char(12) not null primary key,
       DateOrd date not null,
       CustID  char(12) not null,
       ItemID  char(16) not null,
       Qty     integer not null,
       Status  varchar(20) not null);

create table JOURNAL(
    LogID     integer not null primary key autoincrement,
    OpTime    datetime not null,
    Operation char(32) not null,
    TableName varchar(64) not null,
    RowID     varchar(128) not null,
    Arguments varchar(1024));

-- (LogID,OpTime,Operation,TableName,RowID,Status,Arguments);

create trigger TRG_ORDER_LOG_INS
after insert on CUSTORDER
for each row
begin
  insert into JOURNAL(OpTime,Operation,TableName,RowID,Arguments)
         values(current_timestamp_full(),
               'insert',
               'CUSTORDER',
               new.OrdID,
               '"'||new.DateOrd||'"'
              ||',"'||new.CustID||'"'
              ||',"'||new.ItemID||'"'
              ||','||cast(new.Qty as char) 
              ||',"'||new.Status||'"') $;$ 
end;

create trigger TRG_ORDER_LOG_UPD
after update on CUSTORDER
for each row
begin
  insert into JOURNAL(OpTime,Operation,TableName,RowID,Arguments)
         values(current_timestamp_full(),
               'update',
               'CUSTORDER',
               old.OrdID,
               '"'||case when new.DateOrd=old.DateOrd then '' else new.DateOrd end||'"'
              ||',"'||case when new.CustID=old.CustID then '' else new.CustID end||'"'
              ||',"'||case when new.ItemID=old.ItemID then '' else new.ItemID end||'"'
              ||','||case when new.Qty=old.Qty then '' else cast(new.Qty as char) end
              ||',"'||case when new.Status=old.Status then '' else new.Status end||'"') $;$ 
end;

create trigger TRG_ORDER_LOG_DEL
after delete on CUSTORDER
for each row
begin
  insert into JOURNAL(OpTime,Operation,TableName,RowID)
         values(current_timestamp_full(),
               'delete',
               'CUSTORDER',
               old.OrdID) $;$ 
end;

write-b Initial state of table CUSTORDER:;
select * from CUSTORDER;

write-b The operations:;
write   --------------;
write insert into CUSTORDER values('123','2020-02-16','B512','PA45',12,'recorded');;
write insert into CUSTORDER values('184','2020-02-17','H054','PA60',5,'recorded');;
write delete from CUSTORDER where OrdID = '123';;
write insert into CUSTORDER values('270','2020-02-19','F400','PH222',2,'recorded');;
write update CUSTORDER set Status = 'validated' where OrdID = '184';;

insert into CUSTORDER values('123','2020-02-16','B512','PA45',12,'recorded');
insert into CUSTORDER values('184','2020-02-17','H054','PA60',5,'recorded');
delete from CUSTORDER where OrdID = '123';
insert into CUSTORDER values('270','2020-02-19','F400','PH222',2,'recorded');
update CUSTORDER set Status = 'validated' where OrdID = '184';

write-b Content of the journal:;
select LogID,OpTime,Operation,TableName,RowID from JOURNAL;
select LogID,RowID,Arguments from JOURNAL;

write-b Final state of table CUSTORDER:;
select * from CUSTORDER;

closeDB;

write-ab End of script;
return;
