write-b -- ----------------------------- --;
write   -- Active databases              --;
write   -- Redundancy and derived data   --;
write-a -- ----------------------------- --;

-- ------------------------------------- --
--  ORDERS with derived data             --
-- ------------------------------------- --

set ; = ;;

-- DB in RAM (1) or on disk (2)?
set db = 1;
 
if ($db$);
   createDB InMemory;
else;
   createOrReplaceDB ORDERS-Extended.db;
endif;

create table CUSTOMER (
       CustID      char(10)    not null primary key,
       Name        char(32)    not null,
       Address     char(60)    not null,
       City        char(30)    not null,
       Cat         char(2),
       Account     decimal(9,2)not null,
       AmountCust  decimal(8)  not null default 0);

create table ITEM (
       ItemID      char(15)    not null primary key,
       Description char(60)    not null,
       UnitPrice   decimal(6)  not null,
       Qavail      decimal(8)  not null)

create table CUSTORDER (
       OrdID       char(12)    not null primary key,
       CustID      char(10)    not null,
       DateOrd     date        not null,
       AmountOrd   decimal(8)  not null default 0,
       foreign key (CustID) references CUSTOMER
            on delete no action on update cascade);

create table DETAIL (
        OrdID       char(12)    not null,
        ItemID      char(15)    not null,
        Qty         decimal(8)  not null,
        AmountDet   decimal(8)  not null default 0,
        primary key (OrdID,ItemID),
        foreign key (OrdID) references CUSTORDER
             on delete cascade on update cascade,
        foreign key (ItemID) references ITEM
             on delete no action on update cascade);


-- ------------------------------------- --
-- Operations on table ITEM              --
-- ------------------------------------- --

-- UPDATE ITEM (UnitPrice) --

create trigger TRG_PRO_UPD_UnitPrice
after update of UnitPrice on ITEM
for each row
begin
  update DETAIL 
  set    AmountDet = Qty * new.UnitPrice
  where  ItemID = new.ItemID$;$ 
end;


-- ------------------------------------- --
-- Operations on table DETAIL            --
-- ------------------------------------- --

-- INSERT into DETAIL --

create trigger TRG_DET_INS
after insert on DETAIL
for each row
begin
  update DETAIL 
  set    AmountDet = new.Qty * (select UnitPrice
                                from   ITEM
                                where  ItemID = new.ItemID)
  where  OrdID = new.OrdID and ItemID = new.ItemID$;$ 
  --
  update CUSTORDER 
  set    AmountOrd = (select sum(AmountDet)
                      from   DETAIL
                      where  OrdID = new.OrdID)
  where  OrdID = new.OrdID$;$ 
end;


-- DELETE from DETAIL --

create trigger TRG_DET_DEL
after delete on DETAIL
for each row
begin
  update CUSTORDER 
  set    AmountOrd = coalesce((select sum(AmountDet)
                               from   DETAIL
                               where  OrdID = old.OrdID),0)
  where  OrdID = old.OrdID$;$ 
end;


-- UPDATE DETAIL (OrdID,ItemID,Qty,AmountDet) --

create trigger TRG_DET_UPD_OrdID
after update of OrdID on DETAIL
for each row
begin
  update CUSTORDER 
  set    AmountOrd = coalesce((select sum(AmountDet)
                               from   DETAIL
                               where  OrdID = old.OrdID),0)
  where  OrdID = old.OrdID$;$
  --
  update CUSTORDER 
  set    AmountOrd = (select sum(AmountDet)
                      from   DETAIL
                      where  OrdID = new.OrdID)
  where  OrdID = new.OrdID$;$ 
end;

create trigger TRG_DET_UPD_ItemID
after update of ItemID on DETAIL
for each row
begin
  update DETAIL 
  set    AmountDet = new.Qty * (select UnitPrice
                                from   ITEM
                                where  ItemID = new.ItemID)
  where  OrdID = new.OrdID and ItemID = new.ItemID$;$ 
end;

create trigger TRG_DET_UPD_Qty
after update of Qty on DETAIL
for each row
begin
  update DETAIL 
  set    AmountDet = new.Qty * (select UnitPrice
                                from   ITEM
                                where  ItemID = new.ItemID)
  where  OrdID = new.OrdID and ItemID = new.ItemID$;$ 
end;

create trigger TRG_DET_UPD_AmountDet
after update of AmountDet on DETAIL
for each row
begin
  update CUSTORDER 
  set    AmountOrd = (select sum(AmountDet)
                      from   DETAIL
                      where  OrdID = new.OrdID)
  where  OrdID = new.OrdID$;$ 
end;


-- ------------------------------------- --
-- Operations on table CUSTORDER         --
-- ------------------------------------- --

-- INSERT into CUSTORDER --
--        AmountOrd default 0;


-- DELETE from CUSTORDER --

create trigger TRG_COM_DEL
after delete on CUSTORDER
for each row
begin
  update CUSTOMER 
  set    AmountCust = coalesce((select sum(AmountOrd)
                                from   CUSTORDER
                                where  CustID = old.CustID),0)
  where  CustID = old.CustID$;$ 
end;


-- UPDATE CUSTORDER (CustID,AmountOrd) --

create temporary trigger TRG_COM_UPD_CustID
after update of CustID on CUSTORDER
for each row
begin
  update CUSTOMER 
  set    AmountCust = coalesce((select sum(AmountOrd)
                                from   CUSTORDER
                                where  CustID = old.CustID),0)
  where  CustID = old.CustID$;$ 
  --
  update CUSTOMER 
  set    AmountCust = (select sum(AmountOrd)
                       from   CUSTORDER
                       where  CustID = new.CustID)
  where  CustID = new.CustID$;$ 
end;

create temporary trigger TRG_COM_UPD_AmountOrd
after update of AmountOrd on CUSTORDER
for each row
begin
  update CUSTOMER 
  set    AmountCust = (select sum(AmountOrd)
                       from   CUSTORDER
                       where  CustID = new.CustID)
  where  CustID = new.CustID$;$ 
end;

-- ------------------------------------- --
-- Operations on table CUSTOMER          --
-- ------------------------------------- --

-- INSERT into CUSTOMER --
--        AmountCust default 0;


-- ------------------------------------- --
-- Loading raw data                      --
-- ------------------------------------- --

insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('B112','HANSENNE','23, r. Dumont','Poitiers','C1',1250);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('C123','MERCIER','25, r. Lematre','Namur','C1',-2300);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('B332','MONTI','112, r. Neuve','Genve','B2',0);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('F010','TOUSSAINT','5, r. Godefroid','Poitiers','C1',0);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('K111','VANBIST','180, r. Florimont','Lille','B1',720);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('S127','VANDERKA','3, av. des Roses','Namur','C1',-4580);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('B512','GILLET','14, r. de l''Et','Toulouse','B1',-8700);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('B062','GOFFIN','72, r. de la Gare','Namur','B2',-3200);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('C400','FERARD','65, r. du Tertre','Poitiers','B2',350);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('C003','AVRON','8, ch. de la Cure','Toulouse','B1',-1700);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('K729','NEUMAN','40, r. Bransart','Toulouse',null,0);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('F011','PONCELET','17, Cls des Erables','Toulouse','B2',0);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('L422','FRANCK','60, r. de Wpion','Namur','C1',0);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('S712','GUILLAUME','14a, ch. des Roses','Paris','B1',0);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('D063','MERCIER','201, bvd du Nord','Toulouse',null,-2250);
insert into CUSTOMER(CustID,Name,Address,City,Cat,Account) values
    ('F400','JACOB','78, ch. du Moulin','Bruxelles','C2',0);

insert into ITEM values ('CS262','CHEV. SAPIN 200x6x2',75,45);
insert into ITEM values ('CS264','CHEV. SAPIN 200x6x4',120,2690);
insert into ITEM values ('CS464','CHEV. SAPIN 400x6x4',220,450);
insert into ITEM values ('PA45' ,'POINTE ACIER 45 (1K)',105,580);
insert into ITEM values ('PA60' ,'POINTE ACIER 60 (1K)',95,134);
insert into ITEM values ('PH222','PL. HETRE 200x20x2',230,782);
insert into ITEM values ('PS222','PL. SAPIN 200x20x2',185,1220);

insert into CUSTORDER(OrdID,CustID,DateOrd) values ('30178','K111','2015-12-21');
insert into CUSTORDER(OrdID,CustID,DateOrd) values ('30179','C400','2015-12-22');
insert into CUSTORDER(OrdID,CustID,DateOrd) values ('30182','S127','2015-12-23');
insert into CUSTORDER(OrdID,CustID,DateOrd) values ('30184','C400','2015-12-23');
insert into CUSTORDER(OrdID,CustID,DateOrd) values ('30185','F011','2016-01-02');
insert into CUSTORDER(OrdID,CustID,DateOrd) values ('30186','C400','2016-01-02');
insert into CUSTORDER(OrdID,CustID,DateOrd) values ('30188','B512','2016-01-03');


write-b table CUSTOMER (no DETAIL data yet);
select * from CUSTOMER order by CustID;

write-b table ITEM (no DETAIL data yet);
select * from ITEM order by ItemID;

write-b table CUSTORDER (no DETAIL data yet);
select * from CUSTORDER order by OrdID;

write-b DETAIL table (no DETAIL data yet);
select * from DETAIL order by OrdID, ItemID;


insert into DETAIL(OrdID,ItemID,Qty) values ('30178','CS464',25);
insert into DETAIL(OrdID,ItemID,Qty) values ('30179','PA60',20);
insert into DETAIL(OrdID,ItemID,Qty) values ('30179','CS262',60);
insert into DETAIL(OrdID,ItemID,Qty) values ('30182','PA60',30);
insert into DETAIL(OrdID,ItemID,Qty) values ('30184','CS464',120);
insert into DETAIL(OrdID,ItemID,Qty) values ('30184','PA45',20);
insert into DETAIL(OrdID,ItemID,Qty) values ('30185','PA60',15);
insert into DETAIL(OrdID,ItemID,Qty) values ('30185','PS222',600);
insert into DETAIL(OrdID,ItemID,Qty) values ('30185','CS464',260);
insert into DETAIL(OrdID,ItemID,Qty) values ('30186','PA45',3);
insert into DETAIL(OrdID,ItemID,Qty) values ('30188','PA60',70);
insert into DETAIL(OrdID,ItemID,Qty) values ('30188','PH222',92);
insert into DETAIL(OrdID,ItemID,Qty) values ('30188','CS464',180);
insert into DETAIL(OrdID,ItemID,Qty) values ('30188','PA45',22);

write-b commitDB;
commitDB;

write-b -- ---------------------------------- --;
write   -- Initial state of the DB            --;
write   -- ---------------------------------- --;

write-b table CUSTOMER (DETAIL data loaded);
select * from CUSTOMER order by CustID;

write-b table ITEM (DETAIL data loaded);
select * from ITEM order by ItemID;

write-b table CUSTORDER (DETAIL data loaded);
select * from CUSTORDER order by OrdID;

write-b DETAIL table (DETAIL data loaded);
select * from DETAIL order by OrdID, ItemID;

write-b -- ---------------------------------- --;
write   -- UPDATE ITEM.UnitPrice              --;
write   -- ---------------------------------- --;

write-ab -- update ITEM set UnitPrice = -20 where ItemID = 'PA45' --;
update ITEM set UnitPrice = -20 where ItemID = 'PA45';

write-b table CUSTOMER;
select * from CUSTOMER order by CustID;

write-b table ITEM;
select * from ITEM order by ItemID;

write-b table CUSTORDER;
select * from CUSTORDER order by OrdID;

write-b table DETAIL;
select * from DETAIL order by OrdID, ItemID;

write-b abortDB;
abortDB;

write-b -- ---------------------------------- --;
write   -- UPDATE CUSTORDER.Qty               --;
write   -- ---------------------------------- --;

write;
write-ab -- update DETAIL set Qty = -100 where OrdID = '30188' and ItemID = 'PA45' --;
update DETAIL set Qty = -100 where OrdID = '30188' and ItemID = 'PA45';

write-b table CUSTOMER;
select * from CUSTOMER order by CustID;

write-b table ITEM;
select * from ITEM order by ItemID;

write-b table CUSTORDER;
select * from CUSTORDER order by OrdID;

write-b table DETAIL;
select * from DETAIL order by OrdID, ItemID;

write-b abortDB;
abortDB;

write-b -- ---------------------------------- --;
write   -- UPDATE DETAIL.ItemID               --;
write   -- ---------------------------------- --;

write;
write-b -- update DETAIL set ItemID = 'CS262' where ItemID = 'PA45' --;
update DETAIL set ItemID = 'CS262' where ItemID = 'PA45';

write-b table CUSTOMER;
select * from CUSTOMER order by CustID;

write-b table ITEM;
select * from ITEM order by ItemID;

write-b table CUSTORDER;
select * from CUSTORDER order by OrdID;

write-b table DETAIL;
select * from DETAIL order by OrdID, ItemID;

write-b abortDB;
abortDB;

write-b -- ---------------------------------- --;
write   -- UPDATE DETAIL.ItemID, OrdID,Qty    --;
write   -- ---------------------------------- --;

write;
write-b -- update DETAIL;
write   -- set OrdID = '30186',;
write   --     ItemID = 'PA60',;
write   --     Qty = 68;
write   -- where OrdID = '30185' and ItemID = 'PS222';

update DETAIL
set OrdID = '30186',
    ItemID = 'PA60',
    Qty = 68
where OrdID = '30185' and ItemID = 'PS222';

write-b table CUSTOMER;
select * from CUSTOMER order by CustID;

write-b table ITEM;
select * from ITEM order by ItemID;

write-b table CUSTORDER;
select * from CUSTORDER order by OrdID;

write-b table DETAIL;
select * from DETAIL order by OrdID, ItemID;

write-b abortDB;
abortDB;

write-b -- ---------------------------------- --;
write   -- UPDATE CUSTORDER.CustID            --;
write   -- ---------------------------------- --;

write;
write-b -- update CUSTORDER set CustID = 'C400' where CustID = 'B512' --;
update CUSTORDER set CustID = 'C400' where CustID = 'B512';

write-b table CUSTOMER;
select * from CUSTOMER order by CustID;

write-b table ITEM;
select * from ITEM order by ItemID;

write-b table CUSTORDER;
select * from CUSTORDER order by OrdID;

write-b table DETAIL;
select * from DETAIL order by OrdID, ItemID;

closeDB;

if (not $db$) deleteDB ORDERS-Extended.db;

write-b End of script;
return;
