-- ----------------------------------- --
-- Bases de donnes Blockchain         --
-- ----------------------------------- --

createOrReplaceDB BLOCK_CHAIN.db;

create table ACCOUNT(
   AccountID  varchar(64)   not null primary key,
   PrivName   varchar(64)   not null,
   SecretKey  varchar(360)  not null unique,
   RecTime    datetime      not null);

create table DIRECTORY(
   AccountID  varchar(64)   not null unique,
   PublicKey  varchar(1400) not null,
   Miner      integer       not null default 0);

create table BTRANSACTION(
   TransKey   integer      not null primary key autoincrement,
   TransID    varchar(72 ) not null unique,
   Bhash      varchar(48)  references BLOCK(Bhash),
   Operation  char(12)     not null,
   Source     varchar(48),
   Recip      varchar(48),
   Amount     decimal(16,10),
   RecTime    datetime     not null,
   Signature  varchar(512) not null);

create table BLOCK(
   BlockKey   integer      not null primary key autoincrement,
   Bhash      varchar(48)  not null unique,
   Transact   integer      not null,
   Nonce      varchar(32),
   RecTime    datetime     not null,
   Status     integer      not null default 0,
   Bprev      varchar(48) references BLOCK(Bhash));

create view BTRANSACTION_PRIV(TransKey,TransID,Bhash,Operation,Source,Recip,Amount,RecTime,Signature) as
   select TransKey,TransID,Bhash,Operation,S.PrivName as "Source",R.PrivName as "Recip",Amount,T.RecTime,Signature
   from   BTRANSACTION T left outer join ACCOUNT S on(S.AccountID = T.Source)
                         left outer join ACCOUNT R on(R.AccountID = T.Recip);

closeDB;

write-b Databases BLOCK_CHAIN.db created.;
