-- ----------------------- --
-- Examine transactions    --
-- ----------------------- --

openDB BLOCK_CHAIN.db;

write-b Transactions;
select Source,Operation,Amount,Recipient,TimeCreated,InBlock
from (
   select S.PrivName as Source,operation,null as Amount,null as Recipient,
          T.RecTime as TimeCreated,case when Bhash is null then null else 'yes' end as InBlock
   from   ACCOUNT S, BTRANSACTION T
   where  T.Source = S.AccountID
   and    operation = 'REGISTER'
      union all
   select null as Source,operation,Amount,M.PrivName as Recipient,
          T.RecTime as TimeCreated,case when Bhash is null then null else 'yes' end as InBlock
   from   BTRANSACTION T, ACCOUNT as M
   where  T.Recip = M.AccountID
   and    operation = 'DEPOSIT'
      union all
   select S.PrivName as Source,operation,Amount,R.PrivName as Recipient,
          T.RecTime as TimeCreated,case when Bhash is null then null else 'yes' end as InBlock
   from   ACCOUNT S, BTRANSACTION T, ACCOUNT R
   where  T.Source = S.AccountID
   and    T.Recip  = R.AccountID
   and    operation = 'TRANSFER'
      )
order by TimeCreated;

write-b Account balance;
select Account, sum(plusD) as Deposit,
                sum(plusT) as Received,
               -sum(minus) as Transfered,
       sum(plusD) + sum(plusT) - sum(minus) as Balance
from (
   select M.PrivName as Account,Amount as plusD,0 as plusT,0 as minus
   from   BTRANSACTION T, ACCOUNT M
   where  T.Recip = M.AccountID
   and    operation = 'DEPOSIT'
      union all
   select M.PrivName as Account,0 as plusD,Amount as plusT,0 as minus
   from   BTRANSACTION T, ACCOUNT M
   where  T.Recip = M.AccountID
   and    operation = 'TRANSFER'
      union all
   select M.PrivName as Account,0 as plusD,0 as plusT,Amount as minus
   from   BTRANSACTION T, ACCOUNT M
   where  T.Source = M.AccountID
   and    operation = 'TRANSFER'   
      )
group by Account;
write;

closeDB;
