
-- --------------------------------- --
-- Compare two directory instances   --
-- --------------------------------- --

if (not fileExists('dbDirectory/DIRECTORY.db'));
   showMessage Database "DIRECTORY.db" not found;
   return;
endif;

openDB DIRECTORY.db;

set sub, comp = 0, 1;
askCombo-u [/bSelect the directory instances to compare]
           [ask inst1,inst2 = Instance 1:[!$listInstances$]
                             |Instance 2:[!$listInstances$]]
          |[selectMany sub = Compare subdirectories in these directories]
          |[selectOne-u comp = {Type of comparison}
                              |on Full name and Type
                              |on Full name, Type, Size and Last update];
if ('$DIALOGbutton$' = 'Cancel' or '$inst1$' = '' or '$inst2$' = '')
   goto EXIT;

create temp table NODE1 as select * from NODE where InstID = $inst1$;
create temp table NODE2 as select * from NODE where InstID = $inst2$;

if ($sub$);
   ask dir1,dir2 = Subdirectory 1:[!select FullName from NODE1 where  NType = 'D']
                  |Subdirectory 2:[!select FullName from NODE2 where  NType = 'D'];
   if ('$DIALOGbutton$' = 'Cancel' or '$dir1$' = '' or '$dir2$' = '')
      goto EXIT;

   delete from NODE1 where not startswith(Prefix,'dir1');
   delete from NODE2 where not startswith(Prefix,'dir2');

   extract tim1 = select InstDate||' '||InstTime from INSTANCE where InstID = $inst1$;
   extract tim2 = select InstDate||' '||InstTime from INSTANCE where InstID = $inst2$;
else;
   extract dir1,tim1 = select DirName,InstDate||' '||InstTime from DIRECTORY D, INSTANCE I
                       where  D.DirID = I.DirID and InstID = $inst1$;
   extract dir2,tim2 = select DirName,InstDate||' '||InstTime from DIRECTORY D, INSTANCE I
                       where  D.DirID = I.DirID and InstID = $inst2$;
endif;

-- "NODE1" and "NODE2" contains the NODE rows of the directories to compare --
-- "dir1" and "dir2" contains the "FullName" of the directories to compare --

set maxSelectWidth = 100;

set title1 = Comparing Directory (1) "$dir1$ ($tim1$)";
set title2 = @S5with Directory (2) "$dir2$ ($tim2$)";
compute lenLine = max(length('title1'),length('title2'));
compute subl = repeat('-',$lenLine$);


if ($comp$ = 1);
   write-b -- -------------------------------------- --;
   write   -- Comparison based on Name and Type      --;
   write-a -- -------------------------------------- --;

   write   -- Directory (1): $dir1$ [$tim1$];
   write-a -- Directory (2): $dir2$ [$tim2$];

   select NType as "Type",
          case when NType = 'F'
               then ' '||Name
               else '['||Name||']'
          end as "In Directory (1) but not in Directory (2)"
   from (select replace(FullName,'dir1/','') as Name,NType from NODE1
            except
         select replace(FullName,'dir2/','') as Name,NType from NODE2);

   write;
   select NType as "Type",
          case when NType = 'F'
               then ' '||Name
               else '['||Name||']'
          end as "In Directory (2) but not in Directory (1)"
   from (select replace(FullName,'dir2/','') as Name,NType from NODE2
            except
         select replace(FullName,'dir1/','') as Name,NType from NODE1);
endif;

if ($comp$ = 2);
   write-b -- --------------------------------------------------------- --;
   write   -- Comparison based on Name, Type, Size and Last update      --;
   write-a -- --------------------------------------------------------- --;

   write   -- Directory (1): $dir1$ [$tim1$];
   write-a -- Directory (2): $dir2$ [$tim2$];

   select NType as "Type",
          case when NType = 'F'
               then ' '||FullName
               else '['||FullName||']'
          end as "In Directory (1) but not (or modified) in Directory (2)",
          frame(thousandSep(Size,' '),13,'>',' ') as "Size",
          LastUpdate
   from (select FullName,NType,Size,LastUpdate from NODE1
            except
         select FullName,NType,Size,LastUpdate from NODE2);

write;
select NType as "Type",
       case when NType = 'F'
            then ' '||FullName
            else '['||FullName||']'
       end as "In Directory (2) but not (or modified) in Directory (1)",
       frame(thousandSep(Size,' '),13,'>',' ') as "Size",
       LastUpdate
from (select FullName,NType,Size,LastUpdate from NODE2
         except
      select FullName,NType,Size,LastUpdate from NODE1);
endif;

label EXIT;
   closeDB;
   set maxSelectWidth = 80;

return;