-- ---------------------------------------- --
-- Load Hash values of a Directory instance --
-- ---------------------------------------- --

if (not fileExists('dbDirectory/DIRECTORY.db'));
   showMessage Database "DIRECTORY.db" not found;
   return;
endif;

openDB DIRECTORY.db;

-- Select an instance --
set inst = ;
ask-u inst = Instance:[!$listInstances$];
if ('$DIALOGbutton$' = 'Cancel' or '$inst$' = '') goto EXIT;

-- Compute and store Hash values --
start-timer;
with H(Node,Val) 
     as (select NodeID,hashFile(FullName)
         from   NODE 
         where  InstID = $inst$
         and    NType = 'F')
update NODE
set Hash = (select Val from H where Node = NODE.NodeID)
where  InstID = $inst$;

commitDB;

-- Write metrics --
set htim = $timer$;
extract hcnt,hsiz = select count(*), sum(Size) from NODE where InstID = $inst$;
compute hcnt = thousandsep('$hcnt$',',');
compute hsiz = thousandsep('$hsiz$',',');
write-ab Hash values of $hcnt$ files ($hsiz$ B) loaded in $timer$ s.;

label EXIT;
   closeDB;

return;
