-- --------------------------------------------- --
-- Load an instance in DIRECTORY database        -- 
-- --------------------------------------------- --

if (not fileExists('dbDirectory/DIRECTORY.db'));
   showMessage Database "DIRECTORY.db" not found;
   return;
endif;

set path = D:/;

selectDirectory path = $path$;
if ('path' = '') return;

set dat = $date$;
compute tim = substr('$time$',1,8);


-- Extract name list in directory "Files/names.txt" --
-- ------------------------------------------------ --
set namesFile  = $fileDirectory$/names.txt;
dirFileNamesAll-r $namesFile$ = $path$;

openDB DIRECTORY.db;

-- Get root directory nickname --
-- --------------------------- --
set rnn = $path$;
ask-u rnn = [/bSelect/Enter the nickname of the root directory] 
             Root directory nickname:[select DirNick from DIRECTORY];
if ('$DIALOGbutton$' = 'Cancel') goto EXIT;

start-timer;

-- Insert DIRECTORY if not exists and get Directory ID --
-- --------------------------------------------------- --
extract dirID = select DirID from DIRECTORY where DirNick = 'rnn';
if ('$SQLdiag$' = 'NONE');
   insert into DIRECTORY(DirName,DirNick) values ('path','rnn');
   extract dirID = select DirID from DIRECTORY where DirNick = 'rnn';
endif;

-- Insert INSTANCE and get Instance ID --
-- ----------------------------------- --
insert into INSTANCE(DirID,InstDate,InstTime) values ($dirID$,'$dat$','$tim$');
extract instID = select InstID from INSTANCE 
                 where  DirID = $dirID$
                 and    InstDate = '$dat$'
                 and    InstTime = '$tim$';

-- Insert NODEs --
-- ------------ --

drop index if exists ndxFullName;
drop index if exists ndxParentID;

onError continue;
set NN = 0;
for line = [file $namesFile$];
   compute NN = $NN$ + 1;

   function ful,pre,dir,fil,ext = LFile:splitPath {$line$};

   if ('fil' = '');
      insert into NODE(InstID,NType,FullName,Prefix,LocalName)
                values($instID$,'D','ful','pre','dir');
   else;
      function siz,cre,upd,acc = LFile:infoFile {$line$};
      insert into NODE(InstID,NType,FullName,Prefix,LocalName,Extension,
                       Size,LastUpdate,LastAccess)
                values($instID$,'F','ful','pre','fil','ext',
                       $siz$,'$upd$','$acc$');
      if ('$SQLdiag$' <> 'OK');
         write Insert error: $SQLdiag$ - [$EXTENDEDdiag$].;
         write insert into NODE values($instID$,'F','ful','pre','fil','ext',
                       $siz$,'$upd$','$acc$');

      endif; 
   endif;
endfor;
onError stop;
write $NN$ nodes from $path$ loaded in $timer$ s.;

onError continue;
create index ndxFullName on NODE(InstID,FullName);
   if ('$SQLdiag$' <> 'OK')
      write Create index error: $SQLdiag$ - [$EXTENDEDdiag$].;
create index ndxParentID on NODE(ParentID);
   if ('$SQLdiag$' <> 'OK')
      write Create index error: $SQLdiag$ - [$EXTENDEDdiag$].;
onError stop;

start-timer;

-- Compute "Level" --
-- --------------- --
compute NL = countInst('path','/');
update NODE set Level = countInst(FullName,'/') - $NL$
where InstID = $instID$;

-- Compute "ParentID" --
-- ------------------ --
update NODE
set    ParentID = (select N1.NodeID
                   from   NODE N1
                   where  N1.FullName  = NODE.Prefix
                   and    N1.InstID  = $instID$
                   )
where InstID = $instID$;
 
-- Compute "Path" (nd = number of digits of "NN") --
-- ---------------------------------------------- --
extract nd = select length(cast(max(NodeID) as char)) from NODE;

create temp table GRAPH as
select ParentID,NodeID,NType,Level,Path from NODE where InstID = $instID$;

create index ndxGRAPH1 on GRAPH(ParentID);
create index ndxGRAPH2 on GRAPH(NodeID);

extract maxLevel = select max(Level) from GRAPH; 
update GRAPH set Path = 'D'||frame(NodeID,$nd$,'>','0') where Level = 0;
for iLev = [1,$maxLevel$];
   update GRAPH
   set Path = (select G.Path from GRAPH G where G.NodeID = GRAPH.ParentID)
                    ||' '||NType||frame(NodeID,$nd$,'>','0')
   where Level = $iLev$;
endfor;

update NODE
set    Path = (select Path from GRAPH where NodeID = NODE.NodeID)
where  InstID = $instID$;

write Table NODE updated in $timer$ s.;

commitDB;

label EXIT;
  closeDB;
  deleteFile namesFile;
  return;

