### -*- coding: UTF-8 -*-

# LIFE GAME in Python

# See also:
#   http://code.activestate.com/recipes/578928-game-of-life-python-34-tkinter/?in=lang-python

import datetime as dt

def initArray(nl,nc,v):
    # Return an array of 'nl'+2 rows and 'nc'+2 columns
    # Set values to 'v'
    a = []
    for L in range(0,nl+2):
        line = [v for C in range(0,nc+2)]
        a.append(line)
    return a

def setElem(a,l,c,v):
    # Set element ('c','l') to value 'v' in array 'a'
    # Do nothing if coordinates out ot range
    global nL,nC
    if c not in range(0,nC+2):
        #print 'Index range error: C =',c
        return
    if l not in range(0,nL+2):
        #print 'Index range error: L =',l
        return
    a[l][c] = v
    return

def getElem(a,l,c):
    # Return value of element ('c','l') of array 'a'
    # Return 0 if coordinates out ot range
    global nL,nC
    #print '   ',nL,nC,'-',l,c
    if c not in range(0,nC+2):
        elem = 0
    if l not in range(0,nL+2):
        elem = 0
    return a[l][c]

def printCELL():
    # Print array 'cell', row by row, on the console
    global CELL
    nl = len(CELL)
    nc = len(CELL[0])
    for L in range(nl):
        for C in range(nc):
            print 'X' if CELL[L][C] else '.',
        print
    print
    return

def printSCORE():
    # Print array 'score', row by row, on the console
    global SCORE
    nl = len(SCORE)
    nc = len(SCORE[0])
    for L in range(nl):
        for C in range(nc):
            print SCORE[L][C],
        print
    print
    return

def createAutomaton(auto):
    # Return an array for automaton 'auto'
    # and set simulation parameter nG
    # The array and simulation parameters are specified
    # in auto[0]
    global nL,nC,nG
    nL,nC,nG = auto[0]
    cell = initArray(nL,nC,0)
    for el in auto[1:]:
        L = el[0]
        for C in el[1]:
            setElem(cell,L,C,1)
    return cell

def computeNeighbours():
    # Returns array 'score' with the sum of the neighbours
    # of each element of 'a'
    global CELL,SCORE,nL,nC
    global minL,minC,maxL,maxC
    
    SCORE = initArray(nL,nC,0)
    
    minL = nL
    minC = nC
    for L in range(1,nL+1):
        for C in range(1,nC+1):
            if getElem(CELL,L,C) > 0:
                minL = min(minL,L)
                minC = min(minC,C)

    maxL = 0
    maxC = 0
    for L in range(nL,0,-1):
        for C in range(nC,0,-1):
            if getElem(CELL,L,C) > 0:
                maxL = max(maxL,L)
                maxC = max(maxC,C)

    #print 'Bounding box:',minL,minC,'-',maxL,maxC,'\n'

    for L in range(minL-1,maxL+2):
        for C in range(minC-1,maxC+2):
            sumV = - getElem(CELL,L,C)
            for l in range(L-1,L+2):
                for c in range(C-1,C+2):
                    #print '    - cell[',l,c,'] =',getElem(cell,l,c)
                    sumV += getElem(CELL,l,c)
            #print '  ',L,C,'=',sumV
            setElem(SCORE,L,C,sumV)
    return

def computeNextGeneration():
    # Returns in 'cell' the next generation computed from array 'score'.
    global CELL,SCORE,nL,nC
    global minL,minC,maxL,maxC

    for L in range(minL-1,maxL+2):
        for C in range(minC-1,maxC+2):
            if getElem(SCORE,L,C) == 3:
                setElem(CELL,L,C,1)
            elif getElem(SCORE,L,C) == 2:
                pass
            else:
                setElem(CELL,L,C,0)
    return

def computeNextStep():
    global compTime
    stepStart = dt.datetime.now()
    computeNeighbours()
    #printSCORE()
    computeNextGeneration()
    #printCELL()
    stepElapsed = (dt.datetime.now() - stepStart).total_seconds()
    compTime += stepElapsed
    return

def generateGrid(F):
    # Writes in file 'F' the first SQLdraw commands
    # of the simulation (header + grid)
    F.write('area %s,%s\n' % (nC*12,nL*12))
    
    F.write('polyline "Vgrid",1,"gray80",""\n')
    maxL = nL * 12
    for C in range(0,nC+1):
        F.write('%s,%s\n' % (C*12,maxL))
        F.write('%s,%s\n' % (C*12,0))
        F.write('%s,%s\n' % (C*12,maxL))

    F.write('polyline "Hgrid",1,"gray80",""\n')
    maxC = nC * 12
    for L in range(0,nL+1):
        F.write('%s,%s\n' % (maxC,L*12))
        F.write('%s,%s\n' % (0,L*12))
        F.write('%s,%s\n' % (maxC,L*12))
    return

def generateDrawing(F,G):
    # Write in file 'F' the SQLdraw commands for the current
    # contents of 'cell' ('G' is the generation number)
    global CELL,nL,nC
    if G > 0:
        F.write('delete last\n')
    F.write('points "life-'+str(G)+'",1,"black","black",23\n')
    for L in range(1,nL+1):
        LL = L*12 - 6
        for C in range(1,nC+1):
            if getElem(CELL,L,C) == 1:
                CC = C*12 - 6
                F.write('%s,%s,""\n' % (CC,LL))
    F.write('wait '+str(wait)+'\n')
    return

def closeDraw(F):
    F.close()
    return

# =========== MAIN ==============

# automaton = [[param],[line1],[line2],..,[lineN]]
# param = nL,nC,nG
# line = [line#,[col#,col#,...]]

glider = [(60,60,100),
    [2,[3]],
    [3,[4]],
    [4,[2,3,4]]]

waveGuide = [(25,100,100),
    [3,[4,5]],
    [4,[3,6,10,14,18,22,26,30,34,38,42,46,50,54,58,62,66,70]],
    [5,[7,9,11,13,15,17,19,21,23,25,27,29,31,33,35,37,39,41,43,45,47,49,51,53,55,57,59,61,63,65,67,69,71]],
    [6,[3,6,10,14,18,22,26,30,34,38,42,46,50,54,58,62,66,70]],
    [7,[4,5,6]],
    [8,[5,6]],
    [9,[5,6]],
    [10,[6]],
    [11,[6]],
    [12,[6]],
    [13,[6]],
    [14,[6]],
    [15,[6]],
    [16,[6]],
    [17,[5,6]],
    [18,[5,6]],
    [19,[4,5,6]],
    [20,[3,6,10,14,18,22,26,30,34,38,42,46,50,54,58,62,66,70]],
    [21,[7,9,11,13,15,17,19,21,23,25,27,29,31,33,35,37,39,41,43,45,47,49,51,53,55,57,59,61,63,65,67,69,71]],
    [22,[3,6,10,14,18,22,26,30,34,38,42,46,50,54,58,62,66,70]],
    [23,[4,5]]]
  
# Array and simulation parameters (default)
A = []      # Array of the current generation of the automaton
nL = 10     # number of rows (lines) of the array
nC = 10     # number of columns of the array
nG = 0      # number of generations of the simulation

wait = 150

print '\nStarting'
# Create an automaton (give the name of the automaton)
CELL = createAutomaton(glider)

# Print array CELL
#printCELL()

start = dt.datetime.now()

F = open('LIFEGAME-draw.draw','w')
generateGrid(F)
generateDrawing(F,0)
compTime = 0.0
for Gen in range(1,nG):
    computeNextStep()
    generateDrawing(F,Gen)
    #print 'Generation',Gen+1
closeDraw(F)

elapsed = (dt.datetime.now() - start).total_seconds()

print '\nComputing time =',compTime
print '\nTotal time =',elapsed
