
-- ---------------------------------------------------------- --
-- SQL Tutor - The student application                        --
-- ---------------------------------------------------------- --

set dir =  $scriptDirectory$/Case-Studies/Case_SQL_Interpreters/SQL-Tutor;

-- Select the exercise module that will be loaded --
-- ---------------------------------------------- --

-- Build a list of module names                   --
fileNames-s $fileDirectory$/tutorFiles.txt = $dir$,sql;
set tList = ;
for line = [file $fileDirectory$/tutorFiles.txt]
  if (startsWith('$line$','SQL-Exercises-'))
     compute tList = '$tList$'||','||replace('$line$','.sql','');
compute tList = trim('$tList$',',');
deleteFile $fileDirectory$/tutorFiles.txt;

-- Then let the user select a module  --
set exerciseModule = ;
ask exerciseModule = [/bSQL Tutor: Learning SQL at your own pace
                      @nSelect an exercise module.] 
                     Module:[!($tList$)];
if ('$DIALOGbutton$' = 'Cancel' or '$exerciseModule$' = '') stop;

set exerciseModule = $exerciseModule$.sql;

-- -------------------------------------------------------- --
-- Create "SQLtutor.db database and load an exercise module --
-- -------------------------------------------------------- --

if (not fileExists('$dir$/exerciseModule'));
   showMessage @nExercise module "$exerciseModule$" not found@n;
   stop;
endif;

execSQL $dir$/_Create-SQL-Tutor.sql;

openDB SQLtutor.db;
   execSQL $dir$/$exerciseModule$;
closeDB;


-- -------------------------------------------------------- --
-- Start the student activity                               --
-- -------------------------------------------------------- --

outputAppend window;

set entryWidth2 = 48;

execSQL $dir$/_Get-parameters.sql;
   if ('$DIALOGbutton$' = 'Cancel') stop;

if ($help$ = 1) execSQL $dir$/_Display-exercises-instructions.sql;

set oper = 0;

set xId = ;
set curPos = 0;
set clear   = 0;

while (True);
   -- ---------------------------- --
   -- Select next current exercise --
   -- ---------------------------- --

   set xQuery  = ;
   set xResult = ;
   set xDiag   = ;

   execSQL $dir$/_Select-an-exercise.sql;
      if ('$DIALOGbutton$' = 'Cancel' or 'xId' = '') exit;

   -- Ask for an answer --
   execSQL $dir$/_Submit-an-answer.sql;
      if ('$DIALOGbutton$' = 'Cancel') next;

   if (not fileExists('$dbDirectory$/$xDB$'));
      showMessage Tutor database "$xDB$" does not exist.;
      next;
   endif;

   while (True);
      -- --------------------------------------- --
      -- Next evaluation of the current exercise --
      -- --------------------------------------- --

      openDB $xDB$;

      if ($clear$ = 0) set mode = Append;
      if ($clear$ = 1) set mode = Open;

      -- ------------------------------ --
      -- 1. Execute student's query     --
      -- ------------------------------ --
      if ($oper$ = 1);
         execSQL $dir$/_Execute-student-query.sql;
         closeDB;
         if ('$DIALOGbutton$' = 'Cancel') exit;
         next;
      endif;

      -- ------------------------------ --
      -- 2. Execute reference query     --
      -- ------------------------------ --
      if ($oper$ = 2);
         execSQL $dir$/_Execute-reference-query.sql;
         closeDB;
         if ('$DIALOGbutton$' = 'Cancel') exit;
         next;
      endif;

      -- ------------------------------ --
      -- 3. Evaluate student's query    --
      -- ------------------------------ --
      if ($oper$ = 3);
         set status = 0;
         execSQL $dir$/_Compute-metrics.sql;
         -- closeDB; ----------------------

         if ($status$ = 0);
            execSQL $dir$/_Create-diagnostic-EN.sql;
         endif;

         output$mode$ xResult.var;
            write EXERCICE [$xId$] - EVALUATION OF YOUR QUERY:;
            write-a $xDiag$;
         outputAppend window;

         execSQL $dir$/_Display-result-and-correct.sql;

         closeDB;

         if ($status$ = 0) exit; 
         if ('$DIALOGbutton$' = 'Cancel') exit;
      endif;

   endwhile;

endwhile;

closeHelp;

deleteFile-e $dir$/SQLtutor.tuto;

showMessage @nSESSION "$xCat$ - Level $xLev$" COMPLETED@n;

stop;
