openDB CLICOM.db;

set maxSelectWidth = 80;
set dynaColumns = 1;

create temp table CLI(NCLI char(10) not null,
                      NOM  char(20) not null);

write-b Table "CLI" as defined by "create table" statement;
select * from CLI;

insert into CLI(NCLI,NOM,!LOCALITE) values ('B112','Pierre','Poitiers');
insert into CLI(NCLI,NOM,!LOCALITE) values ('C123','Anne','Toulouse');
insert into CLI(NCLI,NOM,!LOCALITE) values ('B332','Marie','Genve');
insert into CLI(NCLI,NOM,!LOCALITE) values ('F010','Tony','Poitiers');
insert into CLI(NCLI,NOM,!LOCALITE) values ('K111','John','Lille');
insert into CLI(NCLI,NOM,!LOCALITE) values ('S127','Isa','Toulouse');
insert into CLI(NCLI,NOM,!LOCALITE) values ('B512','Sophie','Poitiers');

extract dCol = select '!'||group_DColNames(',!') from CLI;
select NCLI,NOM,$dCol$ from CLI;

create temp view if not exists V(ID,Name,City) as select CLI.NCLI,CLI.NOM,!LOCALITE from CLI;

write-b Table "CLI" after inserting dynamic values of "LOCALITE";
select * from V;

-- Querying dynamic (hybrid) table "CLI" --
write-b Table "CLI" in which columns "CAT" and "COMPTE" do not exist;
select NCLI,NOM,!LOCALITE,!CAT,!COMPTE from CLI;
select * from CLI;

write-b Loading static table "CLI2" from dynamic table "CLI" (no target column list);
create temp table CLI2(NCLI,NOM,LOCALITE,CAT);
insert into CLI2 select NCLI,NOM,!LOCALITE,!CAT from CLI;
select NCLI,NOM,LOCALITE,CAT from CLI2;

write-b Loading static table "CLI2" from dynamic table "CLI" (with target column list);
delete from CLI2;
insert into CLI2(NCLI,NOM,LOCALITE,CAT) select NCLI,NOM,!LOCALITE,!CAT from CLI;
select NCLI,NOM,LOCALITE,CAT from CLI2;

write-b Updating dynamic column "CAT" in "CLI";
update CLI set !CAT = 'B1' where NCLI = 'F010';
select NCLI,!NOM,!LOCALITE,!CAT from CLI;

write-b The most complex pattern: loading dynamic table "CLI3" from dynamic table "CLI";
--
write-b -- Table "CLI3": native dynamic expression;
create temp table CLI3(NCLI);
insert into CLI3(NCLI,!NOM,!LOCALITE,!CAT,!COMPTE,!BONUS)
            select NCLI,NOM,!LOCALITE,!CAT,1000,'YES'
            from   CLI
            where !LOCALITE = 'Poitiers';
select NCLI,!NOM,!LOCALITE,!CAT,!COMPTE,!BONUS from CLI3;
select * from CLI3;

-- ... must provide the same result as this one,
--     which has precompiled:
write-b -- Table "CLI3": Precompiled version;
drop table CLI3;
create temp table CLI3(NCLI,DColumns);
insert into CLI3(NCLI,DColumns)
select NCLI,
      '{'
          ||coalesce(' "NOM":"'     || NOM       || '"', '')
          ||coalesce(',"LOCALITE":"'|| !LOCALITE || '"', '')
          ||coalesce(',"CAT":"'     || !CAT      || '"', '')
          ||coalesce(',"COMPTE":'   || '1000'          , '')
          ||coalesce(',"BONUS":"'   || 'YES'     || '"', '')
       ||'}' 
from   CLI 
where !LOCALITE = 'Poitiers';
select NCLI,!NOM,!LOCALITE,!CAT,!COMPTE,!BONUS from CLI3;

-- ... and also the same as this one, in pure SQL,
--     which has ben completely compiled:
write-b -- Table "CLI3": fully ccompiled version;
drop table CLI3;
create temp table CLI3(NCLI,DColumns);
insert into CLI3(NCLI,DColumns) 
select NCLI,
       '{'
          || coalesce(' "NOM":"'      || NOM                          || '"', '')
          || coalesce(',"LOCALITE":"' || getDCol(DColumns,'LOCALITE') || '"', '')
          || coalesce(',"CAT":"'      || getDCol(DColumns,'CAT')      || '"', '')
          || coalesce(',"COMPTE":'    || '1000'                             , '')
          || coalesce(',"BONUS":"'    || 'YES'                        || '"', '')
      || '}' 
from   CLI 
where getDCol(DColumns,'LOCALITE') = 'Poitiers';
select NCLI,!NOM,!LOCALITE,!CAT,!COMPTE,!BONUS from CLI3;

--
write-b -- Table "CLI3" loaded through complex source expressions;
drop table CLI3;
create temp table CLI3(NCLI);
insert into CLI3(NCLI,!NOM,!LOCALITE,!CAT,!COMPTE,!BONUS)
       select NCLI,
              NOM,
              !LOCALITE,
              lower(NOM)||' ['||!CAT||']'||(select '('||upper(!LOCALITE)||')'
                                            from   CLI2
                                            where  CLI2.NCLI = C.NCLI),
              1000,
              'YES'
       from   CLI C
       where !LOCALITE = 'Poitiers';
select NCLI,!NOM,!LOCALITE,!CAT,!COMPTE,!BONUS from CLI3;
select * from CLI3;

write-b -- Update table "CLI" from table "CLIENT";
update CLI set !COMPTE = (select COMPTE from CLIENT where NCLI = CLI.NCLI),
               !CAT    = (select CAT from CLIENT where NCLI = CLI.NCLI)
where !LOCALITE = 'Poitiers';
select NCLI,NOM,!LOCALITE,!CAT,!COMPTE from CLI;
select * from CLI;

write-b Extracting dynamic column names;
select NCLI,'NCLI,NOM,'||getDColNames(',') as Colonnes from CLI;

select 'NCLI,NOM,'||group_DColNames(',') as Colonnes from CLI;

write-b Create a dynamic view of an unknown dynamic table;
createDictionary;
extract staticCol = select group_concat(ColName,',') from SYS_TABLE T,SYS_COLUMN C 
                    where  T.TableID = C.TableID and TableName = 'CLI'
                    and ColName <> 'DColumns';
extract dynamicCol = select '!'||group_DColNames(',!') from CLI;
--extract dynamicCol = select '!'||group_DColNames(DColumns,',!') from CLI;
create temp view VCLI as select $staticCol$,$dynamicCol$ from CLI;
select * from VCLI;

closeDB;
