-- --------------------------- --
-- The Column-oriented storage --
-- --------------------------- --

-- Augmentation factor for table CUSTOMER --
-- N = 2: native data
-- N = 1: x2
-- N = 2: x4
-- N = 3: x8
set N = 3;

write-b -- ---------------------------------- --;
write   -- COLUMN-ORIENTED version with PK Id --;
write   -- ---------------------------------- --;

copyFile-r $dbDirectory$/ORDERS.db,$dbDirectory$/TORDERS.db;
openDB TORDERS.db;

-- Columns for table CUSTOMER --
create temp table CUST_NAME(
   CustID    char(10) not null primary key,
   Value     varchar(32) not null);
create temp table CUST_ADDRESS(
   CustID    char(10) not null primary key,
   Value     char(60) not null);
create temp table CUST_CITY(
   CustID    char(10) not null primary key,
   Value     char(30) not null);
create temp table CUST_CAT(
   CustID    char(10) not null primary key,
   Value     char(2));
create temp table CUST_ACCOUNT(
   CustID    char(10) not null primary key,
   Value     decimal(9,2) not null);

-- Augment the size of table CUSTOMER --
set prefix = ABCDEFGHIJKLMNOPQRSTUVWXYZ;
for I = [1,$N$];
  insert into CUSTOMER
  select item('$prefix$',$I$,'')||CustID,Name,Address,City,Cat,Account
  from CUSTOMER;
endfor;

-- Load column-tables for CUSTOMER --
-- ------------------------------- --
start-timer;

insert into CUST_NAME(CustID,Value)    select CustID,Name from CUSTOMER;
insert into CUST_ADDRESS(CustID,Value) select CustID,Address from CUSTOMER;
insert into CUST_CITY(CustID,Value)    select CustID,City from CUSTOMER;
insert into CUST_CAT(CustID,Value)     select CustID,Cat from CUSTOMER;
insert into CUST_ACCOUNT(CustID,Value) select CustID,Account from CUSTOMER;

compute elapsed = int($timer$*1000);
write @S3Load column-tables for CUSTOMER: $elapsed$ ms.;


-- Reconstruction of table CUSTOMER --
-- -------------------------------- --
start-timer;

create temp table M_CUSTOMER as
select C1.CustID,C1.Value as Name,C2.Value as Address,
       C3.Value as City,C4.Value as Cat,C5.Value as Account
from CUST_NAME C1,CUST_ADDRESS C2,CUST_CITY C3,CUST_CAT C4,CUST_ACCOUNT C5
where C2.CustID = C1.CustID
and   C3.CustID = C1.CustID
and   C4.CustID = C1.CustID
and   C5.CustID = C1.CustID;

compute elapsed = int($timer$*1000);
write @S3Reconstruction of CUSTOMER table: $elapsed$ ms.;

extract NC = select count(*) from CUSTOMER;
write @S3Number of M_CUSTOMER rows: $NC$;

closeDB;

write-b -- ---------------------------------------- --;
write   -- COLUMN-ORIENTED version with abstract Id --;
write   -- ---------------------------------------- --;

copyFile-r $dbDirectory$/ORDERS.db,$dbDirectory$/TORDERS.db;
openDB TORDERS.db;

-- Table CUSTOMER --
-- -------------- --

-- Augment the size of table CUSTOMER --
set prefix = ABCDEFGHIJKLMNOPQRSTUVWXYZ;
for I = [1,$N$];
  insert into CUSTOMER
  select item('$prefix$',$I$,'')||CustID,Name,Address,City,Cat,Account
  from CUSTOMER;
endfor;

create temp table CUST_CUSTID(
   Entity    integer      not null primary key autoincrement,
   Value     char(10)     not null unique
   -- The "unique" constraint translates the uniqueness property of PK --
   );
create temp table CUST_NAME(
   Entity    integer      not null primary key,
   Value     varchar(32)  not null);
create temp table CUST_ADDRESS(
   Entity    integer      not null primary key,
   Value     char(60)     not null);
create temp table CUST_CITY(
   Entity    integer      not null primary key,
   Value     char(30)     not null);
create temp table CUST_CAT(
   Entity    integer      not null primary key,
   Value     char(2));
create temp table CUST_ACCOUNT(
   Entity    integer      not null primary key,
   Value     decimal(9,2) not null);


-- Load column-tables for CUSTOMER --
-- ------------------------------- --
start-timer;

-- First assign abstract Id to primary key values --
insert into CUST_CUSTID(Value) select CUSTID from CUSTOMER order by CUSTID;

insert into CUST_NAME(Entity,Value)
  select Entity,Name from CUST_CUSTID I, CUSTOMER C where I.Value = C.CustID;
insert into CUST_ADDRESS(Entity,Value)
  select Entity,Address from CUST_CUSTID I, CUSTOMER C where I.Value = C.CustID;
insert into CUST_CITY(Entity,Value)
  select Entity,City from CUST_CUSTID I, CUSTOMER C where I.Value = C.CustID;
insert into CUST_CAT(Entity,Value)
  select Entity,Cat from CUST_CUSTID I, CUSTOMER C where I.Value = C.CustID;
insert into CUST_ACCOUNT(Entity,Value)
  select Entity,Account from CUST_CUSTID I, CUSTOMER C where I.Value = C.CustID;

compute elapsed = int($timer$*1000);
write @S3Loading time of column-tables for CUSTOMER: $elapsed$ ms.;


-- Reconstruction of table CUSTOMER --
-- -------------------------------- --
start-timer;

create temp table M_CUSTOMER as
select C1.Value as CustID,C1.Value as Name,C2.Value as Address,
       C3.Value as City,C4.Value as Cat,C5.Value as Account
from CUST_NAME C1,CUST_ADDRESS C2,CUST_CITY C3,CUST_CAT C4,CUST_ACCOUNT C5
where C2.Entity = C1.Entity
and   C3.Entity = C1.Entity
and   C4.Entity = C1.Entity
and   C5.Entity = C1.Entity;

compute elapsed = int($timer$*1000);
write @S3Reconstruction of CUSTOMER table: $elapsed$ ms.;

extract NC = select count(*) from CUSTOMER;
write @S3Number of CUSTOMER rows: $NC$;

write;

closeDB;

deleteFile $dbDirectory$/TORDERS.db;

