-- ------------------- --
-- The Key-Value model --
-- ------------------- --

write-b -- ------------------------------------------------------ --;
write   -- KEY-VALUE data model - Version 1                       --;
write   -- ------------------------------------------------------ --;

openDB ORDERS.db;

create temp table M_CUSTOMER(
   Entity    char(10) not null,
   Attribute char(7)  not null,
   Value     varchar(60),
   primary key (Entity,Attribute));

create temp table TMP_CUSTOMER (
   CustID  char(10) not null,
   Name    char(32) not null,
   Address char(60) not null,
   City    char(30) not null,
   Cat     char(2),
   Account decimal(9,2) not null);

create temp table M_DETAIL(
   Entity    char(12) not null,
   Attribute char(6)  not null,
   Value     varchar(12),
   primary key (Entity,Attribute));

create temp table TMP_DETAIL (
   OrdID  char(12) not null,
   ProdID char(15) not null,
   Qord   integer  not null);


write-b -- Load M_CUSTOMER (SQL version) --;
start-timer;
insert into M_CUSTOMER (Entity, Attribute, Value) 
       select CustID,'CustID',CustID   from CUSTOMER
 union select CustID,'Name',Name       from CUSTOMER
 union select CustID,'Address',Address from CUSTOMER
 union select CustID,'City',City       from CUSTOMER
 union select CustID,'Cat',Cat         from CUSTOMER
 union select CustID,'Account',cast(Account as char) from CUSTOMER;
compute elapsed = int($timer$*1000);
write @S3[Loading time = $elapsed$ ms.];

delete from M_CUSTOMER;

write-b -- Load M_CUSTOMER (procedural version) --;
start-timer;
for cus,nam,add,cit,cat,acc = [select CustID,Name,Address,City,Cat,Account
                               from   CUSTOMER]
   insert into M_CUSTOMER values
          ('cus','CustID','cus'),
          ('cus','Name','nam'),
          ('cus','Address','add'),
          ('cus','City','cit'),
          ('cus','Cat',case when 'cat' = '' then null else 'cat' end),
          ('cus','Account',cast($acc$ as char));
compute elapsed = int($timer$*1000);
write @S3[Loading time = $elapsed$ ms.];

write-b -- Contents of M_CUSTOMER --;
select * from M_CUSTOMER order by Entity;

write-b -- Reconstruction of table CUSTOMER (SQL version 1) --;
start-timer;
insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
select M1.Value as CustID, M2.Value as Name, 
       M3.Value as Address, 
       M4.Value as City, M5.Value as Cat, 
       cast(M6.Value as decimal) AS Account
from   M_CUSTOMER M1, M_CUSTOMER M2, M_CUSTOMER M3, 
       M_CUSTOMER M4, M_CUSTOMER M5, M_CUSTOMER M6
where  M2.Entity = M1.Entity and M3.Entity = M1.Entity 
and    M4.Entity = M1.Entity and M5.Entity = M1.Entity
and    M6.Entity = M1.Entity
and    M1.Attribute = 'CustID'
and    M2.Attribute = 'Name'
and    M3.Attribute = 'Address'
and    M4.Attribute = 'City'
and    M5.Attribute = 'Cat'
and    M6.Attribute = 'Account'
order by CustID;
compute elapsed = int($timer$*1000);
write @S3[Regeneration time = $elapsed$ ms.];

delete from TMP_CUSTOMER;

write-b -- Reconstruction of table CUSTOMER (SQL version 2) --;
start-timer;
insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
select 
   max(case when Attribute = 'CustID' then Value else '' end) as CustID,
   max(case when Attribute = 'Name' then Value else '' end) as Name,
   max(case when Attribute = 'Address' then Value else '' end) as Address,
   max(case when Attribute = 'City' then Value else '' end) as City,
   max(case when Attribute = 'Cat' then Value else null end) as Cat,
   max(case when Attribute = 'Account' then cast(Value as decimal) else -999999.9 end) as Account
from  M_CUSTOMER
group by Entity;
compute elapsed = int($timer$*1000);
write @S3[Regeneration time = $elapsed$ ms.];

delete from TMP_CUSTOMER;

write-b -- Reconstruction of table CUSTOMER (procedural version) --;
start-timer;
extract ce = select Entity from M_CUSTOMER order by Entity;
for e,a,v = [select Entity,Attribute,Value
             from   M_CUSTOMER order by Entity];
   if ('SQLdiag$' = 'NONE') exit;
   if ('e' = 'ce');
      -- Build current row --
      if ('$a$' = 'CustID')  set cus = $v$;
      if ('$a$' = 'Name')    set nam = $v$;
      if ('$a$' = 'Address') set add = $v$;
      if ('$a$' = 'City')    set cit = $v$;
      if ('$a$' = 'Cat')     set cat = $v$;
      if ('$a$' = 'Account') set acc = $v$;
   else;
      -- Insert current row --
      insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
      values('cus','nam','add','cit',
             case when 'cat' = '' then null else 'cat' end,
             cast($acc$ as decimal));
      -- and start a new one --
      set ce = $e$;
      if ('$a$' = 'CustID')  set cus = $v$;
      if ('$a$' = 'Name')    set nam = $v$;
      if ('$a$' = 'Address') set add = $v$;
      if ('$a$' = 'City')    set cit = $v$;
      if ('$a$' = 'Cat')     set cat = $v$;
      if ('$a$' = 'Account') set acc = $v$;
   endif;
endfor;
-- Insert the last row --
insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
values('cus','nam','add','cit',
       case when 'cat'='' then null else 'cat' end,
       cast($acc$ as decimal));
compute elapsed = int($timer$*1000);
write @S3[Regeneration time = $elapsed$ ms.];

delete from TMP_CUSTOMER;

write-b -- Reconstruction of table CUSTOMER (improved procedural version) --;
start-timer;
extract ce = select Entity from M_CUSTOMER order by Entity;
for e,a,v = [select Entity,Attribute,Value
             from   M_CUSTOMER order by Entity];
   if ('SQLdiag$' = 'NONE') exit;
   if ('e' = 'ce');
      -- Build current row --
      if ('$a$' = 'CustID');
         set cus = $v$;
      else;
         if ('$a$' = 'Name');
            set nam = $v$;
         else;
            if ('$a$' = 'Address');
               set add = $v$;
            else;
               if ('$a$' = 'City');
                  set cit = $v$;
               else;
                  if ('$a$' = 'Cat');
                     set cat = $v$;
                  else;
                     set acc = $v$;
                  endif;
               endif;
            endif;
         endif;
      endif;
   else;
      -- Insert current row --
      insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
      values('cus','nam','add','cit',
             case when 'cat' = '' then null else 'cat' end,
             cast($acc$ as decimal));
      -- and start a new one --
      set ce = $e$;
      if ('$a$' = 'CustID');
         set cus = $v$;
      else;
         if ('$a$' = 'Name');
            set nam = $v$;
         else;
            if ('$a$' = 'Address');
               set add = $v$;
            else;
               if ('$a$' = 'City');
                  set cit = $v$;
               else;
                  if ('$a$' = 'Cat');
                     set cat = $v$;
                  else;
                     set acc = $v$;
                  endif;
               endif;
            endif;
         endif;
      endif;
   endif;
endfor;
if ('$ce$' <> '')
   -- Insert the last row --
   insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
   values('cus','nam','add','cit',
          case when 'cat'='' then null else 'cat' end,
          cast($acc$ as decimal));
compute elapsed = int($timer$*1000);
write @S3[Regeneration time = $elapsed$ ms.];

write-b -- Table CUSTOMER regenerated --;
select * from TMP_CUSTOMER;
write;

write-b -- Processing table DETAIL with composite primary key  --;
write   -- --------------------------------------------- --;

write-b -- Load M_DETAIL (SQL version) --;
start-timer;
insert into M_DETAIL (Entity, Attribute, Value) 
       select OrdID||'-'||ProdID,'OrdID',OrdID   from DETAIL
 union select OrdID||'-'||ProdID,'ProdID',ProdID from DETAIL
 union select OrdID||'-'||ProdID,'Qord',cast(Qord as char) from DETAIL;
compute elapsed = int($timer$*1000);
write @S3[Loading time = $elapsed$ ms.];

write-b -- Contents of M_DETAIL --;
select * from  M_DETAIL;
write;

write-b -- Application 1: Id, name and city of customers --;
write-a   -- --------------------------------------------- --;

select C1.Entity as CustID,C1.Value as Name,C2.Value as City
from   M_CUSTOMER C1, M_CUSTOMER C2
where  C1.Entity = C2.Entity
and    C1.Attribute = 'Name'
and    C2.Attribute = 'City'
order by C1.Entity;

write-b -- Application 2: Id and name of customers --;
write-a -- --------------------------------------- --;

select Entity as CustID, Value as Name
from   M_CUSTOMER
where  Attribute = 'Name';

write-b -- Application 3: Which customers have a property value including characters B1? --;
write-a -- ----------------------------------------------------------------------------- --;

select Entity,Attribute,Value
from   M_CUSTOMER
where  Value like '%B1%'
order by Entity;

closeDB;


write;
write-b -- -------------------------------------------------------- --;
write   -- KEY-VALUE data model - Version 2                         --;
write   -- -------------------------------------------------------- --;

openDB ORDERS.db;

create temp table ORDERS(
   EType      varchar(18) not null,
   Entity     varchar(15) not null,
   Attribute  varchar(32) not null,
   Value      varchar(64),
   primary key (EType,Entity,Attribute));

create temp table TMP_CUSTOMER (
   CustID  char(10) not null,
   Name    char(32) not null,
   Address char(60) not null,
   City    char(30) not null,
   Cat     char(2),
   Account decimal(9,2) not null);

create temp table TMP_DETAIL (
   OrdID  char(12) not null,
   ProdID char(15) not null,
   Qord   integer  not null);

write-b -- Load CUSTOMER,PRODUCT,CUSTORDER,DETAIL data into ORDERS --;
start-timer;
insert into ORDERS (EType,Entity, Attribute, Value) 
       select 'CUSTOMER',CustID,'CustID',CustID   from CUSTOMER
 union select 'CUSTOMER',CustID,'Name',Name       from CUSTOMER
 union select 'CUSTOMER',CustID,'Address',Address from CUSTOMER
 union select 'CUSTOMER',CustID,'City',City       from CUSTOMER
 union select 'CUSTOMER',CustID,'Cat',Cat         from CUSTOMER
 union select 'CUSTOMER',CustID,'Account',cast(Account as char) from CUSTOMER;
insert into ORDERS (EType,Entity, Attribute, Value) 
       select 'PRODUCT',ProdID,'ProdID',ProdID             from PRODUCT
 union select 'PRODUCT',ProdID,'Description',Description   from PRODUCT
 union select 'PRODUCT',ProdID,'Price',cast(Price as char) from PRODUCT
 union select 'PRODUCT',ProdID,'QonHand',cast(QonHand as char) from PRODUCT;
insert into ORDERS (EType,Entity, Attribute, Value) 
       select 'CUSTORDER',OrdID,'OrdID',OrdID     from CUSTORDER
 union select 'CUSTORDER',OrdID,'CustID',CustID   from CUSTORDER
 union select 'CUSTORDER',OrdID,'DateOrd',DateOrd from CUSTORDER;
insert into ORDERS (EType,Entity, Attribute, Value) 
       select 'DETAIL',OrdID||'-'||ProdID,'OrdID',OrdID     from DETAIL
 union select 'DETAIL',OrdID||'-'||ProdID,'ProdID',ProdID   from DETAIL
 union select 'DETAIL',OrdID||'-'||ProdID,'Qord',cast(Qord as char) from DETAIL;
compute elapsed = int($timer$*1000);
write @S3[Loading time = $elapsed$ ms.];

extract NC = select count(*) from ORDERS;
write @S3Numbers of quadruples: $NC$;
-- select * from ORDERS order by EType,Entity;


write-b -- Reconstruction of table CUSTOMER --;
start-timer;
insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
select M1.Value as CustID, M2.Value as Name, M3.Value as Address, 
       M4.Value as City, M5.Value as Cat, 
       cast(M6.Value as decimal) AS Account
from   ORDERS M1, ORDERS M2, ORDERS M3, 
       ORDERS M4, ORDERS M5, ORDERS M6
where  M1.EType = 'CUSTOMER'
and    M2.EType = M1.EType and M3.EType = M1.EType 
and    M4.EType = M1.EType and M5.EType = M1.EType 
and    M6.EType = M1.EType 
and    M2.Entity = M1.Entity and M3.Entity = M1.Entity 
and    M4.Entity = M1.Entity and M5.Entity = M1.Entity
and    M6.Entity = M1.Entity
and    M1.Attribute = 'CustID'
and    M2.Attribute = 'Name'
and    M3.Attribute = 'Address'
and    M4.Attribute = 'City'
and    M5.Attribute = 'Cat'
and    M6.Attribute = 'Account'
order by CustID;
compute elapsed = int($timer$*1000);
write @S3[Rebuild time $elapsed$ ms.];

select * from TMP_CUSTOMER;

closeDB;

write;
write-b -- -------------------------------------------------------- --;
write   -- KEY-VALUE data model - Version 3                         --;
write   -- -------------------------------------------------------- --;

openDB ORDERS.db;

create temp table ORDERS(
   Entity     varchar(36) not null,
   Attribute  varchar(32) not null,
   Value      varchar(64),
   primary key (Entity,Attribute));

create temp table TMP_CUSTOMER (
   CustID  char(10) not null,
   Name    char(32) not null,
   Address char(60) not null,
   City    char(30) not null,
   Cat     char(2),
   Account decimal(9,2) not null);

create temp table TMP_DETAIL (
   OrdID  char(12) not null,
   ProdID char(15) not null,
   Qord   integer  not null);


write-b -- Load CUSTOMER,PRODUCT,CUSTORDER,DETAIL data into ORDERS --;
start-timer;
insert into ORDERS (Entity, Attribute, Value) 
       select 'CUSTOMER'||'-'||CustID,'EType','CUSTOMER' from CUSTOMER
 union select 'CUSTOMER'||'-'||CustID,'CustID',CustID    from CUSTOMER
 union select 'CUSTOMER'||'-'||CustID,'Name',Name        from CUSTOMER
 union select 'CUSTOMER'||'-'||CustID,'Address',Address  from CUSTOMER
 union select 'CUSTOMER'||'-'||CustID,'City',City        from CUSTOMER
 union select 'CUSTOMER'||'-'||CustID,'Cat',Cat          from CUSTOMER
 union select 'CUSTOMER'||'-'||CustID,'Account',cast(Account as char) from CUSTOMER;
insert into ORDERS (Entity, Attribute, Value) 
       select 'PRODUCT'||'-'||ProdID,'EType','PRODUCT'           from PRODUCT
 union select 'PRODUCT'||'-'||ProdID,'ProdID',ProdID             from PRODUCT
 union select 'PRODUCT'||'-'||ProdID,'Description',Description   from PRODUCT
 union select 'PRODUCT'||'-'||ProdID,'Price',cast(Price as char) from PRODUCT
 union select 'PRODUCT'||'-'||ProdID,'QonHand',cast(QonHand as char) from PRODUCT;
insert into ORDERS (Entity, Attribute, Value) 
       select 'CUSTORDER'||'-'||OrdID,'EType','CUSTORDER' from CUSTORDER
 union select 'CUSTORDER'||'-'||OrdID,'OrdID',OrdID       from CUSTORDER
 union select 'CUSTORDER'||'-'||OrdID,'CustID',CustID     from CUSTORDER
 union select 'CUSTORDER'||'-'||OrdID,'DateOrd',DateOrd   from CUSTORDER;
insert into ORDERS (Entity, Attribute, Value) 
       select 'DETAIL'||'-'||OrdID||'-'||ProdID,'EType','DETAIL' from DETAIL
 union select 'DETAIL'||'-'||OrdID||'-'||ProdID,'OrdID',OrdID    from DETAIL
 union select 'DETAIL'||'-'||OrdID||'-'||ProdID,'ProdID',ProdID  from DETAIL
 union select 'DETAIL'||'-'||OrdID||'-'||ProdID,'Qord',cast(Qord as char) from DETAIL;
compute elapsed = int($timer$*1000);
write @S3[Loading time = $elapsed$ ms.];

extract NC = select count(*) from ORDERS;
write @S3Numbers of triples: $NC$;
select * from ORDERS order by Entity;


write-b -- Reconstruction of table CUSTOMER --;

delete from TMP_CUSTOMER;

start-timer;
insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
select M1.Value as CustID, M2.Value as Name, M3.Value as Address, 
       M4.Value as City, M5.Value as Cat, 
       cast(M6.Value as decimal) AS Account
from   ORDERS M0,
       ORDERS M1, ORDERS M2, ORDERS M3, 
       ORDERS M4, ORDERS M5, ORDERS M6
where  M0.Attribute = 'EType' and M0.Value = 'CUSTOMER'
and    M1.Entity = M0.Entity and M2.Entity = M0.Entity  
and    M3.Entity = M0.Entity and M4.Entity = M0.Entity 
and    M5.Entity = M0.Entity and M6.Entity = M0.Entity
and    M1.Attribute = 'CustID'
and    M2.Attribute = 'Name'
and    M3.Attribute = 'Address'
and    M4.Attribute = 'City'
and    M5.Attribute = 'Cat'
and    M6.Attribute = 'Account'
order by CustID;
compute elapsed = int($timer$*1000);
write @S3[Rebuild time $elapsed$ ms.];

select * from TMP_CUSTOMER;

write;
write-b -- Triples with abstract Entity Ids --;
write   -- -------------------------------- --;

create temp table TABLES(
   TName   varchar(36) not null,
   TId     char(2) not null,
   LastRow integer,
   primary key (TName));

insert into TABLES values
   ('CUSTOMER','01',0),('PRODUCT','02',0),('CUSTORDER','03',0),('DETAIL','04',0);

write-b -- Load CUSTOMER,PRODUCT,CUSTORDER,DETAIL data into ORDERS --;
start-timer;

delete from ORDERS;

extract tid,last = select TId,LastRow from TABLES where TName = 'CUSTOMER';
for cus,nam,add,cit,cat,acc = [select * from CUSTOMER order by CustID];
   compute last = $last$ + 1;
   compute entity = '$tid$'||'-'||frame('$last$',5,'>','0');
   insert into ORDERS (Entity,Attribute,Value) values
      ('$entity$','EType','CUSTOMER'),
      ('$entity$','CustID','cus'),
      ('$entity$','Name','nam'),
      ('$entity$','Address','add'),
      ('$entity$','City','cit'),
      ('$entity$','Cat',case when 'cat'='' then null else 'cat' end),
      ('$entity$','Account',cast($acc$ as char));
endfor;
update TABLES set LastRow = $last$ where TName = 'CUSTOMER';

extract tid,last = select TId,LastRow from TABLES where TName = 'PRODUCT';
for pro,des,pri,qon = [select * from PRODUCT order by ProdID];
   compute last = $last$ + 1;
   compute entity = '$tid$'||'-'||frame('$last$',5,'>','0');
   insert into ORDERS (Entity,Attribute,Value) values
      ('$entity$','EType','PRODUCT'),
      ('$entity$','ProdID','pro'),
      ('$entity$','Description','des'),
      ('$entity$','Price',cast($pri$ as char)),
      ('$entity$','QonHand',cast($qon$ as char));
endfor;
update TABLES set LastRow = $last$ where TName = 'PRODUCT';

extract tid,last = select TId,LastRow from TABLES where TName = 'CUSTORDER';
for ord,cus,dat = [select * from CUSTORDER order by OrdID];
   compute last = $last$ + 1;
   compute entity = '$tid$'||'-'||frame('$last$',5,'>','0');
   insert into ORDERS (Entity,Attribute,Value) values
      ('$entity$','EType','CUSTORDER'),
      ('$entity$','OrdID','ord'),
      ('$entity$','CustID','cus'),
      ('$entity$','DateOrd','$dat$');
endfor;
update TABLES set LastRow = $last$ where TName = 'CUSTORDER';

extract tid,last = select TId,LastRow from TABLES where TName = 'DETAIL';
for ord,pro,qor = [select * from DETAIL order by OrdID,ProdID];
   compute last = $last$ + 1;
   compute entity = '$tid$'||'-'||frame('$last$',5,'>','0');
   insert into ORDERS (Entity,Attribute,Value) values
      ('$entity$','EType','DETAIL'),
      ('$entity$','OrdID','ord'),
      ('$entity$','ProdID','pro'),
      ('$entity$','Qord',cast($qor$ as char));
endfor;
update TABLES set LastRow = $last$ where TName = 'DETAIL';

compute elapsed = int($timer$*1000);
write @S3[Loading time = $elapsed$ ms.];

extract NC = select count(*) from ORDERS;
write @S3Numbers of triples: $NC$;
select * from TABLES order by TName;
select * from ORDERS order by Entity;

write-b -- Reconstruction of table CUSTOMER --;

delete from TMP_CUSTOMER;

start-timer;
insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
select M1.Value as CustID, M2.Value as Name, M3.Value as Address, 
       M4.Value as City, M5.Value as Cat, 
       cast(M6.Value as decimal) AS Account
from   ORDERS M0,
       ORDERS M1, ORDERS M2, ORDERS M3, 
       ORDERS M4, ORDERS M5, ORDERS M6
where  M0.Attribute = 'EType' and M0.Value = 'CUSTOMER'
and    M1.Entity = M0.Entity and M2.Entity = M0.Entity  
and    M3.Entity = M0.Entity and M4.Entity = M0.Entity 
and    M5.Entity = M0.Entity and M6.Entity = M0.Entity
and    M1.Attribute = 'CustID'
and    M2.Attribute = 'Name'
and    M3.Attribute = 'Address'
and    M4.Attribute = 'City'
and    M5.Attribute = 'Cat'
and    M6.Attribute = 'Account'
order by CustID;
compute elapsed = int($timer$*1000);
write @S3[Rebuild time $elapsed$ ms.];

select * from TMP_CUSTOMER;

write-b -- Generate assertions for Description Logic --;
write   -- ----------------------------------------- --

-- Generate class assertions --
select distinct Value||'('''||Entity||''')' as Individuals 
from   ORDERS 
where  Attribute = 'EType'; 

-- Generate property assertions --
select Attribute||'('||''''||Entity||''','''||coalesce(doubleQuote(Value),'')||''')' as Property
from   ORDERS 
where  Attribute <> 'EType';

-- Derive "placed_by" role --
select 'placed_by('''||O1.Entity||''','''||O3.Entity||''')' as "Placed by"
from  ORDERS O1, ORDERS O2, ORDERS O3, ORDERS O4
where O1.Attribute = 'EType' and O1.Value = 'CUSTORDER'
and   O2.Attribute = 'CustID' and O2.Entity = O1.Entity
and   O3.Attribute = 'EType' and O3.Value = 'CUSTOMER'
and   O4.Attribute = 'CustID' and O4.Entity = O3.Entity
and   O2.Value = O4.Value;

closeDB;

