-- --------------------- --
-- The Object models     --
-- --------------------- --

-- Save current state of ORDERS.db --
copyFile-r $dbDirectory$/ORDERS.db,$dbDirectory$/ORDERS.db.bak;

openDB ORDERS.db;

create temp table if not exists M_CUSTOMER(
   Entity    varchar(32)   not null,
   Attribute varchar(32)   not null,
   Value     varchar(32));

insert into M_CUSTOMER ( Entity, Attribute, Value) 
       select CustID, 'CustID', CustID from CUSTOMER
 union select CustID, 'Name', Name from CUSTOMER
 union select CustID, 'Address', Address from CUSTOMER
 union select CustID, 'City', City from CUSTOMER
 union select CustID, 'Cat', Cat from CUSTOMER
 union select CustID, 'Account', Account from CUSTOMER;

write-b -- ------------------------------ --;
write   -- From Key-value to object model --;
write   -- ------------------------------ --;

write-b -- Build objects for each entity --;
        -- ----------------------------- --

set maxSelectWidth = 216;

select Entity,
       '{'
       ||group_concat('"'||Attribute||'":"'||Value||'"',',')
       ||'}' as Attributes
from   M_CUSTOMER
group by Entity;

write-b -- Build objects for each entity (improved numeric display) --;
        -- -------------------------------------------------------- --
select Entity,
       '{'||group_concat('"'||Attribute||'":'
                            ||case when Attribute in ('Account')
                                   then Value
                                   else '"'||Value||'"'
                              end, ',')
     ||'}' as Attributes
from M_CUSTOMER
group by Entity;

write-b -- ----------------------------------- --;
write   -- Dealing with multivalued properties --;
write   -- ----------------------------------- --;

insert into M_CUSTOMER(Entity,Attribute,Value)
select CustID,'Amount',sum(Qord * Price)
from   CUSTORDER O, DETAIL as D, PRODUCT as P
where  O.OrdID = D.OrdID and D.ProdID = P.ProdID
group by CustID,O.OrdID;

write-b -- Excerpts of table M_CUSTOMER with multivalued attribute "Amount" --;
        -- ---------------------------------------------------------------- --
select * from M_CUSTOMER
where Entity in ('B062','C400','F011')
order by Entity;

create temp view CUST_AMOUNT(Entity,Attribute,Value) as
select Entity,'Amounts','['||group_concat(Value,',')||']'
from   M_CUSTOMER 
where  Attribute = 'Amount'
group by Entity
  union
select Entity,Attribute,Value
from   M_CUSTOMER
where  Attribute <> 'Amount';


write-b -- The values of attribute "Amount" are grouped into arrays --;
        -- -------------------------------------------------------- --
select * from CUST_AMOUNT
where Entity in ('B062','C400','F011')
order by Entity;

write-b -- Complex JSON objects with multivalued attribute "Amount" --;
        -- -------------------------------------------------------- --
select Entity,
       '{'
       ||group_concat('"'
                      ||Attribute
                      ||'":'
                      ||case when Attribute in ('Amount','Account')
                             then Value
                             else '"'||Value||'"'
                        end,
                      ',')
       ||'}' as Attributes
from  CUST_AMOUNT
where Entity in ('B062','C400','F011')
group by Entity;


write-b -- --------------------------------------------- --;
write   -- Dealing with multivalued composite properties --;
write   -- --------------------------------------------- --;

delete from M_CUSTOMER where Attribute = 'Amount';

write-b -- Associating an array of orders to customers --;
        -- ------------------------------------------- --
create temp view CUST_ORDERS(Entity,Attribute,Value) as
select CustID,'Orders',
     '['||group_concat('{'
        ||'"OrdID":"'||OrdID||'","DateOrd":"'||DateOrd||'"'
        ||'}', ',')||']'
from   CUSTORDER
group by CustID;;

select * from CUST_ORDERS;


write-b -- Complex JSON objects with multivalued composite attribute "Orders" --;
        -- ------------------------------------------------------------------ --
create temp table CUSTOMER_JSON as
select Entity,
       '{'
       ||group_concat('"'
                    ||Attribute
                    ||'":'
                    ||case when Attribute in ('Orders','Account')
                           then Value
                           else '"'||Value||'"'
                      end,
                    ',')
       ||'}' as Attributes
from  (select Entity,Attribute,Value 
       from M_CUSTOMER
         union
       select Entity,Attribute,Value 
       from CUST_ORDERS)
where Entity in ('C400','B062','F011')
group by Entity;

select * from CUSTOMER_JSON;


write-b -- --------------------------------------------- --;
write   -- Indexing objects                              --;
write   -- --------------------------------------------- --;

write-b -- JSON objects of customers of 'Poitiers' (w/o index) --;
        -- --------------------------------------------------- -- 
select * 
from   CUSTOMER_JSON 
where  Attributes like '%"City":"Poitiers"%';

create index XCITYVAL on M_CUSTOMER(Value)
where Attribute = 'City'; 

write-b -- JSON objects of customers of 'Poitiers' (with index) --;
        -- ---------------------------------------------------- -- 
select Entity,Attributes 
from   CUSTOMER_JSON 
where  Entity in (select Entity 
                  from   M_CUSTOMER 
                  where  Attribute = 'City'
                  and    Value = 'Poitiers');

closeDB;

set maxSelectWidth = 64;

-- Restore original state of ORDERS.db --
renameFile-r $dbDirectory$/ORDERS.db.bak,ORDERS.db;

write-ab -- END OF SCRIPT --;
