# -*- coding: UTF-8 -*-

import sys

print '\n\n Generates the SQL "insert" queries that loads the description of a maze'
print ' in tables NODE (maze nodes) and NPATH (maze paths).'
print
print ' The source description is in text file "Maze_map.txt" in which the maze is'
print " drawn with characters '*' for nodes, '--' for horizontal paths and '|' for "
print " vertical paths. "
print " The maximum size of the maze is 26 x 26."
print
print ' The SQL queries are written in text file "_Maze_Loader.sql".'
print

rowRange = 'A;B;C;D;E;F;G;H;I;J;K;L;M;N;O;P;Q;R;S;T;U;V;W;X;Y;Z'.split(';')
colRange = '01;02;03;04;05;06;07;08;09;10;11;12;13;14;15;16;17;18;19;20;21;22;23;24;25;26'.split(';')

# Loading text lines in "Lines"
Lines = []
fin = open('Maze_map.txt','r')
for lin in fin:
    if lin[-1] == '\n':
        lin = lin[:-1]
    Lines.append(lin)
fin.close()

# Computing "nRows" and "nColumns"
nLines = len(Lines)
nRows = (nLines+1)/2
nChars = len(Lines[1])
nColumns = (nChars+2)/3

if nRows > 26 or nColumns > 26:
    print "\nSorry, the maze is too large.\n"
    sys.exit(0)
    
# Create "nodes" array
nodes = []
for rowName in rowRange[:nRows]:
    cityRow = []
    for colName in colRange[:nColumns]:
        cityRow.append(rowName+colName)
    nodes.append(cityRow)

fout = open('_Maze_Loader.sql','w')

# Write "insert into NODE" queries
for iRow in range(nRows):
    query = "insert into NODE(Name) values ('"+nodes[iRow][0]+"')"
    for iCol in range(1,nColumns):
        query += ",('"+nodes[iRow][iCol]+"')"
    query += ";"
    # print query
    fout.write(query+'\n')

# Write "insert into NPATH" queries"
lineType = 0    
for iLine in range(nLines):
    line = Lines[iLine]
    iRow = iLine/2 + 1
    if lineType == 0:   # Row of nodes with "horizontal paths"
        if line.find('--') == -1:   # No horizontal path
            lineType = 1
            continue
        hPaths = line[1:-1].split('*')
        query = "insert into NPATH values "
        for iPath in range(nColumns-1):
            path = hPaths[iPath]
            if path == '--':
                query += "('"+nodes[iRow-1][iPath]+"','"+nodes[iRow-1][iPath+1]+"',1),"
        query = query[:-1]+";"
        # print query
        fout.write(query+'\n')
        lineType = 1
    elif lineType == 1: # Row of "vertical paths"
        # There should exist at least one vertical path (otherwise ...)
        vPaths = []
        for i in range(nColumns):
            vPaths.append(line[i*3])
        query = "insert into NPATH values "
        for iPath in range(nColumns):
            path = vPaths[iPath]
            if path == '|':
                query += "('"+nodes[iRow-1][iPath]+"','"+nodes[iRow][iPath]+"',1),"
        query = query[:-1]+";"
        fout.write(query+'\n')
        # print query
        lineType = 0
    else:
        pass

# Write post-processing queries
fout.write('\n')
fout.write("-- Insert inverse paths --"+'\n')
fout.write("insert into NPATH(Node1,Node2,Cost) select Node2,Node1,Cost from NPATH;"+'\n')
fout.write('\n')

fout.write("--Convert NPATH into PATH --"+'\n')
fout.write("insert into PATH select C1.NodeID, C2.NodeID, P.Cost"+'\n')
fout.write("                 from   NODE C1, NPATH P, NODE C2"+'\n')
fout.write("                 where  C1.Name = P.Node1 and P.Node2 = C2.Name;"+'\n')
fout.write('\n')

fout.write("-- Table NODE initialization --"+'\n')
fout.write("update NODE set Mark = 0, Path = '', Cost = 999999;"+'\n')
fout.write('\n')

fout.write("return;"+'\n')
    
fout.close()

