write-b -- --------------------------------------------------------------- --;
write   -- TEMPORAL FIXED-LENGTH INTERVAL AGGREGATION                      --;
write   -- Applicable to valid time databases only                         --;
write-a -- --------------------------------------------------------------- --;

set Tfuture = 9999-12-31;
set limit = 50;

openDB TDB.db;

-- H_EMPLOYEE must be valid-time.
extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' <> 'char');
      showMessage WARNING:
                  @n@S3TDB.db is not a valid time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;


write-b -- --------------------------------------------------------------- --;
write   -- CREATE table CALENDAR                                           --;
write-a -- --------------------------------------------------------------- --;

-- Create a sequence of fixed-length intervals --
-- Parameters: 
-- - minDate: first date
-- - maxDate: last date
-- - period: length of the intervals

extract minDate,maxDate = select min(start),addToDate(max(end),100)
                          from H_EMPLOYEE where end < '$Tfuture$';

create temp table CALENDAR(Start date,End date);

with recursive Dates(Start,End)
as (select '$minDate$',nextDate('$minDate$')
      union all
    select D.End, nextDate(D.End) as Last
    from   Dates D
    where  Last <= '$maxDate$'
   )
insert into CALENDAR select * from Dates;

select thousandSep(count(*),',') as "CALENDAR",'$minDate$' as "From",'$maxDate$' as "To" from CALENDAR;
select Start,End,durationDays(Start,End) as Duration from CALENDAR limit $limit$;


write;
write-b -- --------------------------------------------------------------- --;
write   -- EMP_DAILY: conversion of H_EMPLOYEE with daily intervals        --;
write-a -- --------------------------------------------------------------- --;

create temp table EMP_DAILY as
  select E.CODE,E.NAME,E.PROJECT,E.CITY,E.SALARY,C.Start,C.End
  from   CALENDAR C,H_EMPLOYEE E
  where  C.Start >= E.start
  and    C.End   <= E.end;

select thousandSep(count(*),',') as "EMP_DAILY" from EMP_DAILY;
select PROJECT,CODE,NAME,CITY,SALARY,Start,End 
from   EMP_DAILY 
order by CITY,CODE,Start limit $limit$;


write;
write-b -- --------------------------------------------------------------- --;
write   -- Reconstruction of H_EMPLOYEE from EMP_DAILY                     --;
write-a -- --------------------------------------------------------------- --;

function status = LTemp:project {select CODE,NAME,SALARY,CITY,PROJECT
                                 from   EMP_DAILY into HH_EMPLOYEE};

update HH_EMPLOYEE set end = '$Tfuture$' where end = '$maxDate$';

select thousandSep(count(*),',') as "H_EMPLOYEE" from HH_EMPLOYEE;
select * from HH_EMPLOYEE order by CODE,Start limit $limit$;

drop table HH_EMPLOYEE;


write;
write-b -- --------------------------------------------------------------- --;
write   -- Evolution of the number of employees per PROJECT                --;
write-a -- --------------------------------------------------------------- --;

create temp table E_COUNT as
  select PROJECT, count(*) as Number,Start,End
  from   EMP_DAILY
  group by PROJECT,Start,End;

function status = LTemp:reduce {select PROJECT,Number from E_COUNT};

update E_COUNT set End = '$Tfuture$' where End = '$maxDate$';

select thousandSep(count(*),',') as "E_COUNT" from E_COUNT;
select * from E_COUNT order by PROJECT,Start limit $limit$;

drop table E_COUNT;


write;
write-b -- --------------------------------------------------------------- --;
write   -- Evolution of SALARY statistics (min,max,avg) per PROJECT        --;
write-a -- --------------------------------------------------------------- --;

create temp table STATISTICS  as
  select PROJECT,
         min(SALARY) as "Min",
         max(SALARY) as "Max",
         round(avg(SALARY),1) as Average,
         Start,End
  from   EMP_DAILY
  group by PROJECT,Start,End;

function status = LTemp:reduce {select PROJECT,Min,Max,Average from STATISTICS };

update STATISTICS set End = '$Tfuture$' where End = '$maxDate$';

select thousandSep(count(*),',') as "STATISTICS" from STATISTICS;
select * from STATISTICS  order by PROJECT,Start limit $limit$;

drop table STATISTICS;


write;
write-b -- --------------------------------------------------------------- --;
write   -- Evolution of monthly SALARY cost per PROJECT                    --;
write-a -- --------------------------------------------------------------- --;

write-b -- Evolution of the monthly salary cost per project (simple)--;
create temp table MONTH_COST as
  select  PROJECT,
          substr(Start,1,7) as Month,
          round(sum(SALARY/30.42),1) as TCost
  from    EMP_DAILY
  group by PROJECT,Month limit $limit$;

select thousandSep(count(*),',') as "MONTH_COST" from MONTH_COST;
select * from MONTH_COST order by PROJECT,Month;

drop table MONTH_COST;

write-b -- Evolution of the monthly salary cost per project (detailed) --;
create temp table MONTH_COST as
   select  PROJECT,
           substr(Start,1,7) as Month,
           group_concat(distinct SALARY) as Salaries,
           durationDays(min(Start),max(End)) as Duration,
           round(sum(SALARY/30.42),1) as TCost,
           min(Start) as MinStart,max(End) as MaxEnd
   from    EMP_DAILY
   group by PROJECT,Month limit $limit$;

select thousandSep(count(*),',') as "MONTH_COST" from MONTH_COST;
select * from MONTH_COST order by PROJECT,Month;

drop table MONTH_COST;

write-b -- Total salary cost per project --;
select  PROJECT,round(sum(SALARY/30.42),1) as TCost
from    EMP_DAILY
group by PROJECT limit $limit$; 


write;
write-b -- --------------------------------------------------------------- --;
write   -- Evolution of the number of employees per (PROJECT,CITY)         --;
write-a -- --------------------------------------------------------------- --;

create temp table E_COUNT as
  select PROJECT,CITY,count(*) as Number,Start,End
  from   EMP_DAILY
  group by PROJECT,CITY,Start,End;

function status = LTemp:reduce {select PROJECT,CITY,Number from E_COUNT};

update E_COUNT set End = '$Tfuture$' where End = '$maxDate$';

select thousandSep(count(*),',') as "E_COUNT" from E_COUNT;
select * from E_COUNT order by PROJECT,CITY,Start limit $limit$;

drop table E_COUNT;


write;
write-b -- ------------------------------------------------------------------;
write   -- Evolution of SALARY statistics (min,max,avg) per (PROJECT,CITY) --;
write-a -- --------------------------------------------------------------- --;

create temp table STATISTICS as
  select PROJECT,CITY, round(avg(SALARY),1) as Average, min(SALARY) as "Min", max(SALARY) as "Max",
         Start,End
  from   EMP_DAILY
  group by PROJECT,CITY,Start,End;

function status = LTemp:reduce {select PROJECT,CITY,Average,Min,Max from STATISTICS};

select thousandSep(count(*),',') as "STATISTICS" from STATISTICS;
select * from STATISTICS order by PROJECT,CITY,Start limit $limit$;

drop table STATISTICS


write;
write-b -- --------------------------------------------------------------- --;
write   -- Evolution of SALARY cost per (PROJECT,CITY)                     --;
write-a -- --------------------------------------------------------------- --;

write-b -- Evolution of the monthly salary cost per project,city (simple)  --;
create temp table MONTH_COST as
   select  PROJECT,CITY,
           substr(Start,1,7) as Month,
           round(sum(SALARY/30.42),1) as TCost
   from    EMP_DAILY
   group by PROJECT,CITY,Month limit $limit$;

select thousandSep(count(*),',') as "MONTH_COST" from MONTH_COST;
select * from MONTH_COST order by PROJECT,CITY,Month;

drop table MONTH_COST;


write-b -- Evolution of the monthly salary cost per project,city (detailed) --;
create temp table MONTH_COST as
   select  PROJECT,CITY,
           substr(Start,1,7) as Month,
           group_concat(distinct SALARY) as Salaries,
           durationDays(min(Start),max(End)) as Duration,
           round(sum(SALARY/30.42),1) as TCost,
           min(Start) as MinStart,max(End) as MaxEnd
   from    EMP_DAILY
   group by PROJECT,CITY,Month limit $limit$;

select thousandSep(count(*),',') as "MONTH_COST" from MONTH_COST;
select * from MONTH_COST order by PROJECT,CITY,Month;


write-b -- Total salary cost per project,city --;
create temp table COST as
   select  PROJECT,CITY,round(sum(SALARY/30.42),1) as TCost
   from    EMP_DAILY
   group by PROJECT,CITY limit $limit$; 

select thousandSep(count(*),',') as "COST" from COST;
select * from COST order by PROJECT,CITY;


closeDB;

write-b End of script;

return;


