write-b -- ==================================================================== --;
write   -- TEMPORAL AGGREGATION: min,max,avg,sum                                --;
write   -- Applicable to valid time databases only                              --;
write-a -- ==================================================================== --;

-- --------------------------------------------------------------------------------- --
--   Evolution of salary statistics (min,max,avg,sum) per PROJECT and (PROJECT,CITY) --
-- --------------------------------------------------------------------------------- --


set Tfuture = 9999-12-31;
set limit = 100;

openDB TDB.db;

-- H_EMPLOYEE must be non-empty and valid-time.
extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' = 'int');
      showMessage WARNING:
                  @n@S3TDB.db is not a valid time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;

closeDB;

set style = 2;
selectOne-u style = [/bEvolution of salary statistics (min,max,avg).
                     @nChoose predicative or procedural interval computing:]
                     Predicative:||Procedural:;
if ('$DIALOGbutton$' = 'Cancel') return;

if ($style$ = 1) goto PREDICATIVE;
if ($style$ = 2) goto PROCEDURAL;
return;


-- ---------------
label PREDICATIVE;
-- ---------------

write-b -- ==================================================================== --;
write   -- 1. Evolution of salary statistics (min,max,avg) per PROJECT          --;
write   --    Predicative technique                                             --;
write-a -- ==================================================================== --;

openDB TDB.db;

select CODE,SALARY,PROJECT,start,end 
from H_EMPLOYEE order by PROJECT,start limit $limit$;

write-b -- -------------------------------------------------------------------- --;
write   -- EVENT: distinct events of each project                               --;
write-a -- -------------------------------------------------------------------- --;

create temp table EVENT as
  select distinct PROJECT,Instant
  from (select PROJECT,start as Instant from H_EMPLOYEE
         union
        select PROJECT,end as Instant from H_EMPLOYEE);

select thousandSep(count(*),',') as "EVENT" from EVENT;
select * from EVENT order by PROJECT,Instant limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- STABLE_INTERVAL: stable intervals of each project                    --;
write-a -- -------------------------------------------------------------------- --;

create temp table STABLE_INTERVAL as
  select PROJECT,
         Instant as Start,
         (select min(Instant)
          from   EVENT
          where  PROJECT = V.PROJECT
          and    Instant > V.Instant) as End
  from   EVENT V
  where  End is not null;

select thousandSep(count(*),',') as "STABLE_INTERVAL" from STABLE_INTERVAL;
select * from STABLE_INTERVAL order by PROJECT,Start limit $limit$;

create index XSI on STABLE_INTERVAL(PROJECT,start);


write-b -- -------------------------------------------------------------------- --;
write   -- VALUED_STABLE_INTERVAL:  PROJECT,interval -->-> SALARY               --;
write-a -- -------------------------------------------------------------------- --;

create temp table VALUED_STABLE_INTERVAL as
  select S.PROJECT,E.SALARY,S.Start,S.End
  from   STABLE_INTERVAL S,H_EMPLOYEE E
  where  S.PROJECT = E.PROJECT
  and    S.Start >= E.start
  and    S.End <=  E.end;

select thousandSep(count(*),',') as "VALUED_STABLE_INTERVAL" from VALUED_STABLE_INTERVAL;
select * from VALUED_STABLE_INTERVAL order by PROJECT,start limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- Evolution of SALARY statistics (min,max,avg) per project             --;
write-a -- -------------------------------------------------------------------- --;

create temp table STATISTICS as
  select  PROJECT,
          min(SALARY) as Min,
          max(SALARY) as Max,
          round(avg(SALARY),1) as Average,
          Start,End
  from    VALUED_STABLE_INTERVAL
  group by PROJECT,Start,End limit $limit$;

function LTemp:reduce {select PROJECT,Min,Max,Average from STATISTICS};

select thousandSep(count(*),',') as "STATISTICS" from STATISTICS;
select * from STATISTICS order by PROJECT,start limit $limit$;


write;
write-b -- -------------------------------------------------------------------- --;
write   -- Evolution of SALARY cost per project                                 --;
write-a -- -------------------------------------------------------------------- --;

extract maxDate = select addToDate(max(end),100) from H_EMPLOYEE where end < '$Tfuture$';

write-b maxDate = $maxDate$ instead of $Tfuture$;

write-b -- Preliminary: VALUED_STABLE_INTERVAL completed with salary information --;
select  PROJECT,SALARY as MonthSal,
        round(SALARY/30.42,2) as DailySal,
        durationDays(Start,min(End,'$maxDate$')) as Duration,
        round(SALARY/30.42 * durationDays(Start,min(End,'$maxDate$')),1) as Cost,
        Start,End
from    VALUED_STABLE_INTERVAL
order by PROJECT,Start limit $limit$;

write-b -- Evolution of SALARY cost per project --;
select  PROJECT,
        round(sum(SALARY/30.42 * durationDays(Start,min(End,'$maxDate$'))),1) as Cost,
        Start,End
from    VALUED_STABLE_INTERVAL
group by PROJECT,Start,End limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- Total SALARY cost of each project                                    --;
write-a -- -------------------------------------------------------------------- --;

select  PROJECT,
        round(sum(SALARY/30.42 * durationDays(Start,min(End,'$maxDate$'))),1) as TCost
from    VALUED_STABLE_INTERVAL
group by PROJECT;


write-b -- -------------------------------------------------------------------- --;
write   -- Complement: augmented VALUED_STABLE_INTERVAL                         --;
write-a -- -------------------------------------------------------------------- --;

drop table VALUED_STABLE_INTERVAL;
create temp table VALUED_STABLE_INTERVAL as
  select S.PROJECT,E.SALARY,
         durationDays(S.Start,min(S.End,'$date$')) as Duration,
         round(E.SALARY/30.42,1) AS DailySal,
         round(E.SALARY/30.42*durationDays(S.Start,min(S.End,'$date$')),1) as TotalSal,
         S.Start,S.End
  from   STABLE_INTERVAL S,H_EMPLOYEE E
  where  S.PROJECT = E.PROJECT
  and    S.Start >= E.start
  and    S.End <=  E.end;

select thousandSep(count(*),',') as "VALUED_STABLE_INTERVAL" from VALUED_STABLE_INTERVAL;
select * from VALUED_STABLE_INTERVAL order by PROJECT,start limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- Average SALARY per project                                           --;
write-a -- -------------------------------------------------------------------- --;

write -- Average monthly SALARY per project (incorrect average) --;
select PROJECT,round(avg(SALARY),1) as AvgSal
from   VALUED_STABLE_INTERVAL
group by PROJECT limit $limit$;

write-b -- Average monthly SALARY per project (correct average) --;
select PROJECT,
       sum(TotalSal) as TotalSalary,sum(Duration) as Days,
       round(1.*sum(TotalSal)/sum(Duration)*30.42,1) as AverageSalary
from   VALUED_STABLE_INTERVAL
group by PROJECT limit $limit$;

closeDB;


write-ab;
write-b -- ==================================================================== --;
write   -- 2. Evolution of salary statistics (min,max,avg) per PROJECT,CITY     --;
write   --    Predicative technique                                             --;
write-a -- -------------------------------------------------------------------- --;

openDB TDB.db;

write-b -- -------------------------------------------------------------------- --;
write   -- EVENT: distinct events of each (project,city)                        --;
write-a -- -------------------------------------------------------------------- --;

create temp table EVENT as
  select distinct PROJECT,CITY,Instant
  from (select PROJECT,CITY,start as Instant from H_EMPLOYEE
         union
        select PROJECT,CITY,end as Instant from H_EMPLOYEE);

select thousandSep(count(*),',') as "EVENT" from EVENT;
select * from EVENT order by PROJECT,CITY,Instant limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- STABLE_INTERVAL: stable intervals of each (project,city)             --;
write-a -- -------------------------------------------------------------------- --;

create temp table STABLE_INTERVAL as
  select PROJECT,CITY,
         Instant as Start,
         (select min(Instant)
          from   EVENT
          where  (PROJECT,CITY) = (V.PROJECT,V.CITY)
          and    Instant > V.Instant) as End
  from   EVENT V
  where  End is not null;

select thousandSep(count(*),',') as "STABLE_INTERVAL" from STABLE_INTERVAL;
select * from STABLE_INTERVAL order by PROJECT,CITY,Start limit $limit$;


create index XSI on STABLE_INTERVAL(PROJECT,CITY,start);


write-b -- -------------------------------------------------------------------- --;
write   -- VALUED_STABLE_INTERVAL:  PROJECT,CITY,interval -->-> SALARY          --;
write-a -- -------------------------------------------------------------------- --;

create temp table VALUED_STABLE_INTERVAL as
  select S.PROJECT,S.CITY,E.SALARY,S.Start,S.End
  from   STABLE_INTERVAL S,H_EMPLOYEE E
  where  (S.PROJECT,S.CITY) = (E.PROJECT,E.CITY)
  and    S.Start >= E.start
  and    S.End <=  E.end;

select thousandSep(count(*),',') as "VALUED_STABLE_INTERVAL" from VALUED_STABLE_INTERVAL;
select * from VALUED_STABLE_INTERVAL order by PROJECT,CITY,start limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- Evolution of SALARY statistics (min,max,avg) per (project,city)      --;
write-a -- -------------------------------------------------------------------- --;

create temp table STATISTICS as
  select  PROJECT,CITY,
          min(SALARY) as Min,
          max(SALARY) as Max,
          round(avg(SALARY),1) as Average,
          Start,End
  from    VALUED_STABLE_INTERVAL
  group by PROJECT,CITY,Start,End limit $limit$;

function LTemp:reduce {select PROJECT,CITY,Min,Max,Average from STATISTICS};

select thousandSep(count(*),',') as "STATISTICS" from STATISTICS;
select * from STATISTICS order by PROJECT,CITY,start limit $limit$;


write;
write-b -- -------------------------------------------------------------------- --;
write   -- Evolution of SALARY cost per project and city                        --;
write   -- -------------------------------------------------------------------- --;

extract maxDate = select addToDate(max(end),100) from H_EMPLOYEE where end < '$Tfuture$';

write-b maxDate = $maxDate$ instead of $Tfuture$;

write-b -- VALUED_STABLE_INTERVAL completed with salary information --;
select  PROJECT,CITY,SALARY,
        round(SALARY/30.42,2) as DailySal,
        durationDays(Start,min(End,'$maxDate$')) as Duration,
        round(SALARY/30.42 * durationDays(Start,min(End,'$maxDate$')),1) as Cost,
        Start,End
from    VALUED_STABLE_INTERVAL
order by PROJECT,CITY,Start limit $limit$;

write-b -- Evolution of SALARY cost per project and city --;
select  PROJECT,CITY,
        round(sum(SALARY/30.42 * durationDays(Start,min(End,'$maxDate$'))),1) as Cost,
        Start,End
from    VALUED_STABLE_INTERVAL
group by PROJECT,CITY,Start,End limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- Total SALARY cost of each project                                    --;
write-a -- -------------------------------------------------------------------- --;

select  PROJECT,
        round(sum(SALARY/30.42 * durationDays(Start,min(End,'$maxDate$'))),1) as TCost
from    VALUED_STABLE_INTERVAL
group by PROJECT;


write-b -- -------------------------------------------------------------------- --;
write   -- Total SALARY cost of each city                                       --;
write-a -- -------------------------------------------------------------------- --;

select  CITY,
        round(sum(SALARY/30.42 * durationDays(Start,min(End,'$maxDate$'))),1) as TCost
from    VALUED_STABLE_INTERVAL
group by CITY;



write-b -- -------------------------------------------------------------------- --;
write   -- Complement: augmented VALUED_STABLE_INTERVAL                         --;
write-a -- -------------------------------------------------------------------- --;

write-a -- Has be disabled. "Decomment" the query if needed.;

--drop table VALUED_STABLE_INTERVAL;
--create temp table VALUED_STABLE_INTERVAL as
--  select S.PROJECT,S.CITY,E.CODE,E.SALARY,
--         durationDays(S.Start,min(S.End,'$maxDate$')) as Duration,
--         round(E.SALARY/30.42,1) AS DailySal,
--         round(E.SALARY/30.42*durationDays(S.Start,min(S.End,'$maxDate$')),1) as TotalSal,
--         S.Start,S.End
--  from   STABLE_INTERVAL S,H_EMPLOYEE E
--  where  (S.PROJECT,S.CITY) = (E.PROJECT,E.CITY)
--  and    S.Start >= E.start
--  and    S.End <=  E.end;

--select thousandSep(count(*),',') as "VALUED_STABLE_INTERVAL" from VALUED_STABLE_INTERVAL;
--select * from VALUED_STABLE_INTERVAL order by PROJECT,CITY,CODE,start limit $limit$;

closeDB;

-- -------
goto EXIT;
-- -------


-- -------------
label PROCEDURAL;
-- -------------

write-b -- ==================================================================== --;
write   -- 1. Evolution of salary statistics (min,max,avg,sum) per PROJECT      --;
write   --    Procedural technique (LTemp:valuedInterval)                       --;
write-a -- ==================================================================== --;

openDB TDB.db;


write-b -- -------------------------------------------------------------------- --;
write   -- VALUED_STABLE_INTERVAL:  PROJECT,interval -->-> SALARY               --;
write-a -- -------------------------------------------------------------------- --;

function LTemp:valuedInterval {select PROJECT,SALARY from H_EMPLOYEE
                               group by PROJECT into VALUED_STABLE_INTERVAL};

select thousandSep(count(*),',') as "VALUED_STABLE_INTERVAL" from VALUED_STABLE_INTERVAL;
select * from VALUED_STABLE_INTERVAL order by PROJECT,Start limit $limit$;

write-b -- Computed by  {select PROJECT,SALARY from H_EMPLOYEE                  --;
write-a --               group by PROJECT into VALUED_STABLE_INTERVAL}          --;


write-b -- -------------------------------------------------------------------- --;
write   -- Evolution of SALARY statistics (min,max,avg) per project             --;
write-a -- -------------------------------------------------------------------- --;

create temp table STATISTICS as
  select  PROJECT,
          min(SALARY) as Min,
          max(SALARY) as Max,
          round(avg(SALARY),1) as Average,
          Start,End
  from    VALUED_STABLE_INTERVAL
  group by PROJECT,Start,End limit $limit$;

function LTemp:reduce {select PROJECT,Min,Max,Average from STATISTICS};

select thousandSep(count(*),',') as "STATISTICS" from STATISTICS;
select * from STATISTICS order by PROJECT,start limit $limit$;


write;
write-b -- -------------------------------------------------------------------- --;
write   -- Evolution of SALARY cost per project                                 --;
write   -- -------------------------------------------------------------------- --;

extract maxDate = select addToDate(max(end),100) from H_EMPLOYEE where end < '$Tfuture$';

write-b maxDate = $maxDate$ instead of $Tfuture$;

write-b -- Evolution of SALARY cost per project --;
select  PROJECT,
        round(sum(SALARY/30.42 * durationDays(Start,min(End,'$maxDate$'))),1) as Cost,
        Start,End
from    VALUED_STABLE_INTERVAL
group by PROJECT,Start,End limit $limit$;

closeDB;


write-b;
write-b -- ==================================================================== --;
write   -- 2. Evolution of salary statistics (min,max,avg) per PROJECT,CITY     --;
write   --    Procedural technique (LTemp:valuedInterval)                       --;
write-a -- ==================================================================== --;

openDB TDB.db;

write-b -- -------------------------------------------------------------------- --;
write   -- SVALUED_STABLE_INTERVAL:  PROJECT,CITY,interval -->-> SALARY         --;
write-a -- -------------------------------------------------------------------- --;

function LTemp:valuedInterval {select PROJECT,CITY,SALARY from H_EMPLOYEE
                               group by PROJECT,CITY into VALUED_STABLE_INTERVAL};

select thousandSep(count(*),',') as "VALUED_STABLE_INTERVAL" from VALUED_STABLE_INTERVAL;
select * from VALUED_STABLE_INTERVAL order by PROJECT,CITY,Start limit $limit$;

write-b -- Computed by  {select PROJECT,CITY,SALARY from H_EMPLOYEE             --;
write-a --               group by PROJECT,CITY into VALUED_STABLE_INTERVAL}     --;


write-b -- -------------------------------------------------------------------- --;
write   -- Evolution of SALARY statistics (min,max,avg) per project and city    --;
write-a -- -------------------------------------------------------------------- --;

create temp table STATISTICS as
  select  PROJECT,CITY,
          min(SALARY) as Min,
          max(SALARY) as Max,
          round(avg(SALARY),1) as Average,
          Start,End
  from    VALUED_STABLE_INTERVAL
  group by PROJECT,CITY,Start,End limit $limit$;

function LTemp:reduce {select PROJECT,CITY,Min,Max,Average from STATISTICS};

select thousandSep(count(*),',') as "STATISTICS" from STATISTICS;
select * from STATISTICS order by PROJECT,CITY,start limit $limit$;


write;
write-b -- -------------------------------------------------------------------- --;
write   -- Evolution of SALARY cost per (project,city)                          --;
write-a -- -------------------------------------------------------------------- --;

extract maxDate = select addToDate(max(end),100) from H_EMPLOYEE where end < '$Tfuture$';

write-b maxDate = $maxDate$ instead of $Tfuture$;

write-b -- Evolution of SALARY cost per project and city --;
select  PROJECT,CITY,
        round(sum(SALARY/30.42 * durationDays(Start,min(End,'$maxDate$'))),1) as Cost,
        Start,End
from    VALUED_STABLE_INTERVAL
group by PROJECT,CITY,Start,End limit $limit$;

closeDB;


-- ---------
label EXIT;
-- ---------

write-b End of script;

return;
