
set Tfuture = 9999-12-31;
set limit = 10;

write-b -- --------------------------------------------------------------- --;
write   -- TEMPORAL FIXED-LENGTH INTERVAL AGGREGATION                      --;
write   -- Applicable to valid time databases only                         --;
write   -- --------------------------------------------------------------- --;

-- Select a database --
set db = TDB;
ask-u db = [/bSelect a database] 
            Database:[(TDB,TDB-100,TDB-200,TDB-500,TDB-1000,TDB-2000,TDB-5000,TDB-10000,TDB-20000,TDB-50000,TDB-100000,TDB-200000)];
if ('$DIALOGbutton$' = 'Cancel') return;

set db = $db$.db;

if (not fileExists('$dbDirectory$/$db$'));
   showMessage Database "$db$" not found.
               @nPlease load the experimental temporal databases from SQLfast website;
   return;
endif;


openDB $db$;

-- H_EMPLOYEE must be non-empty and valid-time.
extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' = 'int');
      showMessage WARNING:
                  @n@S3TDB.db is not a valid time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;

extract Nemp = select count(distinct CODE) from H_EMPLOYEE;
extract maxDate = select addToDate(max(end),100) from H_EMPLOYEE where end < '$Tfuture$';

write-b;
select thousandSep($Nemp$,',') as "EMPLOYEE",
       thousandSep(count(*),',') as "H_EMPLOYEE"
from H_EMPLOYEE;


write-b -- --------------------------------------------------------------- --;
write   -- S1: CREATE table CALENDAR                                       --;
write-a -- --------------------------------------------------------------- --;

-- Create a sequence of fixed-length intervals --
-- Parameters: 
-- - minDate: first date
-- - maxDate: last date

extract minDate,maxDate = select min(start),addToDate(max(end),100)
                          from H_EMPLOYEE where end < '$Tfuture$';

start-timer;
create temp table CALENDAR(Start date,End date);

with recursive Dates(Start,End)
as (select '$minDate$',nextDate('$minDate$')
      union all
    select D.End, nextDate(D.End) as Last
    from   Dates D
    where  Last <= '$maxDate$'
   )
insert into CALENDAR select * from Dates;

set S1 = $timer$;
extract S1size = select thousandSep(count(*),',') from CALENDAR;
select thousandSep(count(*),',') as "CALENDAR",'$minDate$' as "From",'$maxDate$' as "To" from CALENDAR;
select Start,End,durationDays(Start,End) as Duration from CALENDAR limit $limit$;


write;
write-b -- --------------------------------------------------------------- --;
write   -- S2: EMP_DAILY: conversion of H_EMPLOYEE with daily intervals    --;
write-a -- --------------------------------------------------------------- --;

start-timer;
create temp table EMP_DAILY as
  select E.CODE,E.NAME,E.PROJECT,E.CITY,E.SALARY,C.Start,C.End
  from   CALENDAR C,H_EMPLOYEE E
  where  C.Start >= E.start
  and    C.End   <= E.end;

set S2 = $timer$;
extract S2size = select thousandSep(count(*),',') from EMP_DAILY;
select thousandSep(count(*),',') as "EMP_DAILY" from EMP_DAILY;
select PROJECT,CODE,NAME,CITY,SALARY,Start,End 
from   EMP_DAILY 
order by CITY,CODE,Start limit $limit$;


write;
write-b -- --------------------------------------------------------------- --;
write   -- S3: Reconstruction of H_EMPLOYEE from EMP_DAILY                 --;
write-a -- --------------------------------------------------------------- --;

start-timer;
function status = LTemp:project {select CODE,NAME,SALARY,CITY,PROJECT
                                 from   EMP_DAILY into HH_EMPLOYEE};
update HH_EMPLOYEE set end = '$Tfuture$' where end = '$maxDate$';

set S3 = $timer$;
extract S3size = select thousandSep(count(*),',') from HH_EMPLOYEE;
select thousandSep(count(*),',') as "H_EMPLOYEE" from HH_EMPLOYEE;
select * from HH_EMPLOYEE order by CODE,Start limit $limit$;

drop table HH_EMPLOYEE;


write;
write-b -- --------------------------------------------------------------- --;
write   -- S4: Evolution of the number of employees per PROJECT            --;
write-a -- --------------------------------------------------------------- --;

start-timer;
create temp table E_COUNT as
  select PROJECT, count(*) as Number,Start,End
  from   EMP_DAILY
  group by PROJECT,Start,End;
function status = LTemp:reduce {select PROJECT,Number from E_COUNT};
update E_COUNT set End = '$Tfuture$' where End = '$maxDate$';

set S4 = $timer$;
extract S4size = select thousandSep(count(*),',') from E_COUNT;
select thousandSep(count(*),',') as "E_COUNT" from E_COUNT;
select * from E_COUNT order by PROJECT,Start limit $limit$;

drop table E_COUNT;


write;
write-b -- --------------------------------------------------------------- --;
write   -- S5: Evolution of SALARY statistics (min,max,avg) per PROJECT    --;
write-a -- --------------------------------------------------------------- --;

start-timer;
create temp table STATISTICS  as
  select PROJECT,
         min(SALARY) as "Min",
         max(SALARY) as "Max",
         round(avg(SALARY),1) as Average,
         Start,End
  from   EMP_DAILY
  group by PROJECT,Start,End;
function status = LTemp:reduce {select PROJECT,Min,Max,Average from STATISTICS };
update STATISTICS set End = '$Tfuture$' where End = '$maxDate$';

set S5 = $timer$;
extract S5size = select thousandSep(count(*),',') from STATISTICS;
select thousandSep(count(*),',') as "STATISTICS" from STATISTICS;
select * from STATISTICS  order by PROJECT,Start limit $limit$;

drop table STATISTICS;


write;
write-b -- --------------------------------------------------------------- --;
write   -- S6: Evolution of monthly SALARY cost per PROJECT                --;
write-a -- --------------------------------------------------------------- --;

write-b -- Evolution of the monthly salary cost per project (simple)--;

start-timer;
create temp table MONTH_COST as
  select  PROJECT,
          substr(Start,1,7) as Month,
          round(sum(SALARY/30.42),1) as TCost
  from    EMP_DAILY
  group by PROJECT,Month limit $limit$;


set S6 = $timer$;
extract S6size = select thousandSep(count(*),',') from MONTH_COST;
select thousandSep(count(*),',') as "MONTH_COST" from MONTH_COST;
select * from MONTH_COST order by PROJECT,Month;

drop table MONTH_COST;


write;
write-b -- --------------------------------------------------------------- --;
write   -- S7: Evolution of the number of employees per (PROJECT,CITY)     --;
write-a -- --------------------------------------------------------------- --;

start-timer;
create temp table E_COUNT as
  select PROJECT,CITY,count(*) as Number,Start,End
  from   EMP_DAILY
  group by PROJECT,CITY,Start,End;
function status = LTemp:reduce {select PROJECT,CITY,Number from E_COUNT};
update E_COUNT set End = '$Tfuture$' where End = '$maxDate$';

set S7 = $timer$;
extract S7size = select thousandSep(count(*),',') from E_COUNT;
select thousandSep(count(*),',') as "E_COUNT" from E_COUNT;
select * from E_COUNT order by PROJECT,CITY,Start limit $limit$;

drop table E_COUNT;


write;
write-b -- ----------------------------------------------------------------------;
write   -- S8: Evolution of SALARY statistics (min,max,avg) per (PROJECT,CITY) --;
write-a -- ------------------------------------------------------------------- --;

start-timer;
create temp table STATISTICS as
  select PROJECT,CITY, round(avg(SALARY),1) as Average, min(SALARY) as "Min", max(SALARY) as "Max",
         Start,End
  from   EMP_DAILY
  group by PROJECT,CITY,Start,End;
function status = LTemp:reduce {select PROJECT,CITY,Average,Min,Max from STATISTICS};

set S8 = $timer$;
extract S8size = select thousandSep(count(*),',') from STATISTICS;
select thousandSep(count(*),',') as "STATISTICS" from STATISTICS;
select * from STATISTICS order by PROJECT,CITY,Start limit $limit$;

drop table STATISTICS


write;
write-b -- --------------------------------------------------------------- --;
write   -- S9,SA: Evolution of SALARY cost per (PROJECT,CITY)              --;
write-a -- --------------------------------------------------------------- --;

write-b -- S9: Evolution of the monthly salary cost per project,city (simple)  --;
start-timer;
create temp table MONTH_COST as
   select  PROJECT,CITY,
           substr(Start,1,7) as Month,
           round(sum(SALARY/30.42),1) as TCost
   from    EMP_DAILY
   group by PROJECT,CITY,Month limit $limit$;


set S9 = $timer$;
extract S9size = select thousandSep(count(*),',') from MONTH_COST;
select thousandSep(count(*),',') as "MONTH_COST" from MONTH_COST;
select * from MONTH_COST order by PROJECT,CITY,Month;

drop table MONTH_COST;


write-b -- SA: Total salary cost per project,city --;
start-timer;
create temp table COST as
   select  PROJECT,CITY,round(sum(SALARY/30.42),1) as TCost
   from    EMP_DAILY
   group by PROJECT,CITY limit $limit$; 

set SA = $timer$;
extract SAsize = select thousandSep(count(*),',') from COST;
select thousandSep(count(*),',') as "COST" from COST;
select * from COST order by PROJECT,CITY;


closeDB;


-- =================================== PERFORMANCE SYNTHESIS ====================================== --

write;
write-ab PERFORMANCE SYNTHESIS;

createDB InMemory;

select thousandSep($Nemp$,',') as "#employees", 'S1' as Query, 'CALENDAR' as Operation, '$S1$' as Time, '$S1size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S2' as Query, 'EMP_DAILY' as Operation, '$S2$' as Time, '$S2size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S3' as Query, 'H_EMPLOYEE' as Operation, '$S3$' as Time, '$S3size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S4' as Query, 'E_COUNT-mono' as Operation, '$S4$' as Time, '$S4size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S5' as Query, 'STATISTICS-mono' as Operation, '$S5$' as Time, '$S5size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S6' as Query, 'MONTH_COST-mono' as Operation, '$S6$' as Time, '$S6size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S7' as Query, 'E_COUNT-multi' as Operation, '$S7$' as Time, '$S7size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S8' as Query, 'STATISTICS-multi' as Operation, '$S8$' as Time, '$S8size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S9' as Query, 'MONTH_COST-multi' as Operation, '$S9$' as Time, '$S9size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'SA' as Query, 'COST-multi' as Operation, '$SA$' as Time, '$SAsize$' as Rows
order by Query;


closeDB;

write-b End of script;

return;


