set Tfuture = 9999-12-31;
set limit = 10;

write-b -- ================================================================ --;
write   -- AGGREGATION: COUNTING EMPLOYEES                                  --;
write   -- Applicable to valid time databases only                          --;
write   -- ================================================================ --;


-- Select a database --
set db = TDB;
ask-u db = [/bSelect a database] 
            Database:[(TDB,TDB-100,TDB-200,TDB-500,TDB-1000,TDB-2000,TDB-5000,TDB-10000,TDB-20000,TDB-50000,TDB-100000,TDB-200000)];
if ('$DIALOGbutton$' = 'Cancel') return;

set db = $db$.db;

if (not fileExists('$dbDirectory$/$db$'));
   showMessage Database "$db$" not found.
               @nPlease load the experimental temporal databases from SQLfast website;
   return;
endif;


openDB $db$;

-- H_EMPLOYEE must be non-empty and valid-time.
extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' = 'int');
      showMessage WARNING:
                  @n@S3TDB.db is not a valid time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;


extract Nemp = select count(distinct CODE) from H_EMPLOYEE;
extract maxDate = select addToDate(max(end),100) from H_EMPLOYEE where end < '$Tfuture$';

write-b;
select thousandSep($Nemp$,',') as "EMPLOYEE",
       thousandSep(count(*),',') as "H_EMPLOYEE"
from H_EMPLOYEE;

closeDB;

-- ========================== COUNTING PER PROJECT ============================ --

openDB $db$;

write-b -- ==================================================================== --;
write   -- Compute the evolution of the number of EMPLOYEEs per PROJECT         --;
write   -- Procedural techniques                                                --;
write-a -- ==================================================================== --;


write-b -- -------------------------------------------------------------------- --;
write   -- C1: Create STABLE_EMPLOYEE: stable intervals of EMPLOYEE par project --;
write   -- -------------------------------------------------------------------- --;

start-timer;
function status = LTemp:valuedInterval {select PROJECT
                                        from   H_EMPLOYEE
                                        group by PROJECT
                                        into STABLE_EMPLOYEE};
set C1 = $timer$;
extract C1size = select thousandSep(count(*),',') from STABLE_EMPLOYEE;
select thousandSep(count(*),',') as "STABLE_EMPLOYEE" from STABLE_EMPLOYEE;
select * from STABLE_EMPLOYEE order by PROJECT,start limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- C2: Create E_COUNT: number of employees in each stable interval      --;
write   -- -------------------------------------------------------------------- --;

start-timer;

-- E-COUNT = 4 s. au lieu de 31 s. sans index --
create index XSC on STABLE_EMPLOYEE(PROJECT,start,end);

create temp table E_COUNT as
   select PROJECT,count(*) as Number,start,end
   from   STABLE_EMPLOYEE
   group by PROJECT,Start;

set C2 = $timer$;
extract C2size = select thousandSep(count(*),',') from E_COUNT;
select thousandSep(count(*),',') as "E_COUNT" from E_COUNT;
select * from E_COUNT order by PROJECT,Start limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- C3: Reduce value-equivalent intervals of E_COUNT                     --;
write   -- -------------------------------------------------------------------- --;

start-timer;
function status = LTemp:reduce {select PROJECT,Number from E_COUNT};

set C3 = $timer$;
extract C3size = select thousandSep(count(*),',') from E_COUNT;
select thousandSep(count(*),',') as "E_COUNT" from E_COUNT;
select * from E_COUNT order by PROJECT,Start limit $limit$;

compute Tot1 = $C1$ + $C2$ + $C3$;

closeDB;

-- ===================== COUNTING PER PROJECT,CITY ============================ --

write;
write-b -- ========================================================================== --;
write   -- 2. Compute the evolution of the number of EMPLOYEEs per PROJECT,CITY       --;
write   --    Procedural techniques                                                   --;
write-a -- ========================================================================== --;

openDB $db$;


write-b -- -------------------------------------------------------------------------- --;
write   -- C11: Create STABLE_EMPLOYEE: stable intervals of EMPLOYEE par PROJECT,CITY --;
write   -- -------------------------------------------------------------------------- --;


start-timer;
function status = LTemp:valuedInterval {select PROJECT,CITY
                                        from   H_EMPLOYEE
                                        group by PROJECT,CITY
                                        into STABLE_EMPLOYEE};
set C11 = $timer$;
extract C11size = select thousandSep(count(*),',') from STABLE_EMPLOYEE;
select thousandSep(count(*),',') as "STABLE_EMPLOYEE" from STABLE_EMPLOYEE;
select * from STABLE_EMPLOYEE order by PROJECT,CITY,start limit $limit$;


write-b -- -------------------------------------------------------------------------- --;
write   -- C12: Create E_COUNT: number of employees in each stable interval           --;
write   -- -------------------------------------------------------------------------- --;

start-timer;
-- E-COUNT = avec index: 4 s. au lieu de 31 s. sans index (Nemp = 5000) --
create index XSC on STABLE_EMPLOYEE(PROJECT,start,end);

create temp table E_COUNT as
   select PROJECT,CITY,count(*) as Number,start,end
   from   STABLE_EMPLOYEE
   group by PROJECT,CITY,start;

set C12 = $timer$;
extract C12size = select thousandSep(count(*),',') from E_COUNT;
select thousandSep(count(*),',') as "E_COUNT" from E_COUNT;
select * from E_COUNT order by PROJECT,CITY,Start limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- C13: Reduce value-equivalent intervals of E_COUNT                     --;
write   -- -------------------------------------------------------------------- --;

start-timer;
function status = LTemp:reduce {select PROJECT,CITY,Number from E_COUNT};

set C13 = $timer$;
extract C13size = select thousandSep(count(*),',') from E_COUNT;
select thousandSep(count(*),',') as "E_COUNT" from E_COUNT;
select * from E_COUNT order by PROJECT,CITY,Start limit $limit$;

compute Tot2 = $C11$ + $C12$ + $C13$;

closeDB;


-- ============================================================================ --

write;
write-ab PERFORMANCE SYNTHESIS;

createDB InMemory;

write-b -- Counting per PROJECT - Procedural technique --;
select thousandSep($Nemp$,',') as "#employees", 'C1' as Query, 'STABLE_EMPLOYEE' as Operation, '$C1$' as Time, '$C1size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'C2' as Query, 'E-COUNT' as Operation, '$C2$' as Time, '$C2size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'C3' as Query, 'Reduce' as Operation, '$C3$' as Time, '$C3size$' as Rows
order by Query;

select $Tot1$ as Total_Time;


write-b -- Counting per PROJECT,CITY - Procedural technique --;
select thousandSep($Nemp$,',') as "#employees", 'C11' as Query, 'STABLE_EMPLOYEE' as Operation, '$C11$' as Time, '$C11size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'C12' as Query, 'E-COUNT' as Operation, '$C12$' as Time, '$C12size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'C13' as Query, 'Reduce' as Operation, '$C13$' as Time, '$C13size$' as Rows
order by Query;

select $Tot2$ as Total_Time;


closeDB;

write-b End of script;

return;
