
set Tfuture = 9999-12-31;
set limit = 10;

write-b -- ==================================================================== --;
write   -- TEMPORAL AGGREGATION: min,max,avg,sum                                --;
write   -- Applicable to valid time databases only                              --;
write-a -- ==================================================================== --;


-- Select a database --
set db = TDB;
ask-u db = [/bSelect a database] 
            Database:[(TDB,TDB-100,TDB-200,TDB-500,TDB-1000,TDB-2000,TDB-5000,TDB-10000,TDB-20000,TDB-50000,TDB-100000,TDB-200000)];
if ('$DIALOGbutton$' = 'Cancel') return;

set db = $db$.db;

if (not fileExists('$dbDirectory$/$db$'));
   showMessage Database "$db$" not found.
               @nPlease load the experimental temporal databases from SQLfast website;
   return;
endif;


openDB $db$;

-- H_EMPLOYEE must be non-empty and valid-time.
extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' = 'int');
      showMessage WARNING:
                  @n@S3TDB.db is not a valid time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;


extract Nemp = select count(distinct CODE) from H_EMPLOYEE;
extract maxDate = select addToDate(max(end),100) from H_EMPLOYEE where end < '$Tfuture$';

write-b;
select thousandSep($Nemp$,',') as "EMPLOYEE",
       thousandSep(count(*),',') as "H_EMPLOYEE"
from H_EMPLOYEE;

closeDB;


-- ========================== 1. STATISTICS PER PROJECT ============================ --

openDB $db$;


write-b -- ====================================================================== --;
write   -- Compute the evolution of statistics of EMPLOYEEs per PROJECT           --;
write   -- Procedural techniques                                                  --;
write   -- ====================================================================== --;

set S1size,S2size,S3size = ,,;


write-b -- ---------------------------------------------------------------------- --;
write   -- S4: Create VALUED_STABLE_INTERVAL: stable intervals valued with SALARY --;
write   -- ---------------------------------------------------------------------- --;

start-timer;
function status = LTemp:valuedInterval {select PROJECT,SALARY
                                        from   H_EMPLOYEE
                                        group by PROJECT
                                        into VALUED_STABLE_INTERVAL};
set S4 = $timer$;
extract S4size = select thousandSep(count(*),',') from VALUED_STABLE_INTERVAL;
select * from VALUED_STABLE_INTERVAL order by PROJECT,start limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- S5: Create STATISTICS: (min,max,avg) in each stable interval         --;
write   -- -------------------------------------------------------------------- --;

start-timer;
--create index XS on VALUED_STABLE_INTERVAL(PROJECT,Start);
write-ba -- Index creation: $timer$;

start-timer;
create temp table STATISTICS as
  select  PROJECT,
          --count(*) as Count,
          min(SALARY) as Min,
          max(SALARY) as Max,
          round(avg(SALARY),1) as Average,
          Start,End
  from    VALUED_STABLE_INTERVAL
  group by PROJECT,Start;

set S5 = $timer$;
extract S5size = select thousandSep(count(*),',') from STATISTICS;
select * from STATISTICS order by PROJECT,Start limit $limit$;


write-b -- ----------------------------------------------------------- --;
write   -- S6: Reduce value-equivalent intervals of STATISTICS         --;
write   -- ----------------------------------------------------------- --;

start-timer;
function LTemp:reduce {select PROJECT,Min,Max,Average from STATISTICS};

set S6 = $timer$;
extract S6size = select thousandSep(count(*),',') from STATISTICS;


write-b -- ----------------------------------------------------------- --;
write   -- Evolution of SALARY STATISTICS per PROJECT                  --;
write-a -- ----------------------------------------------------------- --;

select * from STATISTICS order by PROJECT,Start limit $limit$;


compute Tot = $S4$ + $S5$ + $S6$;


write;
write-b -- -------------------------------------------------------------------- --;
write   -- S7: Create SALARY_COST, evolution of SALARY cost per project         --;
write-a -- -------------------------------------------------------------------- --;

-- For 1,000 employees: 25 s. with UDF "durationDays" and 2.7 s. with native "julianday" --


start-timer;
create temp table SALARY_COST as
   select  PROJECT,
           round(sum(SALARY/30.42 * (julianday(min(End,'$maxDate$')) - julianday(Start))),1) as Cost,
           Start,End
   from    VALUED_STABLE_INTERVAL
   group by PROJECT,Start;

set S7 = $timer$;
extract S7size = select thousandSep(count(*),',') from SALARY_COST;
write-b -- Computation through "julianday" function  --;
select * from SALARY_COST order by PROJECT,Start limit $limit$;


closeDB;


-- ========================== 2. STATISTICS PER PROJECT,CITY ============================ --

openDB $db$;


write-b -- ====================================================================== --;
write   -- Compute the evolution of statistics of EMPLOYEEs per PROJECT,CITY      --;
write   -- Procedural techniques                                                  --;
write   -- ====================================================================== --;

set S11size,S12size,S13size = ,,;


write-b -- ----------------------------------------------------------------------- --;
write   -- S14: Create VALUED_STABLE_INTERVAL: stable intervals valued with SALARY --;
write   -- ----------------------------------------------------------------------- --;

start-timer;
function status = LTemp:valuedInterval {select PROJECT,CITY,SALARY
                                        from   H_EMPLOYEE
                                        group by PROJECT,CITY
                                        into VALUED_STABLE_INTERVAL};
set S14 = $timer$;
extract S14size = select thousandSep(count(*),',') from VALUED_STABLE_INTERVAL;
select * from VALUED_STABLE_INTERVAL order by PROJECT,CITY,start limit $limit$;


write-b -- -------------------------------------------------------------------- --;
write   -- S15: Create STATISTICS: (min,max,avg) in each stable interval        --;
write   -- -------------------------------------------------------------------- --;

-- STATISTICS = xxx instead of xxx without index (2000 employees) --
-- STATISTICS = 16.6 instead of 23.8 for(PROJECT,CITY,Start,End) (2000 employees) --
create index XSC on VALUED_STABLE_INTERVAL(PROJECT,CITY,Start);

start-timer;
create temp table STATISTICS as
  select  PROJECT,CITY,
          --count(*) as Count,
          min(SALARY) as Min,
          max(SALARY) as Max,
          round(avg(SALARY),1) as Average,
          Start,End
  from    VALUED_STABLE_INTERVAL
  group by PROJECT,CITY,Start;

set S15 = $timer$;
extract S15size = select thousandSep(count(*),',') from STATISTICS;
select * from STATISTICS order by PROJECT,CITY,Start limit $limit$;


write-b -- ----------------------------------------------------------- --;
write   -- S16: Reduce value-equivalent intervals of STATISTICS        --;
write   -- ----------------------------------------------------------- --;

start-timer;
function LTemp:reduce {select PROJECT,CITY,Min,Max,Average from STATISTICS};

set S16 = $timer$;
extract S16size = select thousandSep(count(*),',') from STATISTICS;


write-b -- ----------------------------------------------------------- --;
write   -- Evolution of SALARY STATISTICS per PROJECT,CITY             --;
write-a -- ----------------------------------------------------------- --;

select * from STATISTICS order by PROJECT,CITY,Start limit $limit$;


compute Tot1 = $S14$ + $S15$ + $S16$;


write;
write-b -- ---------------------------------------------------------------------- --;
write   -- S17: Create SALARY_COST, evolution of SALARY cost per project,city     --;
write-a -- ---------------------------------------------------------------------- --;

-- For 1,000 employees: 25 s. with UDF "durationDays" and 2.7 s. with native "julianday" --


start-timer;
create temp table SALARY_COST as
   select  PROJECT,CITY,
           round(sum(SALARY/30.42 * (julianday(min(End,'$maxDate$')) - julianday(Start))),1) as Cost,
           Start,End
   from    VALUED_STABLE_INTERVAL
   group by PROJECT,CITY,Start;

set S17 = $timer$;
extract S17size = select thousandSep(count(*),',') from SALARY_COST;
write-b -- Computation through "julianday" function  --;
select * from SALARY_COST order by PROJECT,CITY,Start limit $limit$;


closeDB;


-- =================================== PERFORMANCE SYNTHESIS ====================================== --

write;
write-ab PERFORMANCE SYNTHESIS;

createDB InMemory;

write -- Statistics per PROJECT - Procedural technique  --;
select thousandSep($Nemp$,',') as "#employees", 'S4' as Query, 'VALUED_STABLE_INTERVAL' as Operation, '$S4$' as Time, '$S4size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S5' as Query, 'STATISTICS' as Operation, '$S5$' as Time, '$S5size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S6' as Query, 'Reduce' as Operation, '$S6$' as Time, '$S6size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S7' as Query, 'TOT COST' as Operation, '$S7$' as Time, '$S7size$' as Rows
order by Query;

select $Tot$ as "Total_Time (S4-S6)";

write-b;

write -- Statistics per PROJECT,CITY - Procedural technique  --;
select thousandSep($Nemp$,',') as "#employees", 'S14' as Query, 'VALUED_STABLE_INTERVAL' as Operation, '$S14$' as Time, '$S14size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S15' as Query, 'STATISTICS' as Operation, '$S15$' as Time, '$S15size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S16' as Query, 'Reduce' as Operation, '$S16$' as Time, '$S16size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S17' as Query, 'TOT COST' as Operation, '$S17$' as Time, '$S17size$' as Rows
order by Query;

select $Tot1$ as "Total_Time (S14-S16)";

closeDB;


write-b End of script;

return;

