
set Tfuture = 9999-12-31;
set limit = 10;

write-b -- ========================================================== --;
write   -- QUERYING CURRENT STATES                                    --;
write   -- Applicable to valid time databases only                    --;
write-a -- ========================================================== --;

-- Select a database --
set db = TDB;
ask-u db = [/bSelect a database] 
            Database:[!(TDB,TDB-100,TDB-200,TDB-500,TDB-1000,TDB-2000,TDB-5000,TDB-10000,TDB-20000,TDB-50000,TDB-100000,TDB-200000)];
if ('$DIALOGbutton$' = 'Cancel') return;

set db = $db$.db;

if (not fileExists('$dbDirectory$/$db$'));
   showMessage Database "$db$" not found.
               @nPlease load the experimental temporal databases from SQLfast website;
   return;
endif;

openDB $db$;

-- H_EMPLOYEE must be non-empty and valid-time.
extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' = 'int');
      showMessage WARNING:
                  @n@S3TDB.db is not a valid time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;

extract Nemp = select count(distinct CODE) from H_EMPLOYEE;
extract maxDate = select addToDate(max(end),100) from H_EMPLOYEE where end < '$Tfuture$';

write-b -- Size of the working set --;
select thousandSep($Nemp$,',') as "EMPLOYEE",
       thousandSep(count(*),',') as "H_EMPLOYEE"
from H_EMPLOYEE;

write-b -- -------------------------------------------------------------------- --;
write   -- Q1: What is the current state of employee D10626?                    --;
write   -- -------------------------------------------------------------------- --;
start-timer;
create temp table RES as
   select CODE,NAME,SALARY,CITY,PROJECT
   from   EMPLOYEE
   where  CODE = 'D10626';
set Q1 = $timer$;
extract Q1size = select thousandSep(count(*),',');
select * from RES limit $limit$;
drop table RES;

write-b -- -------------------------------------------------------------------- --;
write   -- Q3: Show the employees(s) with the highest salary                    --;
write   -- -------------------------------------------------------------------- --;
start-timer;
create temp table RES as
   select distinct CODE,NAME,SALARY,CITY,PROJECT
   from   EMPLOYEE
   where  SALARY = (select max(SALARY) from EMPLOYEE);
set Q3 = $timer$;
extract Q3size = select thousandSep(count(*),',');
select * from RES limit $limit$;
drop table RES;

write;
write-ab PERFORMANCE SYNTHESIS;

select thousandSep($Nemp$,',') as "#employees",'Q1' as Query,$Q1$ as Time,$Q1size$ as Rows
  union
select thousandSep($Nemp$,',') as "#employees",'Q3' as Query,$Q3$ as Time,$Q3size$ as Rows
order by Query;

closeDB;

write-b End of script;

return;
