
set Tfuture = 9999-12-31;
set tooSlow4 = 1000;
set tooSlow6 = 500;
set tooSlow7 = 1000;


write-b -- ========================================================== --;
write   -- TEMPORAL PROJECTION                                        --;
write   -- Applicable to valid time databases                         --;
write-a -- ========================================================== --;


-- Select a database and indexes --
set db = TDB;
set style = 1;
set index1,index2 = 0,0;
set limit = 10;

set askDb = ask-u db = [/bSelect a database] 
            Database:[(TDB,TDB-100,TDB-200,TDB-500,TDB-1000,TDB-2000,TDB-5000,TDB-10000,TDB-20000,TDB-50000,TDB-100000,TDB-200000)];
set technique = selectOne-u style = [/bSelect a computing technique] Predicative||Procedural;
set askIndex = selectMany index1,index2 = [/bIf predicative, which index create?] on EVENT|on STABLE_INTERVAL;
set askLimit = ask-u limit = [/bMax number of rows displayed] Limit;

askCombo [$askDb$]|[$askLimit$];
if ('$DIALOGbutton$' = 'Cancel') return;

set db = $db$.db;

if (not fileExists('$dbDirectory$/$db$'));
   showMessage Database not found.;
   return;
endif;


openDB $db$;

-- H_EMPLOYEE must be non-empty and valid-time.
extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' = 'int');
      showMessage WARNING:
                  @n@S3TDB.db is not a valid time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;


extract Nemp = select count(distinct CODE) from H_EMPLOYEE;
select thousandSep($Nemp$,',') as "EMPLOYEE",
       thousandSep(count(*),',') as "H_EMPLOYEE" from H_EMPLOYEE;

closeDB;

write;
write-b -- ======================================================================= --;
write   --   ENTITY-BASED TEMPORAL PROJECTION: H_EMPLOYEE[CODE,CITY]               --;
write   -- ======================================================================= --;

write-b -- ----------------------------------------------------------------------- --;
write   -- P1: (general) Procedural temporal projection (function "LTemp:project") --;
write-a -- ----------------------------------------------------------------------- --;

openDB $db$;

start-timer;
function status = LTemp:project {select CODE,CITY from H_EMPLOYEE
                                 into H_CODE_CITY};
set P1 = $timer$;
extract P1size = select thousandSep(count(*),',') from H_CODE_CITY;
if ($limit$ > 0);
   write;
   select thousandSep(count(*),',') as "H_CODE_CITY" from H_CODE_CITY;
   select * from H_CODE_CITY order by CODE,start limit $limit$;
endif;

closeDB;


write;
write-b -- ----------------------------------------------------------------------- --;
write   -- P2: Predicative temporal projection                                     --;
write-a -- ----------------------------------------------------------------------- --;

openDB $db$;

start-timer;
create temp table H_CODE_CITY as
  select P1.CODE, P1.CITY, P1.start, Pn.end
  from   H_EMPLOYEE P1, H_EMPLOYEE Pn
  where  (P1.CODE,P1.CITY)=(Pn.CODE,Pn.CITY)
  and    P1.start <= Pn.start
  and not exists (select * from H_EMPLOYEE P0
                  where  P0.CODE=P1.CODE and P0.CITY=P1.CITY
                  and    P0.end = P1.start)
  and not exists (select * from   H_EMPLOYEE Pm
                  where  Pm.CODE=P1.CODE and Pm.CITY=P1.CITY
                  and    Pn.end = Pm.start)
  and not exists (select * from   H_EMPLOYEE Pi
                  where  Pi.CODE=P1.CODE and Pi.CITY<>P1.CITY
                  and    P1.end <= Pi.start
                  and    Pi.end <= Pn.start);
set P2 = $timer$;
extract P2size = select thousandSep(count(*),',') from H_CODE_CITY;
if ($limit$ > 0);
   write;
   select thousandSep(count(*),',') as "H_CODE_CITY" from H_CODE_CITY;
   select * from H_CODE_CITY order by CODE,start limit $limit$;
endif;

closeDB;


write;
write-b -- ======================================================================= --;
write   --   GENERAL TEMPORAL PROJECTION: H_EMPLOYEE[CITY,PROJECT]                 --;
write   -- ======================================================================= --;

write-b -- ----------------------------------------------------------------------- --;
write   -- P3: General procedural temporal projection (function "LTemp:project")   --;
write-a -- ----------------------------------------------------------------------- --;

openDB $db$;

start-timer;
function status = LTemp:project {select CITY,PROJECT from H_EMPLOYEE
                                 into   H_CITY_PROJECT};
set P3 = $timer$;
extract P3size = select count(*) from H_CITY_PROJECT;
if ($limit$ > 0);
   write;
   select thousandSep(count(*),',') as "H_CITY_PROJECT" from H_CITY_PROJECT;
   select * from H_CITY_PROJECT order by CITY,start limit $limit$;
endif;

closeDB;


write-b -- ----------------------------------------------------------------------- --;
write   -- P4: General predicative temporal projection (be patient!)               --;
write-a -- ----------------------------------------------------------------------- --;

if ($Nemp$ > $tooSlow4$);
   write-ab This technique is not appropriate for samples of more than $tooSlow4$ employees;
   set P4 = ;
   set P4size = ;
   goto NEXT1;
endif;

openDB $db$;

start-timer;

create temp table H_CITY_PROJECT as
  select distinct E1.CITY, E1.PROJECT, E1.start, En.end
  from   H_EMPLOYEE E1, H_EMPLOYEE En
  where  E1.CITY=En.CITY and E1.PROJECT=En.PROJECT
  and    E1.start <= En.start and E1.end <= En.end
  and not exists (select *
                  from   H_EMPLOYEE E0
                  where  E0.CITY=E1.CITY and E0.PROJECT=E1.PROJECT
                  and    E0.start < E1.start
                  and    E0.end  >= E1.start)
  and not exists (select *
                  from   H_EMPLOYEE Em
                  where  Em.CITY=E1.CITY and Em.PROJECT=E1.PROJECT
                  and    Em.start <= En.end
                  and    Em.end   >  En.end)
  and not exists (select *
                  from   H_EMPLOYEE Eg
                  where  Eg.CITY=E1.CITY and Eg.PROJECT=E1.PROJECT
                  and    E1.end <= Eg.end
                  and    Eg.end <  En.start
                 and    not exists(select *
                                    from   H_EMPLOYEE Ek
                                    where  Ek.CITY=E1.CITY and Ek.PROJECT=E1.PROJECT
                                    and    Eg.end   <  Ek.end
                                    and    Ek.start <= Eg.end));

set P4 = $timer$;
extract P4size = select count(*) from H_CITY_PROJECT;
if ($limit$ > 0);
   write;
   select thousandSep(count(*),',') as "H_CITY_PROJECT" from H_CITY_PROJECT;
   select * from H_CITY_PROJECT order by CITY,start limit $limit$;
endif;

closeDB;

label NEXT1;


write;
write-b PERFORMANCE SYNTHESIS;

createDB InMemory;

write-b -- H_EMPLOYEE[CODE,CITY] --;
select thousandSep($Nemp$,',') as "#employees", 'P1' as Query, 'procedural' as Technique,          '$P1$' as Time, '$P1size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'P2' as Query, 'predicative(entity) ' as Technique, '$P2$' as Time, '$P2size$' as Rows
order by Query;

write-b -- H_EMPLOYEE[CITY,PROJECT] --;
select thousandSep($Nemp$,',') as "#employees", 'P3' as Query, 'procedural' as Technique,          '$P3$' as Time, '$P3size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'P4' as Query, 'predicative(general)' as Technique, '$P4$' as Time, '$P4size$' as Rows
order by Query;


closeDB;

write-b End of script;

return;
