-- -------------------------------------------------------------- --
-- GENERATION OF SQL-DDL CODE FROM THE CONTENTS OF THE DICTIONARY --
-- -------------------------------------------------------------- --

-- J-L Hainaut, February 12, 2016

-- No "match" clause (easy to add)
-- No constraint names (not preserved in system tables; could be extracted from the DDL code)
-- No check constraint
-- No index (easy to add)
-- No trigger

-- Select and open a database (if needed) --
-- -------------------------------------- --

   select * from COMPOSITION;

-- Save the name of the currently opened DB
set currentDB = $dbNameFull$;
write '$nomBD$', '$currentDB$';

if ('$currentDB$' <> '');
   write 1;
   -- a DB is already opened
   if ('$currentDB$' = '$nomBD$');
      write 2;
      -- ... and this is just the one we have to open! Do nothing.
      continue;
   else;
      write 3;
      -- these DB are different: cancel generation.
      showMessage Sorry, another DB is already opened. @nGeneration cancelled.;
      return;
   endif;
   write 4;
else;
   write 5;
   -- No DB opened so far
   if ('$nomBD$' = '');
      write 6;
      -- No DB suggested. Ask user to select one.
      selectFile nomBD = $dbDirectory$,Base de donnes,db;
      if ('$DIALOGbutton$' = 'Cancel' or '$nomBD$' = '') return;
   else;
      write 7;
      -- A DB is suggested
      continue;
   endif;
   -- Finally, open this database 
   write 8
   onError continue;
   openDB $nomBD$;
   write $SQLdiag$, $EXTENDEDdiag$;
   onError stop;
   write 9;
endif;

select * from PRODUIT;

write 10;
-- onError continue;
write 11;
createDictionary;
write 12;
select * from SYS_TABLE;
write $Error$, $SQLdiag$, $EXTENDEDdiag$;
write 13;
execSQL UTIL-SELECT-parameters-for-Lists.sql;

pause 14;
outputOpen ddl.var;
select
-- TABLE NAME
       '@ncreate '||case when T.TableType = 'base' then 'table ' else 'view ' end||T.TableName||'('||
--
-- COLUMNS
       (select group_concat(
               '@n       '||ColName||' '
               ||case C.ColType
                 when 'char'    then 'char('||cast(ColLen1 as char)||')'
                 when 'varchar' then 'varchar('||cast(ColLen1 as char)||')'
                 when 'decimal' then 'decimal('||cast(ColLen1 as char)
                                      ||','||cast(ColLen2 as char)||')'
                 else C.ColType
                 end
               ||case C.ColNull
                 when 'N' then ' not null'
                 else ''
                 end
               ||case
                 when C.DefVal is not null
                      and C.ColType in ('text','char','varchar')
                      then ' default '||''''||C.DefVal||''''
                 when C.DefVal is not null
                      and C.ColType not in ('text','char','varchar') 
                      then ' default '||C.DefVal
                 else ''
                 end
               ,',')
        from   SYS_COLUMN C
        where  TableID = T.TableID)||
--
-- PRIMARY KEY (if any)
        case 
        when T.TableID in (select TableID
                           from SYS_KEY
                           where KeyType = 'PK')
        then ',@n       primary key ('||(select group_concat(comp,',')
                                 from  (select ColName as comp
                                        from  SYS_KEY K,SYS_KEY_COMP KC
                                        where K.TableID = T.TableID
                                        and   K.KeyType = 'PK'
                                        and   KC.KeyID = K.keyID
                                        order by CompSeq))||')'
        else ''
        end||
--
-- UNIQUE KEYS (if any)
        case
        when T.TableID in (select TableID
                           from SYS_KEY
                           where KeyType = 'UN')
        then ',@n       '|| (select group_concat('unique('
                                       ||(select group_concat(comp,',')
                                          from  (select KC.ColName as comp
                                                 from   SYS_KEY_COMP KC
                                                 where  KC.KeyID = K.KeyID
                                                 order  by KC.CompSeq) )
                                       ||')',', ')
                     from  SYS_KEY K
                     where K.KeyType = 'UN'
                     and   K.TableID = T.TableID)
        else ''
        end||
--
-- FOREIGN KEYS (if any)
        case
        when T.TableID in (select TableID
                           from SYS_KEY
                           where KeyType = 'FK')
        then ','|| (select group_concat('@n       foreign key('
                                       ||(select group_concat(comp,',')
                                          from  (select KC.ColName as comp
                                                 from   SYS_KEY_COMP KC
                                                 where  KC.KeyID = K.KeyID
                                                 order  by KC.CompSeq) )
                                       ||')'
                                       ||' references '
                                       ||(select T1.TableName
                                          from   SYS_TABLE T1
                                          where  T1.TableID = K.TableTarget)
                                       ||case
                                         when K.KeyTarget in (select KeyID
                                                              from   SYS_KEY
                                                              where  KeyType = 'UN')
                                         then '('
                                            ||(select group_concat(comp,',')
                                               from  (select KC.ColName as comp
                                                      from   SYS_KEY_COMP KC
                                                      where  KC.KeyID = K.KeyTarget
                                                      order by KC.CompSeq) )
                                            ||')'
                                         else ''
                                         end
                                       ||case
                                         when onDelete = 'NO ACTION'
                                         then ''
                                         else '@n           on delete '||onDelete
                                         end 
                                       ||case
                                         when onUpdate = 'NO ACTION'
                                         then ''
                                         else '@n           on update '||onUpdate
                                         end 
                                       ,', ')
                     from  SYS_KEY K
                     where K.KeyType = 'FK'
                     and   K.TableID = T.TableID)
        else ''
        end||
--
-- END OF TABLE DEFINITION
       ');' as "create table"
from   SYS_TABLE T;
--where  T.TableType in 'base';
pause 15;
write $ddl$;
replace nomBD = .db,.ddl;

outputOpen $nomBD$;

write $ddl$;

outputAppend window;


if ('$currentDB$' = '')
   closeDB;

execSQL UTIL-SELECT-parameters-standard-1.sql;
