write-b -- ------------------------------ --;
write   -- Compute the powerset of a set  --;
write-b -- ------------------------------ --;

createDB InMemory;
create table NAMES(Name varchar(32));
insert into NAMES values ('Peter'),('John'),('Mary'),
                         ('Ann'),('Paul'),('Wim'),
                         ('Tony'),('Bob'),('Suzan')
                         ;

write-b Source set:;
select * from NAMES;

-- ------------------------------------------------------------------------------------- --
-- FIRST TECHNIQUE                                                                       --
-- CTE "POSET" creates all the permutations of all the non empty subsets of NAMES.       --
-- The permutations with the same contents are reduced in the main query by sorting them --
-- then eliminating duplicate sorted permutations:                                       --
--    "select distinct itemSortS(Member,0,0)"                                            --
-- ------------------------------------------------------------------------------------- --

write-b Be patient. The operations may take "some time"!

create temp table MEASURE1(Val varchar(4096));
create temp table MEASURE2(Val varchar(4096));

start-timer;
with recursive
  POSET(Member) as
  (select Name from NAMES
   union all
   select P.Member||';'||N.Name
   from   POSET P, NAMES N
   --where  not itemIncludeS(P.Member,N.Name)
   where  P.Member not like '%'||N.Name||'%'
  )
insert into MEASURE1(Val) select distinct itemSortS(Member,0,0) as Mem from POSET;
write-b Execution time of technique 1: $timer$;


-- ----------------------------------------------------------------------------- --
-- SECOND TECHNIQUE                                                              --
-- CTE "POSET" creates all unique, sorted, permutation.                          --
-- ----------------------------------------------------------------------------- --

start-timer;
with recursive
  POSET(Member) as
  (select Name from NAMES
   union all
   select P.Member||';'||N.Name
   from   POSET P, NAMES N
   where  N.Name > itemS(P.Member,-1)
  )
insert into MEASURE2(Val) select Member from POSET;
write-b Execution time of technique 2: $timer$;

write-ba Now, counting the rows in each CTE:;

with recursive
  POSET(Member) as
  (select Name from NAMES
   union all
   select P.Member||';'||N.Name
   from   POSET P, NAMES N
   --where  not itemIncludeS(P.Member,N.Name)
   where  P.Member not like '%'||N.Name||'%'
  )
select 'Technique 1',count(*) from POSET; 

write;

with recursive
  POSET(Member) as
  (select Name from NAMES
   union all
   select P.Member||';'||N.Name
   from   POSET P, NAMES N
   where  N.Name > itemS(P.Member,-1)
  )
select 'Technique 2',count(*) from POSET;

