write-b -- ------------------------------ --;
write   -- Compute the power set of a set --;
write   -- ------------------------------ --;

write-b -- The source set is a table of names (NAMES(Name)) --;
write-a -- The subsets are represented by name lists        --;

createDB InMemory;
create table NAMES(Name varchar(32));
insert into NAMES values ('Peter'),('John'),('Mary'),('Ann');

select * from NAMES;

write-b -- ------------------------------------------------------------------------------------- --;
write   -- FIRST TECHNIQUE                                                                       --;
write   -- CTE "POSET" creates all the permutations of all the non empty subsets of NAMES.       --;
write   -- The permutations with the same contents are reduced in the main query by sorting them --;
write   -- then eliminating duplicate sorted permutations:                                       --;
write   --    "select distinct itemSortS(Member,0,0)"                                            --;
write-a -- ------------------------------------------------------------------------------------- --;

create temp view POWERSET1(NameList) as
with recursive
  POSET(Member) as
  (select Name from NAMES
   union all
   select P.Member||';'||N.Name
   from   POSET P, NAMES N
   --where  not itemIncludeS(P.Member,N.Name)
   where  P.Member not like '%'||N.Name||'%'
  )
select distinct itemSortS(Member,0,0) as Mem from POSET;

select NameList from POWERSET1 order by itemLenS(NameList),NameList;

write-b -- ----------------------------------------------------------------------------- --;
write   -- SECOND TECHNIQUE                                                              --;
write   -- CTE "POSET" creates all unique, sorted, permutation.                          --;
write-a -- ----------------------------------------------------------------------------- --;

create temp view POWERSET2(NameList) as
with recursive
  POSET(Member) as
  (select Name from NAMES
   union all
   select P.Member||';'||N.Name
   from   POSET P, NAMES N
   where  N.Name > itemS(P.Member,-1)
  )
select Member from POSET;

select * from POWERSET2;

-- --------------------------------------- --
-- Storing the powerset members in a table --
-- --------------------------------------- --

create temp table POWERSET(SetID integer, Member varchar(32));
set setId = 1;
for subset = [select NameList from POWERSET2];
   compute subsetLen = itemLenS('$subset$');
   for idx = [1,$subsetLen$];
      compute name = itemS('$subset$',$idx$);
      insert into POWERSET values ($setId$,'$name$');
   endfor;
   compute setId = $setId$ + 1;
endfor;

select * from POWERSET order by SetID,Member;

-- just to check ... --

select SetID, group_concat(Member,';') as Member from POWERSET group by SetId;

