
write-b -- Building the strict order relation on CustID values in the set of customers --;
write -- --------------------------------------------------------------------------- --;

openDB ORDERS.db;

select prec.CustID AS "From", next.CustID as "To"
from   CUSTOMER prec, CUSTOMER next
where  next.CustID = (select min(CustID)
                      from CUSTOMER
                      where CustID > prec.Custid)
order by prec.CustID;

select prec.CustID AS "From", next.CustID as "To"
from   CUSTOMER prec, CUSTOMER next
where  prec.CustID < next.CustID
and    not exists (select *
                   from   CUSTOMER
                   where  CustID > prec.Custid
                   and    CustID < next.CustID)
order by prec.CustID;

write-b -- Display the last 3 customers ordered by CustID values --;
write   -- ----------------------------------------------------- --;

write-b -- Mathematical definition --;
select CustID,Name
from   CUSTOMER prec
where  (select count(*)
        from   CUSTOMER
        where   CustID > prec.CustID) < 3
order by CustID;

write-b -- Descending ordering + limit 3 --;
select CustID,Name
from   CUSTOMER
order by CustID desc limit 3;

write-b -- Descending ordering + limit 3 + reordering --;
select  CustID,Name
from (select CustID,Name
      from   CUSTOMER
      order by CustID desc limit 3)
order by CustID;

closeDB;

write-b -- Table of observed temperatures                                              --;
write-a   -- --------------------------------------------------------------------------- --;

createDB InMemory;

create table TEMPERATURE(DateT date,ValueT integer);
insert into TEMPERATURE values
      ('2017-12-10',13),
      ('2017-12-11',12),
      ('2017-12-12',12),
      ('2017-12-13',10),
      ('2017-12-14',8),
      ('2017-12-15',9),
      ('2017-12-16',11),
      ('2017-12-17',13),
      ('2017-12-18',14),
      ('2017-12-19',15),
      ('2017-12-20',14),
      ('2017-12-21',13);

select * from TEMPERATURE;

write-ab -- Daily temperature differences --;

select today.DateT as "Date",
       frame(today.ValueT - yesterday.ValueT,5,'>',' ') as Difference
from   TEMPERATURE yesterday, TEMPERATURE today
where  today.DateT = (select min(DateT) from TEMPERATURE where DateT > yesterday.DateT)
order by today.DateT;

closeDB;
