-- Loading and extracting pictures in a database

-- Creating the database
-- ---------------------

createOrReplaceDB PhotoAlbum.db;

create table PHOTO(ID integer not null primary key,
                   FullName text,
                   Name     text,
                   Picture  blob);
commitDB;
closeDB;

-- Getting the list of photo names
-- -------------------------------
set Dir = $scriptDirectory$/SQLfast-Tutorials/Appendix-1/Planes;
deleteFile-e $Dir$/dir.txt;
fileNames-s $Dir$/dir.txt = $Dir$;

-- Loading photos in the database
-- ------------------------------

openDB PhotoAlbum.db;
start-timer;

set id = 1;
for name = [file $Dir$/dir.txt];
   set fullName = $Dir$/$name$;
   BfileToVariable varLOB = $fullName$;
   --write $FILEdiag$,$name$,$fullName$;
   if ('$FILEdiag$' = 'OK');
      insert into PHOTO(ID,FullName,Name) values ($id$,'$fullName$','$name$');
      insertLOB varLOB = Picture from PHOTO where ID = $id$;
   else;
      write Warning: File $name$ has not been found.;
      exit;
   endif;
   compute id = $id$ + 1;
endfor;
commitDB;
write -- Loading time: $timer$;

closeDB;

-- Extracting photos from the database
-- -----------------------------------

openDB PhotoAlbum.db;

select ID,FullName,Name,length(Picture) as "Length" from PHOTO;
select count(*) as Number, sum(length(Picture)) as "Total length" from PHOTO;

start-timer;
for name,photo = [select Name,Picture from PHOTO] 
   VariableToFile photo = $Dir$/X-$name$;
write -- Extraction time: $timer$;

showMessage Have a look at directory $Dir$!;

for name = [file $Dir$/dir.txt];
   write $Dir$/X-$name$;
   deleteFile $Dir$/X-$name$;
endfor;
 
closeDB;
