-- Simulating the "union all" operator --

createDB InMemory;

create table T1(A integer);
create table T2(A integer);
insert into T1 values (1),(1),(2),(3),(3),(4),(4),(6),(7);
insert into T2 values (1),(2),(2),(3),(3),(5),(5),(6),(8);

select group_concat(A) as "T1" from (select A from T1 order by A);
select group_concat(A) as "T2" from (select A from T2 order by A);


-- SIMULATING "UNION ALL" --
-- ---------------------- --
write-b -- T1 union all T2 --;
create table TU as
select A,sum(N) as N
from (select A,count(*) as N,'T1' from T1 group by A
      union
      select A,count(*) as N,'T2' from T2 group by A)
group by A;

with UNFOLD(A,N) as
   (select A,N from TU
      union
    select A,N-1 from UNFOLD where  N > 1)
select group_concat(A) as "T1 union all T2" 
from (select A from UNFOLD order by A);


-- SIMULATING "EXCEPT ALL" --
-- ----------------------- --
write-b -- T1 except all T2 --;

create table TE as
with V1(A,N) as (select A,count(*) from T1 group by A),
     V2(A,N) as (select A,count(*) from T2 group by A)
select V1.A as "A", coalesce(V1.N - V2.N,V1.N) as SN from
 (V1 left join V2 on V1.A = V2.A)
where SN > 0;

with UNFOLD(A,N) as
   (select A,SN from TE
      union 
    select A,N-1 from UNFOLD where  N > 1)
select group_concat(A) as "T1 except all T2"
from (select A from UNFOLD order by A);


-- SIMULATING "INTERSECT ALL" --
-- -------------------------- --
write-b -- T1 intersect all T2 --;

create table TI as
with V1(A,N) as (select A,count(*) from T1 group by A),
     V2(A,N) as (select A,count(*) from T2 group by A)
select V1.A as "A", coalesce(min(V1.N,V2.N),0) as SN from
 (V1 left join V2 on V1.A = V2.A)
where SN > 0;

with UNFOLD(A,N) as
   (select A,SN from TI
      union 
    select A,N-1 from UNFOLD where  N > 1)
select group_concat(A) as "T1 intersect all T2"
from (select A from UNFOLD order by A);

