-- ---------------------------------- --
-- Bulk loading                       --
-- ---------------------------------- --

-- create a large source file for CUSTOMER loading 

write-b SQL insert script will be stored in file "Files/CUSTOMER-load.sql".;

set N = 100000;
write-b Starting source file generation;

outputOpen $fileDirectory$/CUSTOMER-load.sql;

start-timer;
-- The most sophisticated (but much faster) --

openDB ORDERS.db;
  execSQL UTIL-SELECT-parameters-for-Lists.sql;
  set maxSelectWidth = 100;

  with recursive NUM(Num) as (select 1 union select Num+1 from NUM where Num < $N$)
  select 'insert into CUSTOMER
          values(''X'||cast(Num as char)||''',''DUPONT'',''Downing Street'',''London'',''B5'',100);'
  from NUM;

  execSQL UTIL-SELECT-parameters-standard-1.sql;
closeDB;

-- The simplest (but slowest) --
--for I = [1,$N$];
--  write insert into CUSTOMER values ('X$I$','DUPONT','Downing Street','London','B5',100);;
--endfor;

outputAppend window;

write $N$ "insert" statements generated in $timer$ s.;

--
write-b Now (re)select the loading script file.;

selectFile source = $fileDirectory$,Loading files,sql;
if ('$source$' = '') stop;

set from = 1;
set to = 1000000;
set step = 2000;
set com = 1;
set tra = 1;
askCombo [ask-u from,to,step = [/bLoading data from file $source$]
                               from row|to row|checkpoint step]
        |[selectMany com,tra = Commit|Tracing];
if ('$DIALOGbutton$' = 'Cancel') stop;

compute suffix = '-' || itemS(';c',$com$+1) || itemS(';t',$tra$+1); 
if ('$suffix$' = '-') set suffix = ;

write-ab Executing statement "fastExec$suffix$ $source$,$from$,$to$,$step$;";

openDB ORDERS.db;
  write Starting loading data;
  start-timer;

  fastExec$suffix$ $source$,$from$,$to$,$step$;

  write Loading completed in $timer$ s.;
  extract Nr = select count(1) from CUSTOMER;
  write-b Table CUSTOMER now contains $Nr$ rows;
closeDB;

showMessage Don't forget to restore database "ORDERS.db" from archive "SourceDB.zip";
