-- Coping with circuits --

write-b WARNING: The STAFF database must be "fresh" for this experiment.;
write-a It would be wise to recreate it through script "STAFF-create.sql"
      @nbefore creating circuits.;

openDB STAFF.db;

select * from PERSON;

ask P = [Enter a PERSON ID] PersonID:[!select PID from PERSON];

write-b The standard staff hierarchy of $P$;
with recursive ORG(Lev,PID,Path)
as (select 1,'$P$','.$P$.'
      union all
    select Lev+1,P.PID,Path||P.PID||'.'
    from   ORG O, PERSON P
    where  O.PID = P.Manager
    )
select * from ORG;

write-b  Then, we create a circuit p1 --> p7;
update PERSON set Manager = 'p07' where PID = 'p01';

-- Controlling infinite execution: first technique
write-b First protection: Arbitrarily limiting the number of executions;
with recursive ORG(Lev,PID,Path)
as (select 1,'$P$','.$P$.'
      union all
    select Lev+1,P.PID,Path||P.PID||'.'
    from   ORG O, PERSON P
    where  O.PID = P.Manager
    and    Lev < 10
    )
select * from ORG;

-- Controlling infinite execution: second technique
write-b Second protection: Limiting the number of executions to the size of the problem;
extract N = select count(*) from PERSON;
with recursive ORG(Lev,PID,Path)
as (select 1,'$P$','.$P$.'
      union all
    select Lev+1,P.PID,Path||P.PID||'.'
    from   ORG O, PERSON P
    where  O.PID = P.Manager
    and    Lev < $N$
    )
select * from ORG;

-- Controlling infinite execution: third technique
write-b Third protection: Preventing the query from visiting a person twice;
with recursive ORG(Lev,PID,Path)
as (select 1,'$P$','.$P$.'
      union all
    select Lev+1,P.PID,Path||P.PID||'.'
    from   ORG O, PERSON P
    where  O.PID = P.Manager
    and    Path not like '%.'||P.PID||'.%'
    )
select * from ORG;

-- We cancel the circuit. 
update PERSON set Manager = null where PID = 'p01';

commitDB;

closeDB;

-- Why not just abortDB to cancel the modifications? 
-- Because "with" seems to include an implicit "commit"

