
if (not fileExists('$dbDirectory$/TRACING.db'));
  showMessage @nThe tracing database "TRACING.db" is missing.@nPlease create it first.@n;
  return;
endif;

openDB TRACING.db;

-- Variables extracted from,
--   all traces:        "set varNature = 0,1,2;"
--   source traces:     "set varNature = 0,1;"
--   elementary traces: "set varNature = 0,2;"

set varNature = 0,1;

write-b -- Create and fill table MULTIVAR  --;
        -- ------------------------------- --
create temp table MULTIVAR (Variable varchar(512),AssignedIn integer);

-- Variables defined in "set" statements (single variables)
insert into MULTIVAR select head(tail(RawStat,''),'='),TraceID
                     from   TRACES
                     where  Tnature in ($varNature$)
                     and    lower(ltrim(RawStat)) like 'set%=%';

-- Variables defined in "compute" statements
insert into MULTIVAR select  head(tail(RawStat,''),'='),TraceID
                      from   TRACES
                      where  Tnature in ($varNature$)
                      and    lower(ltrim(RawStat)) like 'compute%=%';

-- Variables defined in "range-for", "item-for", "file-for"" statements
insert into MULTIVAR select  head(tail(RawStat,''),'='),TraceID
                      from   TRACES
                      where  Tnature in ($varNature$)
                      and    lower(ltrim(RawStat)) like 'for%=%[%]'
                      and    lower(ltrim(RawStat)) not like 'for%=%[%select%]';

-- Variables defined in "extract" statements
insert into MULTIVAR select  head(tail(RawStat,''),'='),TraceID
                      from   TRACES
                      where  Tnature in ($varNature$)
                      and    lower(ltrim(RawStat)) like 'extract%=%select%';

-- Variables defined in "SQL-for" statements
insert into MULTIVAR select  head(tail(RawStat,''),'='),TraceID
                      from   TRACES
                      where  Tnature in ($varNature$)
                      and    lower(ltrim(RawStat)) like 'for%=%[%select%]';

-- Variables defined in "ask" statements
insert into MULTIVAR select  head(tail(RawStat,''),'='),TraceID
                      from   TRACES
                      where  Tnature in ($varNature$)
                      and   (   (lower(ltrim(RawStat)) like 'ask%=%'
                                 and lower(ltrim(RawStat)) not like 'askcombo%[%')
                             or lower(ltrim(RawStat)) like 'selectOne%=%'
                             or lower(ltrim(RawStat)) like 'selectMany%=%');

select Variable,AssignedIn 
from   MULTIVAR V
order by AssignedIn,Variable;

write-b -- Fill table VARIABLES            --;
        -- ------------------------------- --
delete from VARIABLES;

with recursive VAR(Num,Var,AssignedIn )
as (select 1,item(Variable,1,','),AssignedIn from MULTIVAR
      union all
    select Num+1,item(Variable,Num+1,','),M.AssignedIn from VAR V,MULTIVAR M
    where  itemLen(Variable,',') >= Num+1
   )
insert into VARIABLES select distinct Var,AssignedIn  from VAR;

write-b -- First variant (sorted by trace,variable) --;
select AssignedIn,Variable from VARIABLES order by AssignedIn,Variable ;
write-b -- Second variant (sorted by variable,trace) --;
select * from VARIABLES order by Variable,AssignedIn;
write-b -- Third variant (variable + list of traces) --;
select Variable,group_concat(distinct AssignedIn ) as "AssignedIn" 
from   VARIABLES
group by Variable order by Variable;

closeDB;
